@@ -123,7 +123,7 @@ x86_vect::add(double out[], const double in[], size_t N) const {
 	    const vecd128* av = reinterpret_cast<const vecd128*>(in);
 	    vecd128* cv = reinterpret_cast<vecd128*>(out);
 	    for (size_t i=0; i<nLoop; ++i) {
-	       register vecd128 c = cv[i];
+	       vecd128 c = cv[i];
 	       c += av[i];
 	       cv[i] = c;
 	    }
@@ -160,7 +160,7 @@ x86_vect::add(double out[], const double in1[], const double in2[],
 	    const vecd128* av = reinterpret_cast<const vecd128*>(in1);
 	    const vecd128* bv = reinterpret_cast<const vecd128*>(in2);
 	    for (size_t i=0; i<nLoop; ++i) {
-	       register vecd128 tmp = av[i];
+	       vecd128 tmp = av[i];
 	       tmp += bv[i];
 	       asm("movdqu %1, %0"
 		   : "=m" (out[0])
@@ -218,7 +218,7 @@ x86_vect::add(double out[], const double in1[], const double in2[],
 	    const vecd128* av = reinterpret_cast<const vecd128*>(in1);
 	    const vecd128* bv = reinterpret_cast<const vecd128*>(in2);
 	    for (size_t i=0; i<nLoop; ++i) {
-	       register vecd128 tmp = av[i];
+	       vecd128 tmp = av[i];
 	       tmp += bv[i];
 	       sumu.v = tmp;
 	       *out++ = sumu.d[0];
@@ -250,9 +250,9 @@ x86_vect::add(double out[], double in, size_t N) const {
       //--------------------------------  Multiply 4 at a time
       if (N > 3) {
 	 vecd256* av  = reinterpret_cast<vecd256*>(out);
-	 register vecd256  bv = {in, in, in, in};
+	 vecd256  bv = {in, in, in, in};
 	 while (N > 3) {
-	    register vecd256 c = *av;
+	    vecd256 c = *av;
 	    c *= bv;
 	    N -= 4;
 	    *av++ = c;
@@ -299,8 +299,8 @@ x86_vect::cmodsq(double out[], const double in[], size_t n) const {
 	 const vecd128* av = reinterpret_cast<const vecd128*>(in);
 	 //  Multply two in a shot
 	 while (n > 1) {
-	    register vecd128 prod0 = *av++;
-	    register vecd128 prod1 = *av++;
+	    vecd128 prod0 = *av++;
+	    vecd128 prod1 = *av++;
 	    prod0 *= prod0;
 	    prod1 *= prod1;
 	    asm("haddpd %2, %1;" // does this really || add and pack.
@@ -347,7 +347,7 @@ x86_vect::cmodsq(double out[], const double in[], size_t n) const {
       if (align16_abs(in)) {
 	 const vecd128* av = reinterpret_cast<const vecd128*>(in);
 	 for (size_t i=0; i<n; i++) {
-	    register vecd128 prod = *av++;
+	    vecd128 prod = *av++;
 	    prod *= prod;
 	    asm("pshufd $78, %1, %%xmm1;"
 		//"addpd  %1, %%xmm1;"
@@ -365,7 +365,7 @@ x86_vect::cmodsq(double out[], const double in[], size_t n) const {
       //   imaginary and real ports of successive complex numbers are 
       //   calculated with a single 2-operand SSE2 vector.
       else {
-	 register vecd128 span;
+	 vecd128 span;
 	 asm("movsd %1, %0;"
 	     "mulsd %0, %0"
 	     : "=x" (span)
@@ -376,7 +376,7 @@ x86_vect::cmodsq(double out[], const double in[], size_t n) const {
 	 const vecd128* av = reinterpret_cast<const vecd128*>(++in);
 	 n--;
 	 for (size_t i=0; i<n; i++) {
-	    register vecd128 prod = *av++;
+	    vecd128 prod = *av++;
 	    prod *= prod;
 	    asm("addsd  %2, %1;"
 		"movsd  %1, %0;"
@@ -430,8 +430,8 @@ x86_vect::cvt(float out[], const int in[], size_t N) const {
 	 }
 
 	 //-----------------------------  Convert 4 at a shot
-	 register const veci128* av = reinterpret_cast<const veci128*>(in);
-	 register vecf128* bv = reinterpret_cast<vecf128*>(out);
+	 const veci128* av = reinterpret_cast<const veci128*>(in);
+	 vecf128* bv = reinterpret_cast<vecf128*>(out);
 	 for (size_t nquad=(N>>2); nquad; --nquad) {
 	    asm("cvtdq2ps %1, %%xmm0;"
 		"movdqa   %%xmm0, %0"
@@ -476,8 +476,8 @@ x86_vect::cvt(int out[], const float in[], size_t N) const {
 	 }
 
 	 //-----------------------------  Convert 4 at a shot
-	 register const vecf128* av = reinterpret_cast<const vecf128*>(in);
-	 register veci128* bv = reinterpret_cast<veci128*>(out);
+	 const vecf128* av = reinterpret_cast<const vecf128*>(in);
+	 veci128* bv = reinterpret_cast<veci128*>(out);
 	 for (size_t nquad=(N>>2); nquad; --nquad) {
 	    asm("cvtps2dq %1, %%xmm0;"
 		"movdqa   %%xmm0, %0"
@@ -526,7 +526,7 @@ void x86_vect::div(double out[], const double *in, size_t N) const {
 	    const vecd128* av = reinterpret_cast<const vecd128*>(in);
 	    vecd128* cv = reinterpret_cast<vecd128*>(out);
 	    for (size_t i=0; i<nLoop; ++i) {
-	       register vecd128 c = cv[i];
+	       vecd128 c = cv[i];
 	       c /= av[i];
 	       cv[i] = c;
 	    }
@@ -561,7 +561,7 @@ x86_vect::div(double out[], const double *in1, const double* in2,
 	    const vecd128* av = reinterpret_cast<const vecd128*>(in1);
 	    const vecd128* bv = reinterpret_cast<const vecd128*>(in2);
 	    for (size_t i=0; i<nLoop; ++i) {
-	       register vecd128 tmp = av[i];
+	       vecd128 tmp = av[i];
 	       tmp /= bv[i];
 	       asm("movdqu %1, %0"
 		   : "=m" (out[0])
@@ -619,7 +619,7 @@ x86_vect::div(double out[], const double *in1, const double* in2,
 	    const vecd128* av = reinterpret_cast<const vecd128*>(in1);
 	    const vecd128* bv = reinterpret_cast<const vecd128*>(in2);
 	    for (size_t i=0; i<nLoop; ++i) {
-	       register vecd128 tmp = av[i];
+	       vecd128 tmp = av[i];
 	       tmp /= bv[i];
 	       sumu.v = tmp;
 	       *out++ = sumu.d[0];
@@ -650,9 +650,9 @@ void x86_vect::div(double out[], double in, size_t N) const {
       //--------------------------------  Multiply 4 at a time
       if (N > 3) {
 	 vecd256* av  = reinterpret_cast<vecd256*>(out);
-	 register vecd256  bv = {in, in, in, in};
+	 vecd256  bv = {in, in, in, in};
 	 while (N > 3) {
-	    register vecd256 c = *av;
+	    vecd256 c = *av;
 	    c /= bv;
 	    N -= 4;
 	    *av++ = c;
@@ -715,7 +715,7 @@ x86_vect::dot(const double in1[], const double in2[], size_t N) const {
 	    size_t nLoop = N >> 1;
 	    const vecd128* av = reinterpret_cast<const vecd128*>(in1);
 	    const vecd128* bv = reinterpret_cast<const vecd128*>(in2);
-	    register vecd128 sumv = *av * *bv;
+	    vecd128 sumv = *av * *bv;
 	    for (size_t i=1; i<nLoop; ++i) sumv += av[i] * bv[i];
 	    //--------------------------  sse2 architecture.
 	    if (_sseID == sse2) {
@@ -754,8 +754,8 @@ x86_vect::dot(const double in1[], const double in2[], size_t N) const {
 	    bv = in1;
 	 }
 	 
-	 register vecd128 sumv = {0.0, 0.0};
-	 register vecd128 temp;
+	 vecd128 sumv = {0.0, 0.0};
+	 vecd128 temp;
 	 if (_sseID == sse2) {
 	    for (size_t i=0; i<nVec; i++) {
 	       asm("movupd %1, %0": "=x" (temp) : "m" (bv[0]));
@@ -823,7 +823,7 @@ void x86_vect::mul(double out[], const double *in, size_t N) const {
 	    const vecd128* av = reinterpret_cast<const vecd128*>(in);
 	    vecd128* cv = reinterpret_cast<vecd128*>(out);
 	    for (size_t i=0; i<nLoop; ++i) {
-	       register vecd128 c = cv[i];
+	       vecd128 c = cv[i];
 	       c *= av[i];
 	       cv[i] = c;
 	    }
@@ -837,36 +837,74 @@ void x86_vect::mul(double out[], const double *in, size_t N) const {
 }
 
 //======================================  Two-operand complex vector multiply
+inline void
+dcplx_mul(vecd128& out, const vecd128& in) {
+   double* b = reinterpret_cast<double*>(&out);
+   asm("pshufd $78, %3, %%xmm1;" // xmm1 = {ai, ar}
+       "movddup %1, %%xmm2;"     // xmm2 = {br, br}
+       "mulpd %%xmm2, %0;"       // mla  = {ar*br, ai*br}
+       "movddup %2, %%xmm2;"     // xmm2 = {bi, bi}
+       "mulpd %%xmm2, %%xmm1;"   // xmm1 = {ai*bi, ar*bi}
+       "addsubpd %%xmm1, %0;"    // mla  = {ar*br-ai*bi, ai*br+ar*bi}
+       : "=x" (out) 
+       : "m" (b[0]), "m" (b[1]), "x" (in)
+       : "xmm1", "xmm2"
+       );
+}
+
+#if 0
+//--------------------------------------  The following code is not verified
+inline void
+dcplx_mul2(vecd256& out, const vecd256& in) {
+   double* b = reinterpret_cast<double*>(&out);
+   asm("pshufd $78, %3, %%ymm1;" // ymm1 = {ai, ar}
+       "movddup %1, %%ymm2;"     // ymm2 = {br, br}
+       "mulpd %%ymm2, %0;"       // mla  = {ar*br, ai*br}
+       "movddup %2, %%ymm2;"     // ymm2 = {bi, bi}
+       "mulpd %%ymm2, %%ymm1;"   // ymm1 = {ai*bi, ar*bi}
+       "addsubpd %%ymm1, %0;"    // mla  = {ar*br-ai*bi, ai*br+ar*bi}
+       : "=x" (out)
+       : "m" (b[0]), "m" (b[1]), "y" (in)
+       : "ymm1", "ymm2"
+       );
+}
+#endif
+
 template <>
 void x86_vect::mul(dComplex out[], const dComplex in[], size_t N) const {
    switch (_sseID) {
    case avx:
+      if (align32_rel(in, out) && align16_abs(out)) {
+	 //cout << "avx aligned complex multiply, N= " << N << endl;
+	 const vecd128* av = reinterpret_cast<const vecd128*>(in);
+	 vecd128* bv = reinterpret_cast<vecd128*>(out);
+	 if (!align32_abs(in, out)) {
+	    dcplx_mul(*bv++, *av++);
+	    N--;
+	 }
+	 //  For the time being, this one at a time - should change to use avx
+	 while (N--) {
+	    dcplx_mul(*bv++, *av++);
+	 }
+	 return;
+      }
+  
    case ssse3:
    case sse4_1:
    case sse4_2:
       if (align16_abs(in, out)) {
-	 register const vecd128* av = reinterpret_cast<const vecd128*>(in);
-	 register       vecd128* bv = reinterpret_cast<vecd128*>(out);
-	 while (N--) {
-	    register vecd128 mla = *av++;
-	    double* b = reinterpret_cast<double*>(bv);
-	    asm("pshufd $78, %3, %%xmm1;" // xmm1 = {ai, ar}
-		"movddup %1, %%xmm2;"     // xmm2 = {br, br}
-		"mulpd %%xmm2, %0;"       // mla  = {ar*br, ai*br}
-		"movddup %2, %%xmm2;"     // xmm2 = {bi, bi}
-		"mulpd %%xmm2, %%xmm1;"   // xmm1 = {ai*bi, ar*bi}
-		"addsubpd %%xmm1, %0;"    // mla  = {ar*br-ai*bi, ai*br+ar*bi}
-		: "=x" (mla) 
-		: "m" (b[0]), "m" (b[1]), "x" (mla)
-		: "xmm1", "xmm2"
-		);
-	    *bv++ = mla;
+	 //cout << "sse3 aligned complex multiply, N= " << N << endl;
+	 const vecd128* av = reinterpret_cast<const vecd128*>(in);
+	 vecd128* bv = reinterpret_cast<vecd128*>(out);
+	 for (; N; N--) {
+	    dcplx_mul(*bv++, *av++);
 	 }
 	 return;
       }
 
    default:
       //--------------------------------  Use normal stuff
+      //cout << "unaligned complex multiply, N= " << N << endl;
       while (N--) {
 	 *out++ *= *in++;
       }
@@ -896,7 +934,7 @@ x86_vect::mul(double out[], const double in1[], const double in2[],
 	    const vecd128* av = reinterpret_cast<const vecd128*>(in1);
 	    const vecd128* bv = reinterpret_cast<const vecd128*>(in2);
 	    for (size_t i=0; i<nLoop; ++i) {
-	       register vecd128 tmp = av[i];
+	       vecd128 tmp = av[i];
 	       tmp *= bv[i];
 	       asm("movdqu %1, %0"
 		   : "=m" (out[0])
@@ -955,7 +993,7 @@ x86_vect::mul(double out[], const double in1[], const double in2[],
 	    const vecd128* av = reinterpret_cast<const vecd128*>(in1);
 	    const vecd128* bv = reinterpret_cast<const vecd128*>(in2);
 	    for (size_t i=0; i<nLoop; ++i) {
-	       register vecd128 tmp = av[i];
+	       vecd128 tmp = av[i];
 	       tmp *= bv[i];
 	       sumu.v = tmp;
 	       *out++ = sumu.d[0];
@@ -988,7 +1026,7 @@ x86_vect::mul(double out[], double in, size_t N) const {
       //-----------------------------  Multiply 4 at a time
       if (N > 3) {
 	 vecd256* av  = reinterpret_cast<vecd256*>(out);
-	 register vecd256  bv = {in, in, in, in};
+	 vecd256  bv = {in, in, in, in};
 	 while (N > 3) {
 	    *av++ *= bv;
 	    N -= 4;
@@ -1105,7 +1143,7 @@ void x86_vect::sub(double out[], const double in[], size_t N) const {
 	    const vecd128* av = reinterpret_cast<const vecd128*>(in);
 	    vecd128* cv = reinterpret_cast<vecd128*>(out);
 	    for (size_t i=0; i<nLoop; ++i) {
-	       register vecd128 c = cv[i];
+	       vecd128 c = cv[i];
 	       c -= av[i];
 	       cv[i] = c;
 	    }
@@ -1141,7 +1179,7 @@ x86_vect::sub(double out[], const double in1[], const double in2[],
 	    const vecd128* av = reinterpret_cast<const vecd128*>(in1);
 	    const vecd128* bv = reinterpret_cast<const vecd128*>(in2);
 	    for (size_t i=0; i<nLoop; ++i) {
-	       register vecd128 tmp = av[i];
+	       vecd128 tmp = av[i];
 	       tmp -= bv[i];
 	       asm("movdqu %1, %0"
 		   : "=m" (out[0])
@@ -1200,7 +1238,7 @@ x86_vect::sub(double out[], const double in1[], const double in2[],
 	    const vecd128* av = reinterpret_cast<const vecd128*>(in1);
 	    const vecd128* bv = reinterpret_cast<const vecd128*>(in2);
 	    for (size_t i=0; i<nLoop; ++i) {
-	       register vecd128 tmp = av[i];
+	       vecd128 tmp = av[i];
 	       tmp -= bv[i];
 	       sumu.v = tmp;
 	       *out++ = sumu.d[0];
@@ -1230,7 +1268,7 @@ x86_vect::sub(double out[], double in, size_t N) const {
 template <>
 double
 x86_vect::sum(const double in[], size_t n) const {
-   register math_type out = 0;
+   math_type out = 0;
    size_t nLeft  = n;
    switch (_sseID) {
 
@@ -1243,7 +1281,7 @@ x86_vect::sum(const double in[], size_t n) const {
 
       if (nLeft > 3) {
 	 const vecd256* av  = reinterpret_cast<const vecd256*>(in);
-	 register vecd256  sv = {0, 0, 0, 0};
+	 vecd256  sv = {0, 0, 0, 0};
 	 while (nLeft > 3) {
 	    sv += *av++;
 	    nLeft -= 4;
@@ -1284,7 +1322,7 @@ x86_vect::sum(const double in[], size_t n) const {
       if (nLeft>1) {
 	 size_t nLoop = nLeft >> 1;
 	 const vecd128* av  = reinterpret_cast<const vecd128*>(in);
-	 register vecd128  sv = {0, 0};
+	 vecd128  sv = {0, 0};
 	 while (nLoop--) sv += *av++;
 	 vd2map sumu;
 	 sumu.v  = sv;
