@@ -7,6 +7,7 @@
 #include "gds_fpclass.hh"
 #include "gds_veclib.hh"
 #include <algorithm>
+#include <stdexcept>
 
 using namespace std;
 
@@ -103,6 +104,12 @@ DVecType<dComplex>::getDataType(void) {
     return DVector::t_dcomplex;
 }
 
+template<>
+DVector::DVType
+DVecType<uint32_t>::getDataType(void) {
+    return DVector::t_uint;
+}
+
 //======================================  Test for finite data.
 template<>
 bool
@@ -236,6 +243,41 @@ DVecType<T>::force_copy(void) {
     mData.access();
 }
 
+//======================================  GetUInt() templates
+template<>
+uint32_t
+DVecType<fComplex>::getUInt(size_type inx) const
+{
+    float d = mData[inx].Real();
+    if (d < 0) throw std::runtime_error("Can't convert negative value to uint");
+    return uint32_t(d);
+}
+
+template<>
+uint32_t
+DVecType<dComplex>::getUInt(size_type inx) const
+{
+    double d = mData[inx].Real();
+    if (d < 0) throw std::runtime_error("Can't convert negative value to uint");
+    return uint32_t(d);
+}
+
+template<>
+uint32_t
+DVecType<uint32_t>::getUInt(size_type inx) const
+{
+    return mData[inx];
+}
+
+template<class T>
+uint32_t
+DVecType<T>::getUInt(size_type inx) const
+{
+    T d = mData[inx];
+    if (d < 0) throw std::runtime_error("Can't convert negative value to uint");
+    return uint32_t(d);
+}
+
 //======================================  GetData() templates
 template<>
 DVector::size_type
@@ -278,6 +320,26 @@ DVecType<T>::getData(size_type inx, size_type N, int* d) const
     return nw;
 }
 
+template<>
+DVector::size_type
+DVectU::getData(size_type inx, size_type N, uint32_t* d) const
+{
+    size_type nw = N;
+    check_substr(inx, nw, mData.size());
+    memcpy(d, refTData()+inx, nw*sizeof(uint32_t));
+    return nw;
+}
+
+template<class T>
+DVector::size_type
+DVecType<T>::getData(size_type inx, size_type N, uint32_t* d) const
+{
+    size_type nw = N;
+    check_substr(inx, nw, mData.size());
+    for (size_type i=0; i<nw; ++i) d[i] = getUInt(i+inx);
+    return nw;
+}
+
 template<>
 DVector::size_type
 DVectF::getData(size_type inx, size_type N, float* d) const
@@ -304,7 +366,7 @@ DVectC::getData(size_type inx, size_type N, float* d) const
 {
     size_type nw = N;
     check_substr(inx, nw, mData.size());
-    register const fComplex* rp = refTData() + inx;
+    const fComplex* rp = refTData() + inx;
     for (size_type i=0; i<nw; ++i) d[i] = (rp++)->Real();
     return nw;
 }
@@ -315,7 +377,7 @@ DVectW::getData(size_type inx, size_type N, float* d) const
 {
     size_type nw = N;
     check_substr(inx, nw, mData.size());
-    register const dComplex* rp = refTData() + inx;
+    const dComplex* rp = refTData() + inx;
     for (size_type i=0; i<nw; ++i) d[i] = (rp++)->Real();
     return nw;
 }
@@ -326,7 +388,7 @@ DVecType<T>::getData(size_type inx, size_type N, float* d) const
 {
     size_type nw = N;
     check_substr(inx, nw, mData.size());
-    register const T* rp = refTData() + inx;
+    const T* rp = refTData() + inx;
     for (size_type i=0; i<nw; ++i) d[i] = float(*rp++);
     return nw;
 }
@@ -337,7 +399,7 @@ DVectC::getData(size_type inx, size_type N, double* d) const
 {
     size_type nw = N;
     check_substr(inx, nw, mData.size());
-    register const fComplex* rp = refTData() + inx;
+    const fComplex* rp = refTData() + inx;
     for (size_type i=0; i<nw; ++i) d[i] = (rp++)->Real();
     return nw;
 }
@@ -348,7 +410,7 @@ DVectW::getData(size_type inx, size_type N, double* d) const
 {
     size_type nw = N;
     check_substr(inx, nw, mData.size());
-    register const dComplex* rp = refTData() + inx;
+    const dComplex* rp = refTData() + inx;
     for (size_type i=0; i<nw; ++i) d[i] = (rp++)->Real();
     return nw;
 }
@@ -370,7 +432,7 @@ DVecType<T>::getData(size_type inx, size_type N, double* d) const
 {
     size_type nw = N;
     check_substr(inx, nw, mData.size());
-    register const T* rp = refTData() + inx;
+    const T* rp = refTData() + inx;
     for (size_type i=0; i<nw; ++i) d[i] = double(*rp++);
     return nw;
 }
@@ -673,16 +735,16 @@ template<class T>
 DVecType<T>&
 DVecType<T>::reverse(size_type inx, const T* v, size_type N) {
     if (inx+N > mData.size()) mData.resize(inx+N);
-    register T* pOut = mData.ref() + inx;
+    T* pOut = mData.ref() + inx;
     if (v < pOut+N && pOut < v+N) {
         if (v != pOut) memmove(pOut, v, N);
-	for (register T* pEnd=pOut+N-1; pEnd>pOut; --pEnd) {
-	    register T t = *pEnd;
+	for (T* pEnd=pOut+N-1; pEnd>pOut; --pEnd) {
+	    T t = *pEnd;
 	    *pEnd   = *pOut;
 	    *pOut++ = t;
 	}
     } else {
-	for (register const T* pIn=v+N; pIn > v; ) *pOut++ = *(--pIn);
+	for (const T* pIn=v+N; pIn > v; ) *pOut++ = *(--pIn);
     }
     return *this;
 }
@@ -695,13 +757,13 @@ DVecType<T>::reverse(void) {
     if (!N) return *this;
     if (mData.shared()) {
 	CWVec<T> temp(N);
-	register const T* pIn  = const_cast<const DVecType&>(*this).refTData();
-	register       T* pOut = temp.ref() + N;
+	const T* pIn  = const_cast<const DVecType&>(*this).refTData();
+	      T* pOut = temp.ref() + N;
 	for (size_type i=0; i<N; ++i) *(--pOut) = *pIn++;
 	mData = temp;
     } else {
-	register T* pOut = mData.ref();
-	register T* pEnd = mData.ref() + N;
+	T* pOut = mData.ref();
+	T* pEnd = mData.ref() + N;
 	while (pOut < --pEnd) {
 	    T temp = *pEnd;
 	    *pEnd  = *pOut;
@@ -846,9 +908,9 @@ DVecType<T>::Erase(size_type inx, size_type len)
         mData.substr(0, inx);
     } else if (inx != in2) {
         curl -= in2;
-	register       T* po = refTData()+inx;
-	register const T* pi = refTData()+in2;
-	for (register size_type i=0; i<curl; ++i) *po++ = *pi++;
+	      T* po = refTData()+inx;
+	const T* pi = refTData()+in2;
+	for (size_type i=0; i<curl; ++i) *po++ = *pi++;
 	mData.resize(inx+curl);
     }
 }
@@ -973,19 +1035,19 @@ DVectC::dot(size_type inx, const DVector& rhs, size_type in2, size_type N) const
     check_substr(in2, N, rhs.getLength());
     if (!N) return 0.0;
 
-    register double sum = 0.0;
+    double sum = 0.0;
     const float* myData = reinterpret_cast<const float*>(refTData() + inx);
     if (rhs.C_data()) {
         const float* cdata = reinterpret_cast<const float*>(rhs.refData())
 	                     + 2*in2;
-	for (register size_type i=0; i<N; ++i) {
+	for (size_type i=0; i<N; ++i) {
 	    sum += double(*myData++) * double(*cdata++);
 	    sum -= double(*myData++) * double(*cdata++);
 	}
     } else if (rhs.W_data()) {
 	const double* cdata = reinterpret_cast<const double*>(rhs.refData())
 	                    + 2*in2;
-	for (register size_type i=0; i<N; ++i) {
+	for (size_type i=0; i<N; ++i) {
 	    sum += double(*myData++) * *cdata++;
 	    sum -= double(*myData++) * *cdata++;
 	}
@@ -1015,19 +1077,19 @@ DVectW::dot(size_type inx, const DVector& rhs, size_type in2, size_type N) const
     check_substr(in2, N, rhs.getLength());
     if (!N) return 0.0;
 
-    register double sum = 0.0;
+    double sum = 0.0;
     const double* myData = reinterpret_cast<const double*>(refTData() + inx);
     if (rhs.C_data()) {
         const float* cdata = reinterpret_cast<const float*>(rhs.refData())
 	                     + 2*in2;
-	for (register size_type i=0; i<N; ++i) {
+	for (size_type i=0; i<N; ++i) {
 	    sum += *myData++ * double(*cdata++);
 	    sum -= *myData++ * double(*cdata++);
 	}
     } else if (rhs.W_data()) {
 	const double* cdata = reinterpret_cast<const double*>(rhs.refData())
 	                    + 2*in2;
-	for (register size_type i=0; i<N; ++i) {
+	for (size_type i=0; i<N; ++i) {
 	    sum += *myData++ * *cdata++;
 	    sum -= *myData++ * *cdata++;
 	}
@@ -1058,7 +1120,7 @@ DVecType<T>::dot(size_type inx, const DVector& rhs, size_type in2, size_type N)
     check_substr(in2, N, rhs.getLength());
     if (!N) return 0.0;
 
-    register double sum = 0.0;
+    double sum = 0.0;
     const T* myData = refTData() + inx;
     if (rhs.D_data()) {
         const double* fdata = reinterpret_cast<const double*>(rhs.refData())+in2;
@@ -1166,23 +1228,23 @@ DVectC::mpy(size_type inx, const DVector& rhs, size_type in2, size_type N) {
     check_substr(in2, N, rhs.getLength());
     if (!N) return *this;
 
-    register fComplex* myData = refTData() + inx;
+    fComplex* myData = refTData() + inx;
     if (rhs.getType() == t_double) {
-	register const double* p;
+	const double* p;
 	p = reinterpret_cast<const double*>(rhs.refData()) + in2;
-	for (register size_type i=0; i<N; ++i) myData[i] *= *p++;
+	for (size_type i=0; i<N; ++i) myData[i] *= *p++;
     } else if (rhs.getType() == t_complex) {
-        register const fComplex*
+        const fComplex*
 	    data = reinterpret_cast<const fComplex*>(rhs.refData())+in2;
-        for (register size_type i=0; i<N; ++i) myData[i] *= data[i];
+        for (size_type i=0; i<N; ++i) myData[i] *= data[i];
     } else if (rhs.getType() == t_dcomplex) {
-        register const dComplex* 
+        const dComplex* 
 	    data = reinterpret_cast<const dComplex*>(rhs.refData())+in2;
-        for (register size_type i=0; i<N; ++i) myData[i] *= fComplex(data[i]);
+        for (size_type i=0; i<N; ++i) myData[i] *= fComplex(data[i]);
     } else {
-        register double* data = new double[N];
+        double* data = new double[N];
         rhs.getData(in2, N, data);
-	for (register size_type i=0; i<N; ++i) myData[i] *= data[i];
+	for (size_type i=0; i<N; ++i) myData[i] *= data[i];
 	delete[] data;
     }
     return *this;
@@ -1196,26 +1258,26 @@ DVectW::mpy(size_type inx, const DVector& rhs, size_type in2, size_type N) {
     check_substr(in2, N, rhs.getLength());
     if (!N) return *this;
 
-    register dComplex* myData = refTData() + inx;
+    dComplex* myData = refTData() + inx;
     if (rhs.getType() == t_double) {
-	//register const double* p;
+	//const double* p;
 	//p = reinterpret_cast<const double*>(rhs.refData()) + in2;
-	//for (register size_type i=0; i<N; ++i) myData[i] *= *p++;
+	//for (size_type i=0; i<N; ++i) myData[i] *= *p++;
 	const double* data =
 	    reinterpret_cast<const double*>(rhs.refData()) + in2;
 	vcmuld(data, myData, N);
     } else if (rhs.getType() == t_complex) {
-        register const fComplex*
+        const fComplex*
 	    data = reinterpret_cast<const fComplex*>(rhs.refData())+in2;
-        for (register size_type i=0; i<N; ++i) myData[i] *= dComplex(data[i]);
+        for (size_type i=0; i<N; ++i) myData[i] *= dComplex(data[i]);
     } else if (rhs.getType() == t_dcomplex) {
         const dComplex*
 	   data = reinterpret_cast<const dComplex*>(rhs.refData())+in2;
 	vmul(data, myData, N);
     } else {
-        register double* data = new double[N];
+        double* data = new double[N];
         rhs.getData(in2, N, data);
-	//for (register size_type i=0; i<N; ++i) myData[i] *= data[i];
+	//for (size_type i=0; i<N; ++i) myData[i] *= data[i];
         vcmuld(data, myData, N);
 	delete[] data;
     }
@@ -1235,15 +1297,15 @@ DVecType<T>::mpy(size_type inx, const DVector& rhs, size_type in2, size_type N){
     if (in2 + N > len) N = len - in2;
 
     if (rhs.getType() != getType()) {
-        register T* data = new T[N];
+        T* data = new T[N];
         rhs.getData(in2, N, data);
-	register T* myData = refTData() + inx;
-	for (register size_type i=0; i<N; ++i) myData[i] *= data[i];
+	T* myData = refTData() + inx;
+	for (size_type i=0; i<N; ++i) myData[i] *= data[i];
 	delete[] data;
     } else {
-	register T* myData = refTData() + inx;
-        register const T* data = reinterpret_cast<const T*>(rhs.refData())+in2;
-        for (register size_type i=0; i<N; ++i) myData[i] *= data[i];
+	T* myData = refTData() + inx;
+        const T* data = reinterpret_cast<const T*>(rhs.refData())+in2;
+        for (size_type i=0; i<N; ++i) myData[i] *= data[i];
     }
     return *this;
 }
@@ -1256,23 +1318,23 @@ DVectC::cmpy(size_type inx, const DVector& rhs, size_type in2, size_type N) {
     check_substr(in2, N, rhs.getLength());
     if (!N) return *this;
 
-    register fComplex* myData = refTData() + inx;
+    fComplex* myData = refTData() + inx;
     if (rhs.getType() == t_double) {
-	register const double* p;
+	const double* p;
 	p = reinterpret_cast<const double*>(rhs.refData()) + in2;
-	for (register size_type i=0; i<N; ++i) myData[i] *= *p++;
+	for (size_type i=0; i<N; ++i) myData[i] *= *p++;
     } else if (rhs.getType() == t_complex) {
-        register const fComplex*
+        const fComplex*
 	    data = reinterpret_cast<const fComplex*>(rhs.refData())+in2;
-        for (register size_type i=0; i<N; ++i) myData[i] *= ~data[i];
+        for (size_type i=0; i<N; ++i) myData[i] *= ~data[i];
     } else if (rhs.getType() == t_dcomplex) {
-        register const dComplex* 
+        const dComplex* 
 	    data = reinterpret_cast<const dComplex*>(rhs.refData())+in2;
-        for (register size_type i=0; i<N; ++i) myData[i] *= ~fComplex(data[i]);
+        for (size_type i=0; i<N; ++i) myData[i] *= ~fComplex(data[i]);
     } else {
-        register double* data = new double[N];
+        double* data = new double[N];
         rhs.getData(in2, N, data);
-	for (register size_type i=0; i<N; ++i) myData[i] *= data[i];
+	for (size_type i=0; i<N; ++i) myData[i] *= data[i];
 	delete[] data;
     }
     return *this;
@@ -1286,23 +1348,23 @@ DVectW::cmpy(size_type inx, const DVector& rhs, size_type in2, size_type N) {
     check_substr(in2, N, rhs.getLength());
     if (!N) return *this;
 
-    register dComplex* myData = refTData() + inx;
+    dComplex* myData = refTData() + inx;
     if (rhs.getType() == t_double) {
-	register const double* 
+	const double* 
 	    p = reinterpret_cast<const double*>(rhs.refData()) + in2;
-	for (register size_type i=0; i<N; ++i) myData[i] *= *p++;
+	for (size_type i=0; i<N; ++i) myData[i] *= *p++;
     } else if (rhs.getType() == t_complex) {
-        register const fComplex*
+        const fComplex*
 	    data = reinterpret_cast<const fComplex*>(rhs.refData())+in2;
-        for (register size_type i=0; i<N; ++i) myData[i] *= ~dComplex(data[i]);
+        for (size_type i=0; i<N; ++i) myData[i] *= ~dComplex(data[i]);
     } else if (rhs.getType() == t_dcomplex) {
-        register const dComplex*
+        const dComplex*
 	    data = reinterpret_cast<const dComplex*>(rhs.refData())+in2;
-        for (register size_type i=0; i<N; ++i) myData[i] *= ~data[i];
+        for (size_type i=0; i<N; ++i) myData[i] *= ~data[i];
     } else {
-        register double* data = new double[N];
+        double* data = new double[N];
         rhs.getData(in2, N, data);
-	for (register size_type i=0; i<N; ++i) myData[i] *= data[i];
+	for (size_type i=0; i<N; ++i) myData[i] *= data[i];
 	delete[] data;
     }
     return *this;
@@ -1316,15 +1378,15 @@ DVecType<T>::cmpy(size_type inx, const DVector& rhs, size_type in2, size_type N)
     check_substr(in2, N, rhs.getLength());
     if (!N) return *this;
 
-    register T* myData = refTData() + inx;
+    T* myData = refTData() + inx;
     if (rhs.getType() != getType()) {
-        register T* data = new T[N];
+        T* data = new T[N];
         rhs.getData(in2, N, data);
-	for (register size_type i=0; i<N; ++i) myData[i] *= data[i];
+	for (size_type i=0; i<N; ++i) myData[i] *= data[i];
 	delete[] data;
     } else {
-        register const T* data = reinterpret_cast<const T*>(rhs.refData())+in2;
-        for (register size_type i=0; i<N; ++i) myData[i] *= data[i];
+        const T* data = reinterpret_cast<const T*>(rhs.refData())+in2;
+        for (size_type i=0; i<N; ++i) myData[i] *= data[i];
     }
     return *this;
 }
@@ -1337,18 +1399,18 @@ DVecType<T>::div(size_type inx, const DVector& rhs, size_type in2, size_type N)
     check_substr(in2, N, rhs.getLength());
     if (!N) return *this;
 
-    register T* myData = refTData() + inx;
+    T* myData = refTData() + inx;
     if (rhs.getType() != getType()) {
         T* data = new T[N];
         rhs.getData(in2, N, data);
-	for (register size_type i=0; i<N; ++i) {
+	for (size_type i=0; i<N; ++i) {
 	    if (!data[i]) myData[i]  = T(0);
 	    else          myData[i] /= data[i];
 	}
 	delete[] data;
     } else {
         const T* data = reinterpret_cast<const T*>(rhs.refData())+in2;
-        for (register size_type i=0; i<N; ++i) {
+        for (size_type i=0; i<N; ++i) {
 	    if (!data[i]) myData[i]  = T(0);
 	    else          myData[i] /= data[i];
 	}
@@ -1364,8 +1426,8 @@ DVectS::scale(size_type inx, double rhs, size_type N) {
 	check_substr(inx, N, mData.size());
 	if (!N) return *this;
 
-	register short* myData = refTData() + inx;
-	for (register size_type i=0; i<N; ++i) myData[i]=short(myData[i]*rhs);
+	short* myData = refTData() + inx;
+	for (size_type i=0; i<N; ++i) myData[i]=short(myData[i]*rhs);
     }
     return *this;
 }
@@ -1377,8 +1439,24 @@ DVectI::scale(size_type inx, double rhs, size_type N) {
 	check_substr(inx, N, mData.size());
 	if (!N) return *this;
 
-	register int* myData = refTData() + inx;
-	for (register size_type i=0; i<N; ++i) myData[i] = int(myData[i]*rhs);
+	int* myData = refTData() + inx;
+	for (size_type i=0; i<N; ++i) myData[i] = int(myData[i]*rhs);
+    }
+    return *this;
+}
+
+template<>
+DVectU&
+DVectU::scale(size_type inx, double rhs, size_type N) {
+    if (rhs < 0.0) {
+	throw runtime_error("unsigned vector scaled by a negative constant");
+    }
+    else if (rhs != 1.0) {
+	check_substr(inx, N, mData.size());
+	if (!N) return *this;
+
+	uint32_t* myData = refTData() + inx;
+	for (size_type i=0; i<N; ++i) myData[i] = uint32_t(myData[i]*rhs);
     }
     return *this;
 }
@@ -1410,7 +1488,7 @@ DVecType<T>::scale(size_type inx, double rhs, size_type N) {
 	check_substr(inx, N, mData.size());
 	if (!N) return *this;
 
-	register T* myData = refTData() + inx;
+	T* myData = refTData() + inx;
 	for (size_type i=0; i<N; ++i) *myData++ *= rhs;
     }
     return *this;
@@ -1426,8 +1504,8 @@ DVecType<T>::bias(size_type inx, double rhs, size_type N) {
 	check_substr(inx, N, mData.size());
 	if (!N) return *this;
 
-        register T* myData = refTData() + inx;
-	for (register size_type i=0; i<N; ++i) myData[i] += off;
+        T* myData = refTData() + inx;
+	for (size_type i=0; i<N; ++i) myData[i] += off;
     }
     return *this;
 }
@@ -1454,6 +1532,7 @@ DVecType<T>::Dump(std::ostream& out) const {
 //======================================  Explicit instantiations
 template class DVecType<short>;
 template class DVecType<int>;
+template class DVecType<uint32_t>;
 template class DVecType<float>;
 template class DVecType<double>;
 template class DVecType<fComplex>;
