@@ -220,7 +220,7 @@
 	  xbin_t xBin = xbin_t(fNbinx)/(xEnd - x0);
 	  long   nBinInc = 0;
 	  for ( int i = 0; i < ntimes; i++) {
-	      register xbin_t x = xp[i];
+	      xbin_t x = xp[i];
 	      if ( x >= xEnd ) {  // overflow
 		  fArray[fNbinx+1] += 1.0;
 		  if (fBinErrorFlag) fSumw2[fNbinx+1] += 1.0;
@@ -242,7 +242,7 @@
       }
       else if (fBinType==kVariableBin){  // variable bin
 	 for ( int i = 0; i < ntimes; i++) {
-	     register xbin_t x = xp[i];
+	     xbin_t x = xp[i];
 	     int index = GetBinNumber(x);
 	     fArray[index] += 1.0;
 	     if (fBinErrorFlag) fSumw2[index] += 1.0;
