@@ -76,19 +76,25 @@ TSeries::copy(const TSeries& ts) {
     return *this;
 }
 
-//======================================  Data Constructor
+//======================================  Short integer data Constructor
 TSeries::TSeries(const Time& t0, const Interval& dt, size_type NData, 
 		 const short dData[]) {
     setData(t0, dt, new DVectS(NData, dData));
 }
 
-//======================================  Data Constructor
+//======================================  Integer data Constructor
 TSeries::TSeries(const Time& t0, const Interval& dt, size_type NData, 
 		 const int dData[]) {
     setData(t0, dt, new DVectI(NData, dData));
 }
 
-//======================================  Data Constructor
+//======================================  Unsigned int data Constructor
+TSeries::TSeries(const Time& t0, const Interval& dt, size_type NData, 
+		 const uint32_t dData[]) {
+    setData(t0, dt, new DVectU(NData, dData));
+}
+
+//======================================  Float data Constructor
 TSeries::TSeries(const Time& t0, const Interval& dt, size_type NData, 
 		 const float dData[]) {
     setData(t0, dt, new DVectF(NData, dData));
@@ -387,32 +393,14 @@ TSeries::Convert(int type) {
 	case DVector::t_dcomplex:
 	    mData.reset(new DVectW);
 	    break;
+	case DVector::t_uint:
+	    mData.reset(new DVectU);
+	    break;
 	default:
 	    throw runtime_error("TSeries::Convert: Invalid type specified");
 	}
     } else if (mData->getType() != type) {
-	switch (type) {
-	case DVector::t_short:
-	    mData.reset(new DVectS(*mData));
-	    break;
-	case DVector::t_int:
-	    mData.reset(new DVectI(*mData));
-	    break;
-	case DVector::t_float:
-	    mData.reset(new DVectF(*mData));
-	    break;
-	case DVector::t_double:
-	    mData.reset(new DVectD(*mData));
-	    break;
-	case DVector::t_complex:
-	    mData.reset(new DVectC(*mData));
-	    break;
-	case DVector::t_dcomplex:
-	    mData.reset(new DVectW(*mData));
-	    break;
-	default:
-	    throw runtime_error("TSeries::Convert: Invalid type specified");
-	}
+	mData.reset(mData->convert(DVector::DVType(type))); 
     }
 }
 
@@ -430,7 +418,7 @@ TSeries::setData(const Time& t0, const Interval& dt, DVector* data) {
     mFNyquist = 2/double(dt);
     mStatus   = 0;
     mSigmaW   = 1.0;
-    mData.reset(data); // auto_ptr protected from resetting to the same pointer
+    if (data != mData.get()) mData.reset(data);
     return 0;
 }
 
@@ -499,7 +487,7 @@ TSeries::Append(const TSeries& t, size_type decim8) {
 	    }
 	    else if ( t.mData->size() >= decim8 ) {
 		size_type len = t.mData->size()/decim8;
-		auto_ptr<DVector> ext(t.mData->Extract(0, len, decim8));
+		unique_ptr<DVector> ext(t.mData->Extract(0, len, decim8));
 		mData->Append(*ext);
 	    }
 	    combineStatus(t.mStatus);
