@@ -1110,11 +1110,11 @@ wavearray<float> wavecluster::get(const char* name, int index, size_t type)
 double wavecluster::setMask(WSeries<double>& w, int nc, bool halo)
 {
 #if !defined (__SUNPRO_CC)
-   register int i;
-   register int j;
+   int i;
+   int j;
    int x, y, L, k;
-   register int* q = NULL;
-   register int* p = NULL; 
+   int* q = NULL;
+   int* p = NULL; 
    int* pp; 
    int* pm;
    wavepixel pix;
@@ -1351,7 +1351,7 @@ double wavecluster::setMask(WSeries<double>& w, int nc, bool halo)
       }
    }
 
-   register std::vector<int>* pN;
+   std::vector<int>* pN;
    int nM =  pList.size();
    slice S;
 
