@@ -12,11 +12,7 @@
 #include <cstring>
 
 #include "PConfig.h"
-#ifdef __GNU_STDC_OLD
-#include <gnusstream.h>
-#else
 #include <sstream>
-#endif
 
 #include "wavearray.hh"
 #include "wavefft.h"
@@ -84,7 +80,7 @@ operator=(const wavearray<DataType_t>& a)
    unsigned int i;
    unsigned int N = a.Slice.size();
    unsigned int m = a.Slice.stride();
-   register DataType_t *pm  = a.data + a.Slice.start();
+   DataType_t *pm  = a.data + a.Slice.start();
 
    if (this!=&a && N>0) {
       wavearray<DataType_t>::resize(N);
@@ -117,7 +113,7 @@ operator<<(wavearray<DataType_t>& a)
    unsigned int N = limit(a);
    unsigned int n = Slice.stride();
    unsigned int m = a.Slice.stride();
-   register DataType_t *p = a.data + a.Slice.start();
+   DataType_t *p = a.data + a.Slice.start();
 
    if(size())
       for (i=Slice.start(); i<N; i+=n){ data[i]  = *p; p += m; }
@@ -152,7 +148,7 @@ operator+=(wavearray<DataType_t> &a)
    unsigned int N = limit(a);
    unsigned int n = Slice.stride();
    unsigned int m = a.Slice.stride();
-   register DataType_t *p = a.data + a.Slice.start();
+   DataType_t *p = a.data + a.Slice.start();
 
    if(size())
       for (i=Slice.start(); i<N; i+=n){ data[i]  += *p; p += m; }
@@ -187,7 +183,7 @@ operator-=(wavearray<DataType_t> &a)
    unsigned int N = limit(a);
    unsigned int n = Slice.stride();
    unsigned int m = a.Slice.stride();
-   register DataType_t *p = a.data + a.Slice.start();
+   DataType_t *p = a.data + a.Slice.start();
 
    if(size())
       for (i=Slice.start(); i<N; i+=n){ data[i]  -= *p; p += m; }
@@ -237,7 +233,7 @@ operator*=(wavearray<DataType_t>& a)
    unsigned int N = limit(a);
    unsigned int n = Slice.stride();
    unsigned int m = a.Slice.stride();
-   register DataType_t *p = a.data + a.Slice.start();
+   DataType_t *p = a.data + a.Slice.start();
 
    if(size())
       for (i=Slice.start(); i<N; i+=n){ data[i]  *= *p; p += m; }
@@ -450,9 +446,9 @@ resample(wavearray<DataType_t> const &a, double f, int nF)
 
    const DataType_t *p = a.data;
 
-   register int i;
-   register int iL;
-   register double x;
+   int i;
+   int iL;
+   double x;
    double *temp=new double[nF];
    
    rate(f);
@@ -840,10 +836,10 @@ Stack(const wavearray<DataType_t> &td, double window)
 template<class DataType_t>
 double wavearray<DataType_t>::mean(double sigma) const 
 {
-   register size_t i;
-   register double x = 0;
+   size_t i;
+   double x = 0;
    size_t N = (size()>>2)<<2;
-   register DataType_t *p = data + size() - N;
+   DataType_t *p = data + size() - N;
 
    if(!size()) return 0.;
 
@@ -880,9 +876,9 @@ double wavearray<DataType_t>::mean(double sigma) const
 template<class DataType_t>
 double wavearray<DataType_t>::mean(const std::slice &s)
 {
-   register size_t i;
-   register double x = 0.;
-   register DataType_t *p = data + s.start();
+   size_t i;
+   double x = 0.;
+   DataType_t *p = data + s.start();
    size_t N = s.size();
    size_t m = (s.stride()<=0) ? 1 : s.stride();
    if(size()<limit(s)) N = (limit(s) - s.start() - 1)/m;
@@ -965,11 +961,11 @@ void wavearray<DataType_t>::mean(double t,
 template<class DataType_t>
 double wavearray<DataType_t>::rms() 
 {
-   register size_t i;
-   register double x = 0.;
-   register double y = 0.;
+   size_t i;
+   double x = 0.;
+   double y = 0.;
    size_t N = (size()>>2)<<2;
-   register DataType_t *p = data + size() - N;
+   DataType_t *p = data + size() - N;
 
    if(!size()) return 0.;
 
@@ -1064,11 +1060,11 @@ void wavearray<DataType_t>::rms(double t,
 template<class DataType_t>
 double wavearray<DataType_t>::rms(const std::slice &s) 
 {
-   register size_t i;
-   register double a = 0.;
-   register double x = 0.;
-   register double y = 0.;
-   register DataType_t *p = data + s.start();
+   size_t i;
+   double a = 0.;
+   double x = 0.;
+   double y = 0.;
+   DataType_t *p = data + s.start();
    size_t n = s.size();
    size_t m = (s.stride()<=0) ? 1 : s.stride();
 
@@ -1091,10 +1087,10 @@ double wavearray<DataType_t>::rms(const std::slice &s)
 template<class DataType_t>
 DataType_t wavearray<DataType_t>::max() const
 {
-   register unsigned int i;
-   register DataType_t x = DataType_t(0.);
+   unsigned int i;
+   DataType_t x = DataType_t(0.);
    size_t N = (size()>>2)<<2;
-   register DataType_t *p = data + size() - N;
+   DataType_t *p = data + size() - N;
 
    if(!size()) return 0;
 
@@ -1111,10 +1107,10 @@ DataType_t wavearray<DataType_t>::max() const
 template<class DataType_t>
 DataType_t wavearray<DataType_t>::min() const
 {
-   register size_t i;
-   register DataType_t x = DataType_t(0.);
+   size_t i;
+   DataType_t x = DataType_t(0.);
    size_t N = (size()>>2)<<2;
-   register DataType_t *p = data + size() - N;
+   DataType_t *p = data + size() - N;
 
    if(!size()) return 0;
 
@@ -1133,8 +1129,8 @@ template<class DataType_t>
 int wavearray<DataType_t>::getSampleRank(size_t n, size_t l, size_t r) const
 {
    DataType_t v;
-   register int i = l-1;           // save left boundary 
-   register int j = r;             // save right boundary 
+   int i = l-1;           // save left boundary 
+   int j = r;             // save right boundary 
    
    v = data[n];                    // pivot
    data[n]=data[r]; data[r]=v;     // store pivot
@@ -1529,9 +1525,9 @@ wavearray<double> wavearray<DataType_t>::white(double t) const
      return mode>0 ? norm50 : meDIan;
    }
 
-   register DataType_t *p = data;
+   DataType_t *p = data;
    wavearray<DataType_t> w(m);
-   register double x;
+   double x;
 
    DataType_t ** pp = (DataType_t **)malloc(m*sizeof(DataType_t*));
 
@@ -1588,14 +1584,14 @@ void wavearray<DataType_t>::waveSort(DataType_t** pin, size_t l, size_t r) const
 {
    size_t k;
 
-   register DataType_t v;
-   register DataType_t* p;
-   register DataType_t** pp = pin;
+   DataType_t v;
+   DataType_t* p;
+   DataType_t** pp = pin;
 
    if(pp==NULL) return;      
 
-   register size_t i = (r+l)/2;         // median
-   register size_t j = r-1;             // pivot storage index
+   size_t i = (r+l)/2;         // median
+   size_t j = r-1;             // pivot storage index
 
 // sort l,i,r
    if(*pp[l] > *pp[i]) {p=pp[l]; pp[l]=pp[i]; pp[i]=p;}
@@ -1639,9 +1635,9 @@ void wavearray<DataType_t>::waveSort(DataType_t** pin, size_t l, size_t r) const
 template<class DataType_t>
 double wavearray<DataType_t>::getStatistics(double &m, double &r) const
 {
-   register size_t i;
-   register double a;
-   register double b;
+   size_t i;
+   double a;
+   double b;
    DataType_t *p = const_cast<DataType_t *>(data);
    double y = 0.;
    size_t N = size() - 1 + size_t(size()&1);
