@@ -398,7 +398,7 @@ LscCalib::readXml(const string& file, const string& name, const string& chan) {
 
 void 
 LscCalib::readXml(xsil::Xreader& xr, const string& name, const string& chan) {
-    auto_ptr<xsil::xobj> doc(xr.readDoc());
+    unique_ptr<xsil::xobj> doc(xr.readDoc());
 
     xsil::ligolw* cDoc = dynamic_cast<xsil::ligolw*>(doc->find(name, "LIGO_LW"));
     if (!cDoc) throw runtime_error("Unable to find calibration for channel");
@@ -544,7 +544,7 @@ LscCalib::writeXml(const string& file) {
 static void
 putFSeries(xsil::ligolw* doc, const char* name, const FSeries& fs) {
     if (fs.empty()) return;
-    auto_ptr<xsil::array> pArray(new xsil::array(name, "float"));
+    unique_ptr<xsil::array> pArray(new xsil::array(name, "float"));
     int N=fs.getNStep()+1;
     pArray->addDim("Frequency", N);
     pArray->addDim("Frequency_Modulus_Phase",3);
@@ -568,7 +568,7 @@ static void
 putTSeries(xsil::ligolw* doc, const char* name, const TSeries& ts) {
     long N=ts.getNSample();
     if (!N) return;
-    auto_ptr<xsil::array> pArray(new xsil::array(name, "double"));
+    unique_ptr<xsil::array> pArray(new xsil::array(name, "double"));
     pArray->addDim("GPS", N);
     pArray->addDim("GPS_Value",2);
     vector<double> dat(N);
@@ -591,7 +591,7 @@ static void
 put2TSeries(xsil::ligolw* doc, const char* name, 
 	    const TSeries& ts1, const TSeries& ts2) {
     if (ts1.isEmpty()) return;
-    auto_ptr<xsil::array> pArray(new xsil::array(name, "double"));
+    unique_ptr<xsil::array> pArray(new xsil::array(name, "double"));
     int N=ts1.getNSample();
     pArray->addDim("GPS", N);
     pArray->addDim("GPS_Alpha_AlphaBeta",3);
@@ -617,7 +617,7 @@ put2TSeries(xsil::ligolw* doc, const char* name,
 void 
 LscCalib::writeXml(xsil::Xwriter& xw) {
     prepare();
-    auto_ptr<xsil::ligolw> d(new xsil::ligolw(mName.c_str()));
+    unique_ptr<xsil::ligolw> d(new xsil::ligolw(mName.c_str()));
     if (!d.get()) throw runtime_error("Unable to construct calibration");
 
     if (!mChannel.empty()) 
@@ -639,7 +639,7 @@ LscCalib::writeXml(xsil::Xwriter& xw) {
 
     int nChan = mGainChan.size();
     if (nChan) {
-        auto_ptr<xsil::table> tb(new xsil::table("DARMChannels"));
+        unique_ptr<xsil::table> tb(new xsil::table("DARMChannels"));
 	tb->addColumn("ChannelName", "string");
 	tb->addColumn("RefValue",    "float");
 	for (int i=0; i<nChan; ++i) {
@@ -656,7 +656,7 @@ LscCalib::writeXml(xsil::Xwriter& xw) {
     if (!mAlphaBeta.empty()) putTSeries(d.get(), "OLoopFactor", mAlphaBeta);
 
     //----------------------------------  Put doc into wrapper, write it out.
-    auto_ptr<xsil::ligolw> wrap(new xsil::ligolw);
+    unique_ptr<xsil::ligolw> wrap(new xsil::ligolw);
     wrap->addObject(d.release());
     wrap->Spew(xw);
 }
@@ -678,7 +678,7 @@ LscCalib::writeFrame(const string& file, CalibChanList::cal_format fmt) {
     //----------------------------------  Build the frame
     fw.buildFrame(mStartTime, mDuration);
     fw.addWriterHistory();
-    fw.addHistory("LscCalib", Now(), "$Header: https://redoubt.ligo-wa.caltech.edu/svn/gds/trunk/DMT/EasyCalibrate/LscCalib.cc 6947 2013-12-17 17:29:17Z john.zweizig@LIGO.ORG $");
+    fw.addHistory("LscCalib", Now(), "$Header: https://redoubt.ligo-wa.caltech.edu/svn/gds/trunk/DMT/EasyCalibrate/LscCalib.cc 7601 2016-04-15 20:51:43Z john.zweizig@LIGO.ORG $");
     fw.addHistory(CALFRAME_VERSION_NAME, mCalibTime, mVersion);
     fw.addHistory(CALFRAME_COMMENT_NAME, mCalibTime, mComment);
 
