@@ -1,6 +1,6 @@
 // Yo, Emacs! This is -*- mode: C++; c-basic-offset: 4;  -*-
 //
-// $Id: OperStateCondList.cc 7281 2015-01-20 19:17:30Z ed.maros@LIGO.ORG $
+// $Id: OperStateCondList.cc 7524 2015-11-19 01:18:08Z ed.maros@LIGO.ORG $
 //
 // Author: David Chin <dwchin@umich.edu>
 //
@@ -1500,12 +1500,12 @@ void OperStateCondList::incrChanUtilMaybe(std::string chanName)
  * POSSIBILITY OF SUCH DAMAGE.
  */
 
-char *OperStateCondList::strtok_r(register char *s,
-                                  register const char *delim,
-                                  register char **lasts)
+char *OperStateCondList::strtok_r(char *s,
+                                  const char *delim,
+                                  char **lasts)
 {
-    register char *spanp;
-    register int c, sc;
+    char *spanp;
+    int c, sc;
     char *tok;
 
     if (s == NULL && (s = *lasts) == NULL)
