@@ -1222,20 +1222,19 @@ static int SIStrLog(SIStream * sis, char *condition, int value)
     }
 
   /*------ Write out the message ------*/
-
+#define LOG_BY_DATE 1
     file = NULL;
-    /* See if the log directory /lig/logs/<ifo> exists */
+    /* See if the log directory /ligo/logs/<ifo> exists */
     {
 	struct stat statbuf;
 	mode_t mode = S_IRUSR | S_IRGRP | S_IROTH |
-	    S_IXUSR | S_IXGRP | S_IXOTH | S_IWUSR;
+	    S_IXUSR | S_IXGRP | S_IXOTH | S_IWUSR | S_IWGRP ;
 	/* Test for /ligo */
 	if ((stat("/ligo", &statbuf) == 0) && S_ISDIR(statbuf.st_mode)) {
 	    /* Test for /ligo/logs or attempt to make it. */
-	    if (((stat("/ligo/logs", &statbuf) == 0)
-		 && S_ISDIR(statbuf.st_mode))
-		|| (mkdir("/ligo/logs", mode) == 0)) {
-		char dirbuf[32];
+	    if (((stat("/ligo/logs", &statbuf) == 0) && S_ISDIR(statbuf.st_mode)) || (mkdir("/ligo/logs", mode) == 0)) 
+	    {
+		char dirbuf[64];
 		char *ifo = getenv("ifo");
 
 		if (ifo == NULL) {
@@ -1243,13 +1242,49 @@ static int SIStrLog(SIStream * sis, char *condition, int value)
 		}
 		/* Test for /ligo/logs/<ifo> or attempt to make it. */
 		sprintf(dirbuf, "/ligo/logs/%2s", ifo);
-		if (((stat(dirbuf, &statbuf) == 0)
-		     && S_ISDIR(statbuf.st_mode))
-		    || (mkdir(dirbuf, mode) == 0)) {
+		if (((stat(dirbuf, &statbuf) == 0) && S_ISDIR(statbuf.st_mode)) || (mkdir(dirbuf, mode) == 0)) 
+	        {
+#ifdef LOG_BY_DATE
+		    /* Test for /ligo/logs/<ifo>/injection_log/ or attempt to make it. */
+		    sprintf(dirbuf, "/ligo/logs/%2s/injection_log", ifo) ;
+		    if (((stat(dirbuf, &statbuf) == 0) && S_ISDIR(statbuf.st_mode)) || (mkdir(dirbuf, mode) == 0)) 
+		    {
+			/* Get the date, month, and day to use for directories and file name. */
+			time_t	t ;
+			struct tm *utc ;
+			t = time((time_t *) NULL) ;
+			if (t < 0) 
+			{
+			    fprintf(stderr, "Unable to get time.\n") ;
+			} 
+			else 
+			{
+			    utc = gmtime(&t) ;
+			    /* The directories under /ligo/logs/<ifo>/injection_log/ will be */
+			    /* yyyy/mm/dd/ */
+			    sprintf(dirbuf, "/ligo/logs/%2s/injection_log/%4d", ifo, utc->tm_year+1900) ;
+			    if (((stat(dirbuf, &statbuf) == 0) && S_ISDIR(statbuf.st_mode)) || mkdir(dirbuf, mode) == 0) 
+			    {
+				/* Try the /ligo/logs/<ifo>/injection_log/<yyyy>/<mm> directory */
+				sprintf(dirbuf, "/ligo/logs/%2s/injection_log/%4d/%2.2d", ifo, utc->tm_year+1900, utc->tm_mon+1) ;
+				if (((stat(dirbuf, &statbuf) == 0) && S_ISDIR(statbuf.st_mode)) || mkdir(dirbuf, mode) == 0) 
+				{
+				    sprintf(dirbuf, "/ligo/logs/%2s/injection_log/%4d/%2.2d/%2.2d", ifo, utc->tm_year+1900, utc->tm_mon+1, utc->tm_mday) ;
+				    if (((stat(dirbuf, &statbuf) == 0) && S_ISDIR(statbuf.st_mode)) || mkdir(dirbuf, mode) == 0) 
+				    {
+					sprintf(dirbuf, "/ligo/logs/%2s/injection_log/%4d/%2.2d/%2.2d/injection.txt",  ifo, utc->tm_year+1900, utc->tm_mon+1, utc->tm_mday) ;
+					file = fopen(dirbuf, "a") ;
+				    }
+				} 
+			    }		
+			} 
+		    } 
+#else
 		    /* All is good, log the event in /ligo/logs/<ifo>/injection_log.txt */
 		    sprintf(dirbuf, "/ligo/logs/%2s/injection_log.txt",
 			    ifo);
 		    file = fopen(dirbuf, "a");
+#endif
 		} else {
 		    fprintf(stderr, "stat of %s failed\n", dirbuf);
 		}
