@@ -1,4 +1,4 @@
-VCSID("$(#)$Id: gdscmd.cc 7338 2015-04-14 23:24:47Z ed.maros@LIGO.ORG $");
+VCSID("$(#)$Id: gdscmd.cc 7574 2016-02-16 20:17:52Z james.batch@LIGO.ORG $");
 /*----------------------------------------------------------------------*/
 /*                                                         		*/
 /* Module Name: gdscmd							*/
@@ -585,6 +585,7 @@ extern "C"
          string		val;
          string::size_type pos;
          var = message.substr (3, message.size()).c_str();
+	 // cerr << "gdsCmd() - set " << var << endl ;
 	 // Get rid of leading spaces.
          while ((var.size() > 0) && (var[0] == ' ')) {
             var.erase (0, 1);
@@ -809,6 +810,7 @@ extern "C"
             // setup data broker
             string var = stInputSource;
             string newbrokername = stInputSourceOnline;
+	    // As a reminder, gds is a diagStorage object created in gdsCmdInit().
             gds->get (var, newbrokername, 0);
             //cerr << "INPUT Source = " << newbrokername << endl;
             stringcase isource (newbrokername.c_str());
@@ -887,8 +889,24 @@ extern "C"
                         user_ndsport = atoi (s.c_str() + pos + 1);
                      }
                   }
+
+		  // Extract the epoch start time and stop time if they are
+		  // present in the newbrokername.
+		  string es("epoch_start=") ;
+		  string ee("epoch_end=") ;
+		  string::size_type es_pos = s.find(es) ;
+		  string::size_type ee_pos = s.find(ee) ;
+		  unsigned long epoch_start = 0 ;
+		  unsigned long epoch_stop = 0 ;
+		  if (es_pos != string::npos && ee_pos != string::npos)
+		  {
+		     epoch_start = atoi(s.c_str() + es_pos + es.length()) ;
+		     epoch_stop = atoi(s.c_str() + ee_pos + ee.length()) ;
+		  }
+
                   // create new data broker
                   // cerr << "New user NDS2 data broker" << endl;
+		  // Defined in storage/nds2input.hh,cc.
                   nds2Manager* nds2mgr = new (nothrow) nds2Manager (
                                        gds, tpmgr, _RTDD_LAZY);
                   if (nds2mgr == 0) {
@@ -897,9 +915,10 @@ extern "C"
                   // establish connection to nds2
                   else if (user_ndsserver.empty() ||
                           !nds2mgr->connect (user_ndsserver.c_str(), 
-                                            user_ndsport, true)) {
+                                            user_ndsport, true, 
+					    epoch_start, epoch_stop)) {
                      delete nds2mgr;
-                     *res = cmdreply ("error: Unable to connect to NDS2");
+                     *res = cmdreply ("Error: Unable to connect to NDS2");
                   }
                   else {
                      databroker = nds2mgr;
@@ -1131,7 +1150,7 @@ extern "C"
          // restore from disk
          if (*res == 0) {
             if (gds->frestore (filename, flags)) {
-               *res = cmdreply (filename + " restored");
+//               *res = cmdreply (filename + " restored");
             }
             else {
                *res = cmdreply ("error: " + gds->errmsg());
