@@ -1,4 +1,4 @@
-/* version $Id: testchn.cc 7413 2015-07-17 22:23:49Z ed.maros@LIGO.ORG $ */
+/* version $Id: testchn.cc 7574 2016-02-16 20:17:52Z james.batch@LIGO.ORG $ */
 /*----------------------------------------------------------------------*/
 /*                                                         		*/
 /* Module Name: testchn							*/
@@ -19,7 +19,7 @@ namespace diag {
    bool channelHandler::setSiteIfo (char SiteDefault, char SiteForce, 
                      char IfoDefault, char IfoForce)
    {
-      siteDefault = SiteDefault;
+      siteDefault = siteDefault;
       siteForce = SiteForce;
       ifoDefault = IfoDefault;
       ifoForce = IfoForce;
@@ -28,13 +28,12 @@ namespace diag {
 
 
    bool channelHandler::channelInfo (const string& name, 
-                     gdsChnInfo_t& info) const
+                     gdsChnInfo_t& info, int rate) const
    {
       return (gdsChannelInfo (channelName (name).c_str(), 
                              &info) == 0);
    }
 
-
    string channelHandler::channelName (const string& name) const
    {
       return name;
