@@ -1,4 +1,4 @@
-/* version $Id: diagnames.h 7276 2015-01-09 21:36:34Z james.batch@LIGO.ORG $ */
+/* version $Id: diagnames.h 7574 2016-02-16 20:17:52Z james.batch@LIGO.ORG $ */
 /*----------------------------------------------------------------------*/
 /*                                                         		*/
 /* Module Name: diagnames.h						*/
@@ -67,6 +67,8 @@
 
 #include "dtt/gdsmain.h"
 
+#define TEST_RATE 1
+
 /** @name Names of diagnostics parameters.
     This module is a global resource listing of character strings used
     by diagnostics tests. It lists names of diagnostics tests, 
@@ -658,6 +660,7 @@ namespace diag {
    const char stAverageType[] = "AverageType";
    const char stMeasurementChannel[] = "MeasurementChannel";
    const char stMeasurementChannelActive[] = "MeasurementActive";
+   const char stMeasurementChannelRate[] = "MeasurementChannelRate" ;
    const char stStimulusActive[] = "StimulusActive";
    const char stStimulusType[] = "StimulusType";
    const char stStimulusChannel[] = "StimulusChannel";
@@ -679,6 +682,10 @@ namespace diag {
    const double srMeasurementTimeDef[] = {0.1, 10};
    const char srSettlingTime[] = "SettlingTime";
    const double srSettlingTimeDef = 0.25;
+#ifdef EXC_RAMP_DOWN
+   const char srRampDown[] = "RampDown" ;
+   const double srRampDownDef = 0 ;
+#endif
    const char* const srAverages = stAverages;
    const int srAveragesDef = 1;
    const char* const srAverageType = stAverageType;
@@ -697,6 +704,7 @@ namespace diag {
    const char* const srMeasurementChannelActive = 
    stMeasurementChannelActive;
    const char* const srMeasurementChannel = stMeasurementChannel;
+   const char* const srMeasurementChannelRate = stMeasurementChannelRate ;
    const char srHarmonicOrder[] = "HarmonicOrder";
    const int srHarmonicOrderDef = 1;
    const char srWindow[] = "Window";
@@ -736,6 +744,7 @@ namespace diag {
    const char* const ssMeasurementChannelActive = 
    stMeasurementChannelActive;
    const char* const ssMeasurementChannel = stMeasurementChannel;
+   const char* const ssMeasurementChannelRate = stMeasurementChannelRate ;
    const char ssHarmonicOrder[] = "HarmonicOrder";
    const int ssHarmonicOrderDef = 1;
    const char ssWindow[] = "Window";
@@ -793,6 +802,7 @@ namespace diag {
    const char* const fftMeasurementChannelActive = 
    stMeasurementChannelActive;
    const char* const fftMeasurementChannel = stMeasurementChannel;
+   const char* const fftMeasurementChannelRate = stMeasurementChannelRate ;
 
    /* Time Series */
    const char stTimeSeries[] = "TimeSeries";
@@ -802,6 +812,10 @@ namespace diag {
    const double tsPreTriggerTimeDef = 0;
    const char tsSettlingTime[] = "SettlingTime";
    const double tsSettlingTimeDef = 0;
+#ifdef EXC_RAMP_DOWN
+   const char tsRampDown[] = "RampDown" ;
+   const double tsRampDownDef = 0 ;
+#endif
    const char tsDeadTime[] = "DeadTime";
    const double tsDeadTimeDef = 0;
    const char tsBW[] = "BW";
@@ -838,6 +852,7 @@ namespace diag {
    const char* const tsMeasurementChannelActive = 
    stMeasurementChannelActive;
    const char* const tsMeasurementChannel = stMeasurementChannel;
+   const char* const tsMeasurementChannelRate = stMeasurementChannelRate ;
 
    /* Random Response */
    const char stRandomResponse[] = "RandomResponse";
