@@ -1,4 +1,4 @@
-VCSID("$(#)$Id: stdtest.cc 7338 2015-04-14 23:24:47Z ed.maros@LIGO.ORG $");
+VCSID("$(#)$Id: stdtest.cc 7574 2016-02-16 20:17:52Z james.batch@LIGO.ORG $");
 /* -*- mode: c++; c-basic-offset: 3; -*- */
 /*----------------------------------------------------------------------*/
 /*                                                         		*/
@@ -31,6 +31,7 @@ VCSID("$(#)$Id: stdtest.cc 7338 2015-04-14 23:24:47Z ed.maros@LIGO.ORG $");
 #include <time.h>
 #endif
 
+#define test_RATE 1
 
 namespace diag {
    using namespace std;
@@ -56,23 +57,26 @@ namespace diag {
    }
 
 
-   bool stdtest::stimulus::calcSignal (tainsec_t t0, tainsec_t duration)
+   bool stdtest::stimulus::calcSignal (tainsec_t t0, tainsec_t duration, tainsec_t ramp)
    {
       AWG_Component	comp[2];
       int 		cnum = 1;
+      if (my_debug) cerr << "stdtest::stimulus::calcSignal(t0 = " << t0 << ", duration = " << duration << ", ramp = " << ramp << ")" << endl ;
    
       switch (waveform) {
          case awgSine:
          case awgRamp:
          case awgTriangle:
             {
+	       if (my_debug) cerr << "  waveform type is " << waveform << ", (Sine, Ramp, Triangle)" << endl ;
                if (awgPeriodicComponent (waveform, freq, ampl, phas,
                                     offs, comp) < 0) {
                   return false;
                }
                comp[0].par[2] = phas;
-               comp[0].start = t0;
-               comp[0].duration = duration;
+               comp[0].start = t0 ;
+               comp[0].duration = duration ;
+	       comp[0].ramptime[0]=ramp ;
                break;
             }
          case awgSquare:
@@ -83,19 +87,21 @@ namespace diag {
                      return false;
                   }
                   comp[0].par[2] = phas;
-                  comp[0].start = t0;
-                  comp[0].duration = duration;
+                  comp[0].start = t0 ;
+                  comp[0].duration = duration ;
+		  comp[0].ramptime[0]=ramp ;
                }
                else {
                   if (awgSquareWaveComponent (freq, ampl, phas,
                                        offs, ratio, comp) < 0) {
                      return false;
                   }
-                  comp[0].start = t0 + (tainsec_t) 
+                  comp[0].start = t0  + (tainsec_t) 
                      (__ONESEC * (fMod (phas / TWO_PI, 1) - 1)/ freq + 0.5);
-                  comp[0].duration = duration;
-                  comp[1].start = t0;
-                  comp[1].duration = duration;
+                  comp[0].duration = duration ;
+		  comp[0].ramptime[0]=ramp ;
+                  comp[1].start = t0 ;
+                  comp[1].duration = duration ;
                   cnum = 2;
                }
                break;
@@ -106,8 +112,9 @@ namespace diag {
                                     offs, comp) < 0) {
                   return false;
                }
-               comp[0].start = t0;
-               comp[0].duration = duration;
+               comp[0].start = t0 ;
+               comp[0].duration = duration ;
+	       comp[0].ramptime[0]=ramp ;
                break;
             }
          case awgConst:
@@ -115,8 +122,9 @@ namespace diag {
                if (awgConstantComponent (ampl, comp) < 0) {
                   return false;
                }
-               comp[0].start = t0;
-               comp[0].duration = duration;
+               comp[0].start = t0 ;
+               comp[0].duration = duration ;
+	       comp[0].ramptime[0]=ramp ;
                break;
             }
          case awgNoiseN:
@@ -126,8 +134,9 @@ namespace diag {
                                     frange, ampl, offs, comp) < 0) {
                   return false;
                }
-               comp[0].start = t0;
-               comp[0].duration = duration;
+               comp[0].start = t0 ;
+               comp[0].duration = duration ;
+	       comp[0].ramptime[0]=ramp ;
                break;
             }
          case awgArb:
@@ -136,8 +145,9 @@ namespace diag {
                                     offs, comp) < 0) {
                   return false;
                }
-               comp[0].start = t0;
-               comp[0].duration = duration;
+               comp[0].start = t0 ;
+               comp[0].duration = duration ;
+	       comp[0].ramptime[0]=ramp ;
                break;
             }
          // sweep
@@ -146,11 +156,12 @@ namespace diag {
             {
                long	flag = (waveform == (AWG_WaveType) 10) ? 
                   0 : AWG_SWEEP_LOG;
-               if (awgSweepComponents (t0, duration, freq - frange, freq + 
+               if (awgSweepComponents (t0 , duration , freq - frange, freq + 
                                     frange, ampl - arange, ampl + arange,
                                     flag, comp, &cnum) < 0) {
                   return false;
                }
+	       comp[0].ramptime[0]=ramp ;
                break;
             }
          case awgNone:
@@ -168,10 +179,19 @@ namespace diag {
          ((cnum > 1) && !awgIsValidComponent (comp + 1))) {
          return false;
       }
+      // fill in ramp parameters (amplitude phase-in only)
+      if (ramp > 0) {
+	 comp[0].ramptype = RAMP_TYPE (AWG_PHASING_QUADRATIC, 
+			      AWG_PHASING_QUADRATIC, 
+			      AWG_PHASING_LINEAR);
+	 if (my_debug) cerr << "  calSignal() - Setting ramptype to " << comp[0].ramptype << endl ;
+      }
+
       // add waveforms
       for (int i = 0; i < cnum ; i++) {
          signals.push_back (comp[i]);
       }
+      if (my_debug) cerr << "  calcSignal() return true" << endl ;
       return true;
    }
 
@@ -181,6 +201,7 @@ namespace diag {
    {
       AWG_Component		awg;
    
+      if (my_debug) cerr << "stdtest::stimulus::calcSineSignal(t0="<<t0<<", duration="<<duration<<", ramp="<< ramp << ")"<<endl;
       // check waveform 
       if (waveform != awgSine) {
          return false;
@@ -209,11 +230,13 @@ namespace diag {
       if (signals.empty()) {
          // first signal includes amplitide ramp up
          awg.ramptime[0] = ramp;
+	 if (my_debug) cerr << "  signals.empty(), setting awg.ramptime[0] = "<<ramp<<endl ;
          // set phase to 0 at start
          if (duration >= 0) awg.par[2] = 0; 
          phas = fmod (TWO_PI * freq * (double) ramp / __ONESEC + awg.par[2], TWO_PI);
       }
       else {
+	 if (my_debug) cerr << "  Add ramp to previous signal" << endl ;
          // add ramp to previous signal!
          // patch duration of previous signal
          signals.back().duration = awg.start - signals.back().start;
@@ -233,6 +256,7 @@ namespace diag {
       }
    
       signals.push_back (awg);
+      if (my_debug) cerr << "    awg.ramptime[0] = "<<awg.ramptime[0]<<", ramptime[1] = "<<awg.ramptime[1] << endl ;
       return true;
    }
 
@@ -308,6 +332,7 @@ namespace diag {
       semlock		lockit (mux);
       bool		err = false;
    
+      if (my_debug) cerr << "stdtest::readParam()" << endl ;
       // check storage
       if (storage == 0) {
          errmsg << "No diagnostics parameters" << endl;
@@ -357,6 +382,7 @@ namespace diag {
                           stAverages, averages)) {
          averages = 1;
       }
+      if (my_debug) cerr << "stdtest::readParam() return " << (err ? "false" : "true") << endl ;
       return !err;
    }
 
@@ -367,6 +393,7 @@ namespace diag {
       semlock		lockit (mux);
       bool		err = false;
    
+      if (my_debug) cerr << "stdtest::readMeasParam()" << endl ;
       // make sure max index is at least one
       int	max = (maxIndex > 1) ? maxIndex : 1;
    
@@ -379,6 +406,11 @@ namespace diag {
             diagObjectName::makeName (stMeasurementChannel, i);
          chnactive = (max == 1) ? string (stMeasurementChannelActive) :
             diagObjectName::makeName (stMeasurementChannelActive, i);
+#ifdef TEST_RATE
+	 string			chnrate ;
+	 chnrate = (max == 1) ? string (stMeasurementChannelRate) :
+	    diagObjectName::makeName (stMeasurementChannelRate, i) ;
+#endif
          // read channel name
          if (!test->getParam (*storage->Test, chnname, measchn.name) ||
             (measchn.name.empty())) {
@@ -391,8 +423,21 @@ namespace diag {
                continue;
             }
          }
+#ifdef TEST_RATE
+	 int rate = 0;
+	 if (test->getParam (*storage->Test, chnrate, rate)) {
+	    if (!rate) {
+	       continue ;
+	    }
+	    if (my_debug) cerr << "  rate for " << measchn.name << " is " << rate << endl ;
+	 }
+#endif
          // now get channel info
+#ifdef TEST_RATE
+         if (!dataMgr->channelInfo (measchn.name, measchn.info, rate)  ||
+#else
          if (!dataMgr->channelInfo (measchn.name, measchn.info)  ||
+#endif
             (measchn.info.dataRate <= 0)) {
             errmsg << "Invalid channel name (" << measchn.name << 
                ")" << endl;
@@ -405,6 +450,7 @@ namespace diag {
          errmsg << "No measurement channel defined" << endl;
          err = true;
       }
+      if (my_debug) cerr << "stdtest::readMeasParam() return " << (err ? "false" : "true") << endl ;
       return !err;
    }
 
@@ -419,6 +465,7 @@ namespace diag {
       // make sure max index is at least one
       int	max = (maxIndex > 1) ? maxIndex : 1;
    
+      if (my_debug) cerr << "stdtest::readStimuliParam()" << endl ;
       // read stimuli channel
       for (int i = 0; i < max; i++) {
          stimulus		stim;
@@ -565,6 +612,7 @@ namespace diag {
          }
          stimuli.push_back (stim);
       }
+      if (my_debug) cerr << "stdtest::readStimuliParam() return " << (err ? "false" : "true") << endl ;
       return !err;
    }
 
@@ -575,9 +623,12 @@ namespace diag {
       bool		err = false;
       int		count; 		// in use count
    
+      if (my_debug) cerr << "stdtest::subscribeChannels()" << endl ;
       // subscribe excitation channels
+      if (my_debug) cerr << "  subscribeChannels() - subscribe excitation channels" << endl ;
       for (stimuluslist::iterator iter = stimuli.begin();
           iter != stimuli.end(); iter++) {
+	 if (my_debug) cerr << "  subscribeChannels() - calling dataMgr->add(" << iter->name << ",...)" << endl ;
          if (!testExc->add (iter->name)) {
             errmsg << "Invalid excitation channel (" << iter->name <<
                ")" << endl;
@@ -586,9 +637,11 @@ namespace diag {
       }
    
       // subscribe measurement channels; read back channels first
+      if (my_debug) cerr << "  subscribeChannels() - subscribe read back channels" << endl ;
       for (stimuluslist::iterator iter = stimuli.begin();
           iter != stimuli.end(); iter++) {
          if (iter->isReadback) {
+	    if (my_debug) cerr << "  subscribeChannels() - calling dataMgr->add(" << iter->readback << ",...)" << endl ;
             if (!dataMgr->add (iter->readback, &count)) {
                errmsg << "Invalid measurement channel (" << 
                   iter->readback << ")" << endl;
@@ -597,15 +650,18 @@ namespace diag {
             iter->duplicate = (count > 1);
          }
       }
+      if (my_debug) cerr << "  subscribeChannels() - subscribe measurement channels" << endl ;
       for (measlist::iterator iter = meas.begin();
           iter != meas.end(); iter++) {
-         if (!dataMgr->add (iter->name, &count)) {
+	 if (my_debug) cerr << "  subscribeChannels() - calling dataMgr->add(" << iter->name << ",..., dataRate = " << iter->info.dataRate << ")" << endl ;
+         if (!dataMgr->add (iter->name, &count, iter->info.dataRate)) {
             errmsg << "Invalid measurement channel (" << 
                iter->name << ")" << endl;
             err = true;
          }
          iter->duplicate = (count > 1);
       }
+      if (my_debug) cerr << "stdtest::subscribeChannels() return " << (err ? "false" : "true") << endl ;
    
       return !err;
    }
@@ -615,6 +671,7 @@ namespace diag {
    {
       semlock		lockit (mux);
    
+      if (my_debug) cerr << "stdtest::delMeasurements()" << endl ;
       // clears excitation signals & measurement partitions
       for (stimuluslist::iterator iter = stimuli.begin();
           iter != stimuli.end(); iter++) {
@@ -631,6 +688,7 @@ namespace diag {
       // clears synchronization points
       syncqueue.clear();
    
+      if (my_debug) cerr << "stdtest::delMeasurements() return true" << endl ;
       return true;
    }
 
@@ -640,6 +698,7 @@ namespace diag {
    {
       // highest sampling rate
       int	exp;
+      if (my_debug) cerr << "stdtest::samplingFrequencies()" << endl ;
       frexp (2.0 * fMax - 1E-8, &exp);
       fSampleMax = ldexp (1.0, exp);
    
@@ -658,6 +717,7 @@ namespace diag {
             fSampleMin = iter->info.dataRate;
          }
       }
+      if (my_debug) cerr << "stdtest::samplingFrequencies() return" << endl ;
    }
 
 
@@ -666,6 +726,7 @@ namespace diag {
       double fz = 0;
       bool fzinit = false;
    
+      if (my_debug) cerr << "stdtest::heterodyneFrequency()" << endl ;
       for (stimuluslist::iterator iter = stimuli.begin();
           iter != stimuli.end(); iter++) {
          if (iter->isReadback) {
@@ -711,6 +772,7 @@ namespace diag {
          }
       }
       fZoom = fz;
+      if (my_debug) cerr << "stdtest::heterodyneFrequency() return true" << endl ;
       return true;
    }
 
@@ -719,6 +781,13 @@ namespace diag {
    {
       semlock		lockit (mux);
    
+      if (my_debug) {
+	 cerr << "stdtest::calcTimeGrid( fMax = " << fMax ;
+	 if (t0 != 0)
+	    cerr << ", t0 = " << *t0 << ")" << endl ;
+	 else
+	    cerr << ", NULL)" << endl ;
+      }
       // start with 1/16384 sec.
       //double tGrid = 1.0 / sin (2);
       double tGrid = ldexp (1.0, -14);
@@ -749,6 +818,12 @@ namespace diag {
          *t0 = fineAdjustForSampling (*t0 + (tainsec_t ) 
                               (tGrid/2 * 1E9 - 10000), tGrid);
       }
+      if (my_debug) {
+	 cerr << "stdtest::calcTimeGrid() return" ;
+	 if (t0)
+	    cerr << ", t0 modified to " << *t0 ;
+	 cerr << ", tGrid = " << tGrid << endl ;
+      }
       return tGrid;
    }
 
@@ -758,6 +833,7 @@ namespace diag {
    {
       semlock		lockit (mux);
    
+      if (my_debug) cerr << "stdtest::addMeasurements(t0 = " << t0 << ", measPoint = " << measPoint << ")" << endl ;
       // calculate measurement times
       if (!calcTimes (errmsg, t0)) {
          return false;
@@ -779,6 +855,7 @@ namespace diag {
          return false;
       }
    
+      if (my_debug) cerr << "stdtest::addMeasurements() return true" << endl ;
       return true;
    }
 
@@ -787,6 +864,7 @@ namespace diag {
    {
       semlock		lockit (mux);
    
+      if (my_debug) cerr << "stdtest::startMeasurements()" << endl ;
       // start excitation signals
       for (stimuluslist::iterator iter = stimuli.begin();
           iter != stimuli.end(); iter++) {
@@ -829,6 +907,7 @@ namespace diag {
          }
       }
    
+      if (my_debug) cerr << "stdtest::startMeasurements() return true" << endl ;
       return true;
    }
 
@@ -837,6 +916,7 @@ namespace diag {
    {
       semlock		lockit (mux);
    
+      if (my_debug) cerr << "stdtest::stopMeasurements( firstIndex = " << firstIndex << ")" << endl ;
       int num = (avrgsize < 0) ? intervals.size() - firstIndex : avrgsize;
    
       // stop rtdd: stimulus readbacks first
@@ -874,6 +954,7 @@ namespace diag {
       intervals.clear();
       syncqueue.clear();
    
+      if (my_debug) cerr << "stdtest::stopMeasurements() return true" << endl ;
       return true;
    }
 
@@ -882,6 +963,7 @@ namespace diag {
    {
       semlock		lockit (mux);
    
+      if (my_debug) cerr << "stdtest::begin()" << endl ;
       // make sure object is cleaned up
       syncqueue.clear();
       meas.clear();
@@ -899,6 +981,7 @@ namespace diag {
          return false;
       }
    
+      if (my_debug) cerr << "stdtest::begin() return" << endl ;
       return true;
    }
 
@@ -907,6 +990,7 @@ namespace diag {
    {
       semlock		lockit (mux);
    
+      if (my_debug) cerr << "stdtest::end()" << endl ;
       // unsubscribe excitation signals & rtdd channels
       for (stimuluslist::iterator iter = stimuli.begin();
           iter != stimuli.end(); iter++) {
@@ -925,6 +1009,7 @@ namespace diag {
       meas.clear();
       stimuli.clear();
       intervals.clear();
+      if (my_debug) cerr << "stdtest::end() return true" << endl ;
       return true;
    }
 
@@ -934,6 +1019,7 @@ namespace diag {
    {
       semlock		lockit (mux);
    
+      if (my_debug) cerr << "stdtest::setup( starttime = " << starttime << ")" << endl ;
       // make sure partition and excitation lists are empty
       if (!delMeasurements (errmsg)) {
          return false;
@@ -952,7 +1038,9 @@ namespace diag {
       // get first synchronization point and return
       callbackarg		arg;
       arg.measPeriod = -1;
-      return getNextSyncPoint (arg, sync);
+      bool rc = getNextSyncPoint (arg, sync);
+      if (my_debug) cerr << "stdtest::setup() return " << (rc ? "true" : "false") << endl ;
+      return rc ;
    }
 
 
@@ -962,6 +1050,7 @@ namespace diag {
    {
       semlock		lockit (mux);
    
+      if (my_debug) cerr << "stdtest::addMeasPartitions()" << endl ;
       // calculate partitions for stimulus
       for (stimuluslist::iterator iter = stimuli.begin();
           iter != stimuli.end(); iter++) {
@@ -1011,6 +1100,7 @@ namespace diag {
          iter->partitions.push_back (part);
          //cerr << "add " << pname << " @ " << ival.t0() << endl;
       }
+      if (my_debug) cerr << "stdtest::addMeasPartitions() return true" << endl ;
       return true;
    }
 
@@ -1020,6 +1110,7 @@ namespace diag {
    {
       semlock		lockit (mux);
    
+      if (my_debug) cerr << "stdtest::addSyncPoint( measPeriod = " << measPeriod << ", measPoint = " << measPoint << ")" << endl ;
       syncpointer	sync;
       callbackarg	arg;
       arg.measPoint = measPoint;
@@ -1033,6 +1124,7 @@ namespace diag {
       }
       syncqueue.push_back (sync);
       timeAhead = ival.t0();
+      if (my_debug) cerr << "stdtest::addSyncPoint() return true" << endl ;
       return true;
    }
 
@@ -1045,6 +1137,7 @@ namespace diag {
 
    bool stdtest::prepareNextPoint (const callbackarg& id)
    {
+      if (my_debug) cerr << "stdtest::prepareNextPoint()" << endl ;
       if (avrgsize <= 0) {
          return true;
       }
@@ -1094,6 +1187,7 @@ namespace diag {
       // add new point
       int i = id.measPeriod + syncqueue.size() + 1;
       if (!newMeasPoint (i, id.measPoint)) {
+	 if (my_debug) cerr << "stdtest::prepareNextPoint() return true, line " << __LINE__ << endl ;
          return true;
       }
    
@@ -1106,6 +1200,7 @@ namespace diag {
          dataChannel::partitionlist 	p;
          p.push_back (iter->partitions.back());
          if (!dataMgr->add (iter->readback, p)) {
+	    if (my_debug) cerr << "stdtest::prepareNextPoint() return false, line "<< __LINE__  << endl ;
             return false;
          }
       }
@@ -1119,10 +1214,12 @@ namespace diag {
          dataChannel::partitionlist 	p;
          p.push_back (iter->partitions.back());
          if (!dataMgr->add (iter->name, p)) {
+	    if (my_debug) cerr << "stdtest::prepareNextPoint() return false, line " << __LINE__ << endl ;
             return false;
          }
       }
    
+      if (my_debug) cerr << "stdtest::prepareNextPoint() return true" << endl ;
       return true;
    }
 
@@ -1150,6 +1247,7 @@ namespace diag {
       bool		dataerror = false;
       semlock		lockit (mux);
    
+      if (my_debug) cerr << "stdtest::syncAction()" << endl;
       // analysis disabled: delete data object
       int num = id.measPoint * averages + id.measPeriod;
       if (my_debug) cerr << "sync action (" << num << ") of " << myname << endl;
@@ -1279,15 +1377,18 @@ namespace diag {
       // return next sync point
       bool ret = getNextSyncPoint (id, sync);
    
+      if (my_debug) cerr << "stdtest::syncAction() return " << (ret ? "true" : "false")  << endl;
       return ret;
    }
 
 
    bool stdtest::syncPause (const callbackarg& id, syncpointer& sync)
    { 
+      if (my_debug) cerr << "stdtest::syncPause()" << endl ;
       // continue running; disable analysis
       doAnalysis = false;
       sync.reset();
+      if (my_debug) cerr << "stdtest::syncPause() return" << endl ;
       return true;
    }
 
@@ -1295,8 +1396,10 @@ namespace diag {
    bool stdtest::syncResume (const callbackarg& id, syncpointer& sync)
    {
       // resume analysis
+      if (my_debug) cerr << "stdtest::syncResume() " << endl ;
       doAnalysis = true;
       sync.reset();
+      if (my_debug) cerr << "stdtest::syncResume() return" << endl ;
       return true;
    }
 
@@ -1305,9 +1408,11 @@ namespace diag {
    {
       semlock		lockit (mux);
    
+      if (my_debug) cerr << "stdtest::syncAbort()" << endl ;
       // stop measurements
       int num = id. measPoint * averages + id.measPeriod;
       stopMeasurements (num);
+      if (my_debug) cerr << "stdtest::syncAbort() return" << endl ;
    
       return true;
    }
@@ -1317,12 +1422,15 @@ namespace diag {
    {
       semlock		lockit (mux);
    
+      if (my_debug) cerr << "stdtest::syncTimeAhead()" << endl ;
       // get next sync point
       if (timeAhead && !syncqueue.empty()) {
          ahead = timeAhead;
+	 if (my_debug) cerr << "stdtest::syncTimeAhead() return true" << endl;
          return true;
       }
       else {
+	 if (my_debug) cerr << "stdtest::syncTimeAhead() return false" << endl;
          return false;
       }
    }
@@ -1430,6 +1538,7 @@ extern "C"
       chnrec	rec;		// record 
       chnlist	chns;		// channel list
    
+      if (my_debug) cerr << "stdtest::callChannelAnalysis()" << endl ;
       mux.lock();
       // get names of stimuli channels
       if (inclStim != stimNone) {
@@ -1728,12 +1837,7 @@ extern "C"
 #endif
       }
    
-      if (my_debug)
-      {
-	 char str[128] ;
-	 sprintf(str, "callChannelAnalysis done, returning.\n") ;
-	 cerr << str ;
-      }
+      if (my_debug) cerr << "stdtest::callChannelAnalysis() return" << endl ;
       return ret;
    }
 
