@@ -68,6 +68,8 @@ namespace diag {
 #define _MAX_NDS_DELAY		64.0 // 64 seconds
 #define _NDS_DELAY		1
 
+    static const int my_debug = 0 ;
+
     static const char	taskNds2Name[] = "tNDS2";
     static const int	taskNds2Priority = 0;
     static const char	taskCleanupName[] = "tNDS2cleanup";
@@ -193,41 +195,36 @@ namespace diag {
 	chnorder2() {}
 	bool operator() (const sends::DAQDChannel& c1, 
 			 const sends::DAQDChannel& c2) const {
+#if 0
 	    return strcasecmp (c1.mName.c_str(), c2.mName.c_str()) < 0;
+#else
+	    bool rc = strcasecmp (c1.mName.c_str(), c2.mName.c_str()) < 0;
+	    if (!rc) {
+	       if (strcasecmp (c1.mName.c_str(), c2.mName.c_str()) == 0) {
+		  // Compare the rates.
+		  if (c1.mRate < c2.mRate) {
+		     if (my_debug) cerr << "chnorder2() - " << c1.mName << " (" << c1.mRate << ") is less than " << c2.mName << " (" << c2.mRate << ")" << endl ;
+		     rc = 1 ;
+		  }
+	       }
+	    }
+	    return rc ;
+#endif
 	}
     };
 
-    bool nds2Manager::connect (const char* server, int port, bool usernds)
+    bool nds2Manager::connect (const char* server, int port, bool usernds,
+			       unsigned long epoch_start, unsigned long epoch_stop)
     {
 	int		status;
    
 	// get NDS2 parameters
 	if (server == 0) {
-	    // dynamic configuration
-#if defined (_CONFIG_DYNAMIC)
-	    const char* const* cinfo;	// configuration info
-	    confinfo_t	crec;		// conf. info record
-	    for (cinfo = getConfInfo (0, 0); cinfo && *cinfo; cinfo++) {
-		if ((parseConfInfo (*cinfo, &crec) == 0) &&
-		    (gds_strcasecmp (crec.interface, 
-				     CONFIG_SERVICE_NDS) == 0) &&
-		    (crec.ifo == -1) && (crec.progver == -1)) {
-		    daqServer = crec.host;
-		    daqPort = crec.port_prognum;
-		}
-	    }
-#else
-	    // from parameter file
-	    char tempserver[256];
-	    strncpy(tempserver, DAQD_SERVER, sizeof(tempserver));
-	    loadStringParam (PRM_FILE, PRM_SECTION, PRM_SERVERNAME, tempserver);
-	    daqServer = tempserver;
-	    daqPort = DAQD_PORT;
-	    loadIntParam (PRM_FILE, PRM_SECTION, PRM_SERVERPORT, &daqPort);
-#endif
-	    if (daqPort <= 0) {
-		daqPort = DAQD_PORT;
-	    }
+	    // It's useless to get a configuration hoping that the nds2 server 
+	    // responds to a UDP broadcast, it won't.  For aLIGO, even nds1
+	    // servers don't respond to the UDP broadcast.  So get rid of this
+	    // mess and return false if the server wasn't specified.
+	    return false ;
 	}
 	else {
 	    // user specified
@@ -245,13 +242,26 @@ namespace diag {
 	// get channel list if user NDS
 	userNDS = usernds;
 	if (usernds) {
+	    // Set the epoch if the start != stop.
+	    if (epoch_start != epoch_stop) {
+	       nds.SetEpoch(epoch_start, epoch_stop) ;
+	    }
+
 	    // The second argument to Available is a GPS time for the 
 	    // channel list. 
-	    cerr << "nds2Manager::connect() - get channel list" << endl ;
-	    nds.Available (cRaw, 0, userChnList);
-	    cerr << "nds2Manager::connect() - sort channel list" << endl ;
+	    // If we send the epoch start time as the GPS time to Available, we get the
+	    // channel list at the time of the start of the epoch.  This won't be the
+	    // complete list for the epoch, only the channels that existed at the start
+	    // of the epoch.  Instead, pass a gps time of 0.  This forces the server to 
+	    // give us a complete list of channels for the entire epoch.
+	    // Also see readChnDAQServer() in Diag_Services/fantom/sendsio.cc
+	    if (my_debug) cerr << "nds2Manager::connect() - get channel list" << endl ;
+	    nds.Available (cRaw, 0, userChnList); // Call to DAQC_api::Available(), which NDS2Socket inherits.
+
+	    if (my_debug) cerr << "nds2Manager::connect() - sort channel list" << endl ;
 	    sort (userChnList.begin(), userChnList.end(), chnorder2());
-	    cerr << "nds2Manager::connect() - done" << endl ;
+	    
+	    if (my_debug) cerr << "nds2Manager::connect() - done" << endl ;
 	}
    
 	return true;
@@ -358,14 +368,12 @@ namespace diag {
 	cleartime = 0;
    
 	// setup channels
-	cerr << "setup channels for NDS2" << endl;
+	if (my_debug) cerr << "nds2Manager::set() - setup channels for NDS2" << endl;
 	for (channellist::iterator iter = channels.begin();
 	     iter != channels.end(); iter++) {
 	    // add channel to list
-	    cerr << "Add channel: " << iter->getChnName() << " rate: " 
-		 << iter->getDatarate() << endl;
-	    nds.AddChannel (iter->getChnName(), cUnknown,
-			    iter->getDatarate());
+	    //cerr << "Add channel: " << iter->getChnName() << " rate: " << iter->getDatarate() << endl;
+	    nds.AddChannel (iter->getChnName(), cRaw, iter->getDatarate());
 	    if (iter->isSet()) {
 		continue;
 	    }
@@ -395,24 +403,31 @@ namespace diag {
 	return true;
     }
 
-    bool nds2Manager::channelInfo(const string& name, gdsChnInfo_t& info) const
+    bool nds2Manager::channelInfo(const string& name, gdsChnInfo_t& info, int rate) const
     {
+	if (my_debug) cerr << "nds2Manager::channelInfo(" << name << ", ..., " << rate << ")" << endl;
 	if (!userNDS) {
 	    cout << "Get channel info from channelHandler." << endl;
-	    return channelHandler::channelInfo (name, info);
+	    return channelHandler::channelInfo (name, info, rate);
 	}
 	else {
-	    cout << "Get nds2 channel info for " << name << endl;
+	    cout << "Get nds2 channel info for " << name << ", rate = " << rate << endl;
 	    // find channel
 	    DAQDChannel item;
 	    item.mName =name;
+	    if (rate > 0) {
+	       item.mRate = rate ;
+	    }
+	    // The lower_bound function uses chnorder2() to find the
+	    // first channel that matches the criteria that makes 
+	    // chnorder2() happy. 
 	    vector<DAQDChannel>::const_iterator chn =
 		lower_bound (userChnList.begin(), userChnList.end(),
 			     item, chnorder2());
 	    memset (&info, 0, sizeof (gdsChnInfo_t));
 	    if ((chn == userChnList.end()) ||
 		(strcasecmp (item.mName.c_str(), chn->mName.c_str()) != 0)) {
-		cout << "Nds2 has no channel info for " << name << endl;
+		cout << "NDS2 has no channel info for " << name << endl;
 		return false;
 	    }
 	    else {
@@ -428,6 +443,7 @@ namespace diag {
 		info.offset = chn->mOffset;
 		strncpy (info.unit, chn->mUnit.c_str(), sizeof (info.unit)-1);
 		info.unit[sizeof(info.unit)-1] = 0;
+		if (my_debug) cerr << "channelInfo() - found info, rate = " << info.dataRate << endl ;
 		return true;
 	    }
 	}
@@ -643,7 +659,7 @@ namespace diag {
 
 	// start net writer
 	RTmode = false;
-	cerr << "nds2 start old data" << endl;
+	if (my_debug) cerr << "nds2Manager::ndsStart() - nds2 start old data" << endl;
 	abort = false;
 	nds.setAbort (&abort);
 	if (!nds.isOpen() && (nds.open (daqServer, daqPort) != 0)) {
@@ -653,12 +669,14 @@ namespace diag {
 	}
 
 	online_req = false;
+	if (my_debug) cerr << "nds2Manager::ndsStart() - nds.RequestData(" << start << ", " << duration << ", ...)" << endl ;
 	if (nds.RequestData (start, duration, taskNds2OfflineTimeout) != 0) {     
 	    nds.RmChannel ("all");
 	    cerr << "nds2 error during data request" << endl;
 	    return false;
 	}
    
+	if (my_debug) cerr << "nds2Manager::ndsStart() - Create nds2 task thread" << endl ;
 	// create nds2 task
 	int		attr;	// task create attribute
 	attr = PTHREAD_CREATE_DETACHED;
