@@ -1,4 +1,4 @@
-VCSID("$(#)$Id: rtddinput.cc 7338 2015-04-14 23:24:47Z ed.maros@LIGO.ORG $");
+VCSID("$(#)$Id: rtddinput.cc 7574 2016-02-16 20:17:52Z james.batch@LIGO.ORG $");
 /* -*- mode: c++; c-basic-offset: 3; -*- */
 /*----------------------------------------------------------------------*/
 /*                                                         		*/
@@ -409,16 +409,19 @@ namespace diag {
 
 
    bool rtddManager::channelInfo (const string& name, 
-                     gdsChnInfo_t& info) const
+                     gdsChnInfo_t& info, int rate) const
    {
       if (!userNDS) {
-         return channelHandler::channelInfo (name, info);
+         return channelHandler::channelInfo (name, info, rate);
       }
       else {
          // find channel
          DAQDChannel item;
          strncpy (item.mName, name.c_str(), sizeof (item.mName) - 1);
          item.mName[sizeof(item.mName)-1] = 0;
+	 if (rate > 0) {
+	    item.mRate = rate ;
+	 }
          vector<DAQDChannel>::const_iterator chn =
             lower_bound (userChnList.begin(), userChnList.end(),
                         item, chnorder());
