@@ -28,6 +28,7 @@ namespace dfm {
    const char* const kFuncServerName = "Function callback";
 
 
+   static int my_debug=0 ;
 
 //////////////////////////////////////////////////////////////////////////
 //                                                                      //
@@ -448,12 +449,21 @@ namespace dfm {
 //______________________________________________________________________________
    UDNInfo* dataserver::get (const UDN& udn) const
    {
+      if (my_debug) cerr << "dataserver::get(" << udn << ")" << endl ;
       //((dataserver*)this)->updateUDNs (false);
+      if (my_debug){
+	 cerr << "  There are " << fUDN.size() << " entries in the map." << endl ;
+	 cerr << "  Candidates are: "<< endl ;
+	 for (const_UDNiter i = fUDN.begin(); i != fUDN.end(); ++i)
+	    cerr << "     " << i->first << endl ;
+      }
       const_UDNiter f = fUDN.find (udn);
       if (f != fUDN.end()) {
+	 if (my_debug) cerr << "dataserver::get() succeeded, return ptr to UDNInfo." << endl ;
          return (UDNInfo*) &(f->second);
       }
       else {
+	 if (my_debug) cerr << "dataserver::get() failed, return 0." << endl ;
          return 0;
       }
    }
@@ -461,24 +471,31 @@ namespace dfm {
 //______________________________________________________________________________
    UDNInfo* dataserver::insert (const UDN& udn)
    {
+      if (my_debug) cerr << "dataserver::insert(" << udn << ")" << endl ;
       return insert (udn, UDNInfo());
    }
 
 //______________________________________________________________________________
    UDNInfo* dataserver::insert (const UDN& udn, const UDNInfo& info)
    {
+      if (my_debug) cerr << "dataserver::insert(" << udn << ", info)" << endl ;
+      if (my_debug) cerr << "  info.channels.size() is " << info.channels().size() << endl ;
       UDNiter f = fUDN.find (udn);
       if (f != fUDN.end()) {
          f->second = info;
+	 if (my_debug) cerr << "  fUDN.find(udn) succeeded, return ptr to info." << endl ;
          return &f->second;
       }
       else {
+	 if (my_debug) cerr << "  fUDN.find(udn) failed." << endl ;
          pair <UDNiter, bool> f = fUDN.insert (
                               UDNList::value_type (udn, info));
          if (f.second) {
+	    if (my_debug) cerr << "    fUDN.insert() succeeded. return ptr." << endl ;
             return &f.first->second;
          }
          else {
+	    if (my_debug) cerr << "    fUDN.insert() failed, return 0." << endl ;
             return 0;
          }
       }
@@ -487,18 +504,21 @@ namespace dfm {
 //______________________________________________________________________________
    void dataserver::erase (const UDN& udn)
    {
+      if (my_debug) cerr << "dataserver::erase(" << udn << ")" << endl ;
       fUDN.erase (udn);
    }
 
 //______________________________________________________________________________
    void dataserver::clear()
    {
+      if (my_debug) cerr << "dataserver::clear() - clears entire map." << endl ;
       fUDN.clear ();
    }
 
 //______________________________________________________________________________
    int dataserver::size() const
    {
+      if (my_debug) cerr << "dataserver::size() returns " << fUDN.size() << endl ;
       return fUDN.size();
    }
 
@@ -512,6 +532,7 @@ namespace dfm {
    bool dataserver::login (const UDN& udn,
                      const char* uname, const char* pword)
    {
+      if (my_debug) cerr << "dataserver::login(" << udn << ", " << uname << ", " << pword << ")" << endl ;
       dfmapi* dfm = createDFMapi (fType);
       if ((dfm ==  0) || !*dfm) {
          delete dfm;
@@ -532,7 +553,9 @@ namespace dfm {
 //______________________________________________________________________________
    bool dataserver::updateUDNs (bool force)
    {
+      if (my_debug) cerr << "dataserver::updateUDNs( force = " << (force ? "TRUE" : "FALSE") << ")" << endl ; 
       if (!force && fUpdate) {
+	 if (my_debug) cerr << "  fUpdate is true, return true." << endl ;
          return true;
       }
       if (force) clear();
@@ -541,32 +564,43 @@ namespace dfm {
          delete dfm;
          return false;
       }
+      if (my_debug) cerr << "  dataserver::updateUDNs() caling dfmapi::open(" << fAddr << ")" << endl ;
       if (!dfm->open (fAddr)) {
          delete dfm;
+	 if (my_debug) cerr << "    failed to open " << fAddr << endl ;
          return false;
       }
+      if (my_debug) cerr << "  dataserver::updateUDNs() calling dfmapi::cachedUDNs()" << endl ;
       if (!dfm->cachedUDNs (dataservername (fType, fAddr), 
                            fUDN, force)) {
          delete dfm;
+	 if (my_debug) cerr << "    cachedUDNs() failed, return false" << endl ;
          return false;
       }
       delete dfm;
       fUpdate = true;
+      if (my_debug) cerr << "dataServer::updateUDNs() return true, fUpdate = true" << endl ;
       return true;
    }
 
 //______________________________________________________________________________
    bool dataserver::lookupUDN (const UDN& udn, bool force)
    {
+      if (my_debug) cerr << "dataserver::lookupUDN(" << udn << (force ? ", force=TRUE)" : ", force=FALSE)") << endl ;
       if (!fUpdate) {
+	 if (my_debug) cerr << "lookupUDN() - calling updateUDNs(" << (force ? "TRUE)" : "FALSE)") << endl ;
          if (!updateUDNs (force)) {
+	    if (my_debug) cerr << "  lookupUDN() - updateUDNs() returned false,  return false." << endl ;
             return false;
          }
+	 if (my_debug) cerr << "lookupUDN() - updateUDNs() returned true" << endl ;
       }
       UDNInfo* uinfo = get (udn);
       if (!force && uinfo && (uinfo->getType() != fantom::NONE)) {
+	 if (my_debug) cerr << "lookupUDN() - get(" << udn << ") returned, return true." << endl ;
          return true;
       }
+      //cerr << "lookupUDN() - uinfo returned by get() has " << uinfo->channels().size() << " channels" << endl ;
       cerr << "LOOKUP UDN = " << (const char*)udn << endl;
       dfmapi* dfm = createDFMapi (fType);
       if ((dfm ==  0) || !*dfm) {
@@ -574,26 +608,36 @@ namespace dfm {
          delete dfm;
          return false;
       }
+      if (my_debug) cerr << "lookupUDN() - calling dfm->open(" << fAddr << ")" << endl ;
       if (!dfm->open (fAddr)) {
 	 cerr << "Unable to open " << udn << endl;
          delete dfm;
          return false;
       }
       UDNInfo info;
+      if (my_debug) cerr << "lookupUDN() - calling dfm->cachedUDNInfo(" << udn << ", info, " << (force ? ", force=TRUE)" : ", force=FALSE)") << endl ;
       if (!dfm->cachedUDNInfo (udn, info, force)) {
 	 cerr << "no cached info " << udn << endl;
          delete dfm;
          return false;
       }
+      if (my_debug) cerr << "lookupUDN() - cachedUDNInfo() returned, number of channels is " 
+	 << info.channels().size()  << endl ;
+
       delete dfm;
-      return insert (udn, info) != 0;
+      dfm::UDNInfo *rc = insert (udn, info) ;
+      if (my_debug) cerr << "lookupUDN() - after insert, number of channels is " << info.channels().size()  << endl ;
+      if (rc && my_debug) cerr << "   Return of insert: number of channels is " << rc->channels().size() << endl ;
+      return rc != 0;
    }
 
 //______________________________________________________________________________
    bool dataserver::lookupUDNs (bool force)
    {
+      if (my_debug) cerr << "dataserver::lookupUDNs()" << endl ;
       if (!fUpdate) {
          if (!updateUDNs (force)) {
+	    if (my_debug) cerr << "  dataserver::lookupUDNs() - updateUDNs() returned false" << endl ;
             return false;
          }
       }
@@ -602,17 +646,22 @@ namespace dfm {
          delete dfm;
          return false;
       }
+      if (my_debug) cerr << " dataserver::lookupUDNs() - calling dfm->open(" << fAddr << ")" << endl ;
       if (!dfm->open (fAddr)) {
          delete dfm;
+	 if (my_debug) cerr << "  open failed." << endl ;
          return false;
       }
       for (UDNiter i = begin(); i != end(); ++i) {
+	 if (my_debug) cerr << " dataserver::lookupUDNs(), dfm->cachedUDNInfo(" << i->first << ", ...) " << endl ;
          if (!dfm->cachedUDNInfo (i->first, i->second, force)) {
             delete dfm;
+	    if (my_debug) cerr << "  cachedUDNInfo failed, return false." << endl ;
             return false;
          }
       }
       delete dfm;
+      if (my_debug) cerr << "dataserver::lookupUDNs() return true" << endl ;
       return true;
    }
 
@@ -1307,6 +1356,7 @@ namespace dfm {
    bool dataaccess::insert (const std::string& sname, 
                      const dataserver& ds)
    {
+      if (my_debug) cerr << "dataaccess::insert(" << sname << ", ...)" << endl ;
       if (fSupport.count (ds.getType()) <= 0) {
          fErrMsg = "Data server type not supported.";
          return false;
