@@ -16,14 +16,25 @@ namespace fantom {
    using namespace framefast;
    using namespace sends;
 
+   static int my_debug = 0 ;
+#define TEST_RATE 1
+
 //////////////////////////////////////////////////////////////////////////
 //                                                                      //
 // channel list chache						        //
 //                                                                      //
 //////////////////////////////////////////////////////////////////////////
-   typedef std::pair <std::string, int> ndsserver;
+
+//   typedef std::pair <std::string, int> ndsserver;
+#if TEST_RATE
+   // Replace the channel name with a name/rate pair, since names aren't
+   // enough to define a channel for NDS2. 
+   typedef std::pair<std::string, int> ChannelNameRate ;
+   typedef std::map <ChannelNameRate, DAQDChannel> DAQDChannelList ;
+#else
    typedef std::map <std::string, DAQDChannel> DAQDChannelList;
-   typedef std::map <ndsserver, DAQDChannelList> ndschannelcache;
+#endif
+   typedef std::map <std::string, DAQDChannelList> ndschannelcache;
    static ndschannelcache gNdsChnCache; // global
 
    struct DAQTimeList {
@@ -42,7 +53,8 @@ namespace fantom {
 //                                                                      //
 //////////////////////////////////////////////////////////////////////////
    static bool readChnDAQServer (const char* server, int port,
-				 frametype ftyp, DAQDChannelList& chns);
+				 frametype ftyp, DAQDChannelList& chns,
+				 unsigned long start, unsigned long end);
    static bool readTimeDAQServer (const char* server, int port,
 				  DAQTimeList& time);
 
@@ -61,6 +73,26 @@ namespace fantom {
       return s;
    }
 
+   static string
+   makeServerName(const char* server, int port, frametype utype,
+			   unsigned long start, unsigned long stop)
+   {
+      std::ostringstream ss ;
+      ss << server << ":" << int(port) ;
+      switch (utype) {
+	 default:
+	 case FF: ss << "/frames" ;
+	    break ;
+	 case STF: ss << "/trend" ;
+	    break ;
+	 case MTF: ss << "/minute-trend" ;
+	    break ;
+      }
+      ss << "?epoch_start=" << int(start) << "&epoch_end=" << int(stop) ;
+      return ss.str() ;
+   }
+
+
 
 //////////////////////////////////////////////////////////////////////////
 //                                                                      //
@@ -72,7 +104,7 @@ namespace fantom {
 //______________________________________________________________________________
    sends_support::sends_support (const char* servername, 
                      const char* conf) 
-   : fPort (kNDSPORT), fType (FF), fNDS (0)
+   : fPort (kNDSPORT), fType (FF), fNDS (0), fEpochStart(0), fEpochStop(0)
    {
       setServer (servername);
       setConf (conf);
@@ -91,21 +123,67 @@ namespace fantom {
       fAddr = "";
       fPort = kNDSPORT;
       fType = FF;
-      string::size_type pos = n.find ('/');
-      if (pos != string::npos) {
-         if (strcmp (n.c_str() + pos, "/trend") == 0) {
-            fType = STF;
-         }
-         else if (strcmp (n.c_str() + pos, "/minute-trend") == 0) {
-            fType = MTF;
-         }
-         n.erase (pos);
+
+      // Look for the epoch start, stop times.
+      unsigned long epoch_start = 0, epoch_stop = 0 ;
+      string s(servername) ;
+      string es("epoch_start=") ;
+      string ee("epoch_end=") ;
+      string ff("/frames") ;
+      string stf("/trend") ;
+      string mtf("/minute-trend") ;
+      string ps(":") ;
+      if (s.find(stf) != string::npos)
+      {
+	 fType = STF ;
       }
-      if ((pos = n.find (':')) != string::npos) {
-         fPort = atoi (n.c_str() + pos + 1);
-         n.erase (pos);
+      else if (s.find(mtf) != string::npos)
+      {
+	 fType = MTF ;
+      }
+      else if (s.find(ff) != string::npos)
+      {
+	 // This is also default?
+	 fType = FF ;
+      }
+      string::size_type es_pos = s.find(es) ;
+      string::size_type ee_pos = s.find(ee) ;
+      if (es_pos != string::npos && ee_pos != string::npos) 
+      {
+	 // Both start and stop were found, set the epoch.
+	 fEpochStart = atoi(s.c_str() + es_pos + es.length()) ;
+	 fEpochStop = atoi(s.c_str() + ee_pos + ee.length()) ;
+      }
+      // Look for the port number.
+      string::size_type ps_pos = s.find(ps) ;
+      if (ps_pos != string::npos)
+      {
+	 fPort = atoi(s.c_str() + ps_pos + ps.length()) ;
+      }
+      // The address should be the start of the string, until
+      // the port number or epoch names are found.
+      string::size_type name_end = s.find_first_of(":&?/") ;
+      // Note that name_end = npos works for the end value.
+      string addr = s.substr(0,name_end) ;
+      fAddr = addr.c_str() ;
+      // Debug stuff.
+      if (my_debug) {
+	 cerr << "sends_support::setServer(" << servername << ")" << endl ;
+	 cerr << "  fAddr = " << fAddr << endl ;
+	 cerr << "  fPort = " << fPort << endl ;
+	 cerr << "  fEpochStart = " << fEpochStart << endl ;
+	 cerr << "  fEpochStop = " << fEpochStop << endl ;
+	 switch (fType) {
+	    case STF: cerr << "  fType = STF" << endl ;
+	      break ;
+	    case MTF: cerr << "  fType = MTF" << endl ;
+	      break ;
+	    case FF: cerr << "  fType = FF" << endl ;
+	      break ;
+	    default: cerr << "  fType could not be determined!" << endl ;
+	      break ;
+	 }
       }
-      fAddr = trim (n.c_str());
    }
 
 //______________________________________________________________________________
@@ -125,17 +203,20 @@ namespace fantom {
          }
       }
       // get channel list
-      ndsserver ns (fAddr.c_str(), fPort);
+      string ns = makeServerName(fAddr.c_str(), fPort, fType, fEpochStart, fEpochStop) ;
       ndschannelcache::iterator l = gNdsChnCache.find (ns);
       if (l == gNdsChnCache.end()) {
          return false;
       }
       // Add normal channels to nds
       if (fType == FF) {
-         for (DAQDChannelList::const_iterator i = l->second.begin();
-             i != l->second.end(); ++i) {
-            if (!chns || chns->empty() || 
-		(chns->findMatch (i->first) != 0)) {
+         for (DAQDChannelList::const_iterator i = l->second.begin(); i != l->second.end(); ++i) {
+#ifdef TEST_RATE
+            if (!chns || chns->empty() || (chns->findMatch (i->second.mName) != 0)) 
+#else
+            if (!chns || chns->empty() || (chns->findMatch (i->first) != 0)) 
+#endif
+	    {
                //cerr << "add a channel " << i->second.mName << endl;
                fNDS->AddChannel (i->second);
             }
@@ -490,10 +571,12 @@ namespace fantom {
 
 //______________________________________________________________________________
    bool sends_support::getChannels (const char* server, int port,
-				    channellist& chns, frametype utype)
+				    channellist& chns, frametype utype, 
+				    unsigned long start, unsigned long stop)
    {
-      cerr << "get channels from " << server << " @ " << port << endl;
-      ndsserver ns (server, port);
+      if (my_debug) cerr << "sends_support::getChannels( " << server << ", " << port << ", ..., " << start <<", " << stop << ")" << endl;
+      //ndsserver ns (server, port);
+      string ns = makeServerName(server, port, utype, start, stop) ;
       DAQDChannelList ndschns;
       DAQDChannelList::const_iterator begin;
       DAQDChannelList::const_iterator end;
@@ -501,16 +584,17 @@ namespace fantom {
       //--------------------------------  get channel names (check cache first)
       ndschannelcache::iterator i = gNdsChnCache.find (ns);
       if (i != gNdsChnCache.end()) {
+	 if (my_debug) cerr << "  getChannels() - gNdsChnCache.find() succeeded." << endl ;
          begin = i->second.begin();
          end = i->second.end();
       }
 
       // get names from NDS2 server
-      else if (readChnDAQServer (server, port, utype, ndschns)) {
+      else if (readChnDAQServer (server, port, utype, ndschns, start, stop)) {
          gNdsChnCache.insert (ndschannelcache::value_type (ns, ndschns));
          begin = ndschns.begin();
          end = ndschns.end();
-         cerr << "load from server" << endl;
+         if (my_debug) cerr << "  getChannels() - loaded from server" << endl;
       }
       else {
          cerr << "load failed" << endl;
@@ -549,7 +633,9 @@ namespace fantom {
             }
          }
       }
-      SortChannels (chns);
+      if (my_debug) cerr << "  getChannels() - sort channel list." << endl ;
+      SortChannels (chns, true);
+      if (my_debug) cerr << "  getChannels() return true." << endl ;
       return true;
    }
 
@@ -684,8 +770,10 @@ namespace fantom {
    //////////////////////////////////////////////////////////////////////////
    static bool 
    readChnDAQServer (const char* server, int port, frametype ftyp, 
-		     DAQDChannelList& chns) 
+		     DAQDChannelList& chns, unsigned long start,
+		     unsigned long stop) 
    {
+      if (my_debug) cerr << "sendsio.cc readChnDAQServer(server=" << server << ", port=" << port << "..., start=" << start << " stop=" << stop << ")" << endl ;
       NDS2Socket* daq = new NDS2Socket;
       if (!daq) {
 	 cerr << "Unable to construct NDS2Socket object" << endl;
@@ -717,18 +805,45 @@ namespace fantom {
 	 break;
       }
 
+      //-------------------------------  Set the epoch if start and stop are specified.
+      if (start != stop)
+      {
+	 if (my_debug) cerr << " readChnDaqServer() - calling SetEpoch(" << start << ", " << stop << ")" << endl ;
+	 daq->SetEpoch(start, stop) ;
+      }
+
       //--------------------------------  Get the channel list
+      // If we send the epoch start time as the GPS time to Available, we get the channel list at the
+      // time of the start of the epoch.  This won't be the complete list for the epoch, only the channels
+      // that existed at the start.  Instead, pass a gps time of 0.  This forces the server to give us a
+      // complete list of channels for the epoch.
       DAQC_api::chan_list c;
+      if (my_debug) cerr << " readChnDaqServer() - calling Available(ctyp, " << "0" << ", c) " << endl ;
+      // rc holds the number of channels available if >= 0, otherwise an error code.
       rc = daq->Available (ctyp, 0, c);
       if (rc < 0) {
-	 cerr << "No nds2 channels available, rc = " << rc << endl;	 
+	 cerr << "  No nds2 channels available, rc = " << rc << endl;	 
       }
       else {
+	 if (my_debug) cerr << "  nds2 channels available, rc = " << rc << endl ;
+	 // copy the channels from the chan_list to the DAQDChannelList.
 	 for (DAQC_api::const_channel_iter i=c.begin(); i != c.end(); ++i) {
+#ifdef TEST_RATE
+	    ChannelNameRate nr(i->mName, i->mRate) ;
+	    chns.insert (DAQDChannelList::value_type (nr, *i)) ;
+#else
+	    ////////////////////////////////////////////////////////////////////////////////////////
+	    /// This is the spot that channels with the same name but different rates disappear. ///
+	    ///  The insert is a stl call that uses the channel name as the key, so duplicate    ///
+	    ///  names can't exist in the list.                                                  ///
+	    ////////////////////////////////////////////////////////////////////////////////////////
 	    chns.insert (DAQDChannelList::value_type (i->mName, *i));
+#endif
 	 }
       }
       delete daq;
+      bool retval = rc >= 0 ;
+      if (my_debug) cerr << "sendsio.cc readChnDAQServer() returns " << (retval ? "true)" : "false)") << endl ;
       return rc >= 0;
    }
 
