@@ -1,10 +1,10 @@
-VCSID("$(#)$Id: TLGPrint.cc 7352 2015-04-25 00:09:29Z ed.maros@LIGO.ORG $");
+VCSID("$(#)$Id: TLGPrint.cc 7625 2016-05-13 22:27:25Z john.zweizig@LIGO.ORG $");
 
 #include "PConfig.h"
 #include "ligogui/TLGPrint.hh"
 #include "ligogui/TLGEntry.hh"
 #include "pipe_exec.hh"
-#include <strings.h>
+#include <cstring>
 #include <TStyle.h>
 #include <TPad.h>
 #include <TMath.h>
@@ -13,10 +13,9 @@ VCSID("$(#)$Id: TLGPrint.cc 7352 2015-04-25 00:09:29Z ed.maros@LIGO.ORG $");
 #include <TSystem.h>
 // federated naming service?
 #ifdef __USE_FN_SERVICE
-#include <stdio.h>
+#include <cstdio>
 #include <xfn/xfn.h>
-#include <string.h>
-#include <stdlib.h>
+#include <cstdlib>
 #endif
 #include <string>
 #include <iostream>
@@ -310,7 +309,7 @@ namespace ligogui {
       }
    
    //*-*- open OS file
-      fStream   = new ::ofstream(fname,ios::out);
+      fStream = new ofstream(fname, ios::out);
       if ((fStream == 0) || !*fStream) {
          printf("ERROR in TPostScript::Open: Cannot open file:%sn",fname);
          return;
