@@ -1,4 +1,4 @@
-VCSID("$(#)$Id: FilterFile.cc 7338 2015-04-14 23:24:47Z ed.maros@LIGO.ORG $");
+VCSID("$(#)$Id: FilterFile.cc 7389 2015-06-11 18:51:56Z james.batch@LIGO.ORG $");
 /* -*- mode: c++; c-basic-offset: 3; -*- */
 #include <time.h>
 #include <stdio.h>
@@ -571,12 +571,12 @@ static int linenum ;
 	    {
 	       tokenlist tok ;
 	       tokens (p + lenSampDef, linelen - lenSampDef, tok) ;
-	       if ((tok.size() == 1) && isnum (tok[0]) && issamplerate(tok[0]))
+	       if ((tok.size() == 1) && isnum (tok[0]) && (issamplerate(tok[0]) || any_sample_rate))
 	       {
 		  sampleRate = (double) strtol(tok[0].c_str(), (char **) NULL, 10) ;
 		  break ;	// Found a line, quit looking.
 	       }
-	       else if ((tok.size() == 2) && isnum (tok[1]) && issamplerate(tok[1]))
+	       else if ((tok.size() == 2) && isnum (tok[1]) && (issamplerate(tok[1]) || any_sample_rate))
 	       {
 		  // We really don't care what tok[0] was.
 		  sampleRate = (double) strtol(tok[1].c_str(), (char **) NULL, 10) ;
