@@ -1,6 +1,6 @@
-/* version $Id: foton.cc 7389 2015-06-11 18:51:56Z james.batch@LIGO.ORG $ */
+/* version $Id: foton.cc 7625 2016-05-13 22:27:25Z john.zweizig@LIGO.ORG $ */
 /* -*- mode: c++; c-basic-offset: 4; -*- */
-#include <stdio.h>
+#include <cstdio>
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -16,7 +16,7 @@
 #include "goption.hh"
 
 // Allow output of debug info to a file, /tmp/fotonDebug.txt JCB
-fstream debugLog ;
+std::fstream debugLog ;
 
    // ROOT object
    TROOT root ("GUI", "Foton");
@@ -40,7 +40,7 @@ fstream debugLog ;
    "       -m 'mergefile' Merge matlab designs into existing filter file\n"
    "       -s 'filt:sec'  filter module/section selection\n"
    "       -h             this help\n"
-   "\nFoton version " VERSION "\n$Id: foton.cc 7389 2015-06-11 18:51:56Z james.batch@LIGO.ORG $\n";
+   "\nFoton version " VERSION "\n$Id: foton.cc 7625 2016-05-13 22:27:25Z john.zweizig@LIGO.ORG $\n";
 
 
 //////////////////////////////////////////////////////////////////////////
