@@ -44,6 +44,8 @@ using namespace sends;
 using namespace std;
 using namespace thread;
 
+static const int my_debug = 0 ;
+
 static const DAQC_api::wait_time tSample = 0.1;
 bool NDS2Socket::sasl_init = false;
 
@@ -352,12 +354,15 @@ int NDS2Socket::StopWriter() {
 }
 
 //======================================  Build NDS2 channel list.
+// Add entries of the form "channelname,datarate" to the ostream.
+// The channelname,datarate entries will be space separated if there
+// are more than one.
 ostream&
 list_chans(ostream& str, const DAQC_api::chan_list& list) {
     str << "{";
     for (DAQC_api::const_channel_iter i=list.begin(); i!=list.end(); ++i) {
 	if (i != list.begin()) str << " ";
-	str << i->mName;
+	str << i->mName << "," << i->mRate ; 
     }
     return str << "}";
 }
@@ -406,6 +411,8 @@ int NDS2Socket::RequestData (unsigned long start, unsigned long duration,
     // if (duration > 32) request << "32 ";
     list_chans(request, mRequest_List);
     request << ";\n";
+    if (mDebug) cerr << ">>>>>>>>>>>>>>>>> NDS command >>>>>> " << request.str() << endl;
+
 
     //----------------------------------  Send the request
     char transid[8];
@@ -488,6 +495,7 @@ int
 NDS2Socket::addAvailable (chantype chanty, long gps, vector<DAQDChannel>& list, 
 		       wait_time timeout) {
     semlock lockit (mux);
+    if (my_debug) cerr << "NDS2Socket::addAvailable()" << endl ;
 
     //----------------------------------  Request the channel list.
     ostringstream request;
@@ -503,11 +511,10 @@ NDS2Socket::addAvailable (chantype chanty, long gps, vector<DAQDChannel>& list,
     if (rc != sizeof(int)) return rc;
 
     if (nChan <= 0) {
-	cerr << "NDS2Socket::Available: Can't detemine number of channels" 
-	     << endl;
+	if (my_debug) cerr << "NDS2Socket::addAvailable: Can't detemine number of channels" << endl;
 	return -1;
     }
-    if (mDebug) cout << "Available: Number of channels =" << nChan << endl;
+    if (my_debug) cerr << "Available: Number of channels =" << nChan << endl;
     list.reserve(list.size() + nChan);
 
     /*----------------------------------  Loop over channels              */
@@ -528,11 +535,28 @@ NDS2Socket::addAvailable (chantype chanty, long gps, vector<DAQDChannel>& list,
 			 << chan.mName << " chan-type=" << typestr 
 			 << " rate=" << chan.mRate << " data-type='" 
 			 << datstr << "' (len: " << datstr.size() << ")" 
-			 << endl; 
+			 << endl;
+#if 0
+	// If the channel type is raw, append a ",<datarate>" to the channel
+	// name.  There can be more than one data rate associated with a 
+	// given channel name, and those cases need to be treated as separate
+	// channels with unique names.  
+	// 
+	// It should be noted that a similar problem exists with data types
+	// but this case isn't addressed yet.
+	if (typestr == "raw" && chan.mRate > 1.0) {
+		chan.mName += ",";
+		std::ostringstream ss;
+		ss << int(chan.mRate);
+		std::string sRate(ss.str());
+		chan.mName += sRate;
+	}
+#endif
 	chan.mChanType = DAQDChannel::cvt_str_chantype(typestr);
 	chan.mDatatype = DAQDChannel::cvt_str_datatype(datstr);
 	list.push_back(chan);
     }
+    if (my_debug) cerr << "NDS2Socket::addAvailable() - return" << endl ;
     return rc;
 }
 
@@ -807,16 +831,43 @@ NDS2Socket::SendRequest(const string& text, char *reply, long length,
     return rc;
 }
 
+int
+NDS2Socket::SetEpoch(unsigned long start, unsigned long stop) {
+   ostringstream os ;
+   os << start << "-" << stop ;
+   return SetEpoch(os.str()) ;
+}
+
 //======================================  Set an epoch of interest.
 int
 NDS2Socket::SetEpoch(const std::string& epoch) {
     string cmd = "set-epoch ";
     cmd += epoch;
-    cmd += ";";
+    cmd += ";\n";
     long replysize;
+    if (my_debug) cerr << "NDS2Socket::SetEpoch(" << cmd << ");" << endl ;
     return SendRequest(cmd, 0, 0, &replysize, 10.0);
 }
 
+int
+NDS2Socket::GetEpochList(std::string &epochList) {
+   string cmd = "list-epochs;\n" ;
+   int  retval ;
+   long length = 0 ;
+   // The SendRequest() should send the command and get the 4 byte response.
+   // If the response is 0, continue to get the list, otherwise leave.
+   retval = SendRequest(cmd, 0, 0, &length, 5.0) ;
+   if (retval) {
+      if (my_debug) cerr << "NDS2Socket::GetEpochList(): SendRequest failed, " << retval << endl ;
+      return retval ;
+   }
+   // The return from Send Request was 0, there should be a string which is
+   // preceded by 4 bytes containing the length of the string.
+   retval = RecvStr(epochList) ;
+
+   return retval ;
+}
+
 //======================================  Wait for a message to arrive
 //                                        WaitforData() <= 0 means no data.
 int 
