@@ -18,8 +18,7 @@
 #include <cstdio>
 #include <cstdlib>
 #include <cstring>
-#include <sys/types.h>
-#include <sys/socket.h>
+#include "sockutil.h"
 #if defined (P__LINUX)
 // #include <sys/time.h>
 #include <linux/types.h>
@@ -34,7 +33,6 @@ typedef u_int32_t in_addr_t;
 #else
 #include <sys/sockio.h>
 #endif
-#include <netinet/in.h>
 #include <arpa/inet.h>
 #include <netdb.h>
 #include <unistd.h> 
@@ -56,9 +54,8 @@ using namespace std;
 namespace framexmit {
 
    //===================================  Open the receiver socket.
-   bool frameRecv::open (const char* mcast_addr, const char* interface,
-                     int port)
-   {
+   bool
+   frameRecv::open(const char* mcast_addr, const char* interface, int port) {
       if (sock >= 0) {
          close();
       }
@@ -72,7 +69,7 @@ namespace framexmit {
       // set reuse socket option 
       int reuse = 1;
       if (setsockopt (sock, SOL_SOCKET, SO_REUSEADDR, 
-                     (char*) &reuse, sizeof (int)) != 0) {
+		      (char*) &reuse, sizeof (int)) != 0) {
          close ();
          return false;
       }
@@ -85,7 +82,7 @@ namespace framexmit {
 	 bufsize = bTest;
       }
       if (setsockopt (sock, SOL_SOCKET, SO_RCVBUF, 
-                     (char*) &bufsize, sizeof (int)) != 0) {
+		      (char*) &bufsize, sizeof (int)) != 0) {
          close();
          return false;
       }
@@ -119,7 +116,7 @@ namespace framexmit {
          group.imr_multiaddr.s_addr = inet_addr (mcast_addr);
          group.imr_interface.s_addr = i_addr.s_addr;
          if (setsockopt (sock, IPPROTO_IP, IP_ADD_MEMBERSHIP, 
-                        (char*) &group, sizeof (ip_mreq)) == -1) {
+			 (char*) &group, sizeof (ip_mreq)) == -1) {
             close();
             return false;
          }
@@ -128,26 +125,26 @@ namespace framexmit {
          if (logison) {
             char buf[256];
             sprintf (buf, "Join multicast %s at %s", mcast_addr,
-                    inet_ntoa(i_addr));
+		     inet_ntoa(i_addr));
             addLog (buf);
          }
-      #ifdef DEBUG
+#ifdef DEBUG
          cout << "join multicast " << mcast_addr << " at " 
 	      << inet_ntoa (i_addr) << endl;
-      #endif
+#endif
       }
       else {
          multicast = false;
          // broadcast: enable
          int bset = 1;
          if (setsockopt (sock, SOL_SOCKET, SO_BROADCAST, 
-                        (char*) &bset, sizeof (bset)) == -1) {
+			 (char*) &bset, sizeof (bset)) == -1) {
             close();
             return false;
          }
-      #ifdef DEBUG
+#ifdef DEBUG
          cout << "broadcast port "  << port << endl;
-      #endif
+#endif
       }
 
       //--------------------------------  Open a request socket.
@@ -183,12 +180,14 @@ namespace framexmit {
       }
       pkts.clear();
       first = true;
+      lastSequenceSuccessful = false;
 #if defined(PBTEASEPACKET0)
+      //  Initialize teasing status counters
       if(par.teasePacket0) {
-	 lastSequenceSuccessful = false;
 	 drawOutFrameCount = 0;
-	 lastFrameDrawnOut = 0; // Sequence 0 should normally not be drawn out, since a blocking
-	                        // call is always called when receiving the first frame
+	 lastFrameDrawnOut = 0; // Sequence 0 should not be drawn out -
+	                        // a blocking call is always called when
+	                        // receiving the first frame
 	 currentFrameDrawingOut = 0; // The current frame we are working on
 	 lastFrameDuration = par.lastFrameDurationDefault; // duration of last frame defaults to 4 seconds if no frame seen
       }
@@ -207,7 +206,7 @@ namespace framexmit {
       if (multicast) {
          // multicast drop
          if (setsockopt (sock, IPPROTO_IP, IP_DROP_MEMBERSHIP, 
-                        (char*) &group, sizeof (ip_mreq)) == -1) {
+			 (char*) &group, sizeof (ip_mreq)) == -1) {
             // do nothing; close anyway
          }
       }
@@ -234,7 +233,6 @@ namespace framexmit {
       return !cntrl.received;
    }
 
-
    bool frameRecv::purge ()
    {
       // empty input queue
@@ -249,7 +247,7 @@ namespace framexmit {
 
 
    int64_t frameRecv::calcDiff (const frameRecv::packetlist& pkts, 
-                     unsigned int newseq) {
+				unsigned int newseq) {
       int64_t diff = 0;
       if (pkts.size() > 0) {
          diff = int64_t(pkts[0]->header.seq) - int64_t(newseq);
@@ -263,107 +261,101 @@ namespace framexmit {
       return diff;
    }
 
-
+   //===================================  Receive a frame
    int frameRecv::receive (char*& data, int maxlen, unsigned int* sequence,
 			   unsigned int* time, unsigned int* duration)
    {
       typedef std::vector<packetControl>  ctrl_vect;
       typedef ctrl_vect::iterator         ctrl_iter;
 
+      // check socket
+      if (sock < 0) {
+         return -1;
+      }
+
       int64_t lastTimeSequenceDone = 0;	// the last time a sequence finished
       int64_t firstPktTime = 0;	// when the first packet arrived
 
 #if defined(PBTEASEPACKET0)
       if(par.teasePacket0) {
 	 searchForNewseqStartTime = get_timestamp();
-	 lastRebroadcastTime = 0;  // can only issue new request if this is at zero
-	 retryDrawOut = 0;		// # of rebroadcasts
-	 drawOutFrameCount = 0;            // reset the number of frames drawn out to zero
-	 if(first) {
-	    currentFrameDrawingOut = oldseq; // This is the current frame we are drawing out
-	 }
-	 else {
-	    currentFrameDrawingOut = oldseq + 1; // This is the current frame we are drawing out
-	 }
+	 lastRebroadcastTime = 0;  // can only issue new request if this is zero
+	 retryDrawOut = 0;	   // # of rebroadcasts
+	 drawOutFrameCount = 0;    // zero the number of frames drawn out
+	 currentFrameDrawingOut = oldseq; // Set the frame we are drawing out
+	 if (!first) currentFrameDrawingOut++;
       }
 #endif // PBTEASEPACKET0
-
-      // check socket
-      if (sock < 0) {
-         return -1;
-      }
    
       // check on data array and max. length
       if (data == 0) {
          maxlen = -1;
       }
       else if (maxlen <= 0) {
-#if defined(PBTEASEPACKET0)
-      if(par.teasePacket0) {
 	 lastSequenceSuccessful = false;
-      }
-#endif // PBTEASEPACKET0
          return -2;
       }
    
       // determine qos parameter
       double qosFrac;
       switch (qos) {
-         case 0: 
-            qosFrac = 0.3;
-            break;
-         case 1: 
-            qosFrac = 0.1;
-            break;
-         default: 
-            qosFrac = 0.03;
-            break;
+      case 0: 
+	 qosFrac = 0.3;
+	 break;
+      case 1: 
+	 qosFrac = 0.1;
+	 break;
+      default: 
+	 qosFrac = 0.03;
+	 break;
       }
    
       // read a packet if buffer empty
       if (pkts.size() == 0) {
 #if defined(PBTEASEPACKET0)
-	 if(par.teasePacket0) {
-	 	////////////////////////////////////////////////////////////
-	 	// On first socket read, don't just block (unless we've never seen
-	 	// a sequence or the last sequence was unsuccessful):
-	 	// Wait for 4 seconds, after that issue a rebroadcast
-	 	// request for packet 0 of what we expect to be the new sequence
-	 	// (oldseq + 1)
-	 	// This routine will stop as soon as ANY packet arrives, even
-	 	// if from a different sequence, at which point the code will
-	 	// go to the main packet processing loop further below.
-	 	// This code just says that SOMETHING should have been received
-	 	// in the first four seconds, and if it isn't, then request
-	 	// packet 0 of what we expected to be the next sequence.
-	 	//
-	 	////////////////////////////////////////////////////////////
-	 	// HOWEVER: DO block as long as necessary if we've not recently
-	 	// seen a successful sequence
-	 	if(lastSequenceSuccessful == false) {
-	 	   // request a packet
-	 	   if (!getPacket()) {
-	 	      lastSequenceSuccessful = false;
-	 	      return -3;
-	 	   }
-	 	}
-	 	// Next code run only if we recently saw a successful sequence
-	 	// so that we know that the next packets should be arriving in 4 seconds
-	 	else {
-         	    // request a packet
-         	    while (getPacket (false)) {
-	 		// Has it now been too long? Should we issue a rebroadcast request?
-	 		int64_t now = get_timestamp();
+	 if (par.teasePacket0) {
+	    ////////////////////////////////////////////////////////////
+	    // On first socket read, don't just block (unless we've never
+	    // seen a sequence or the last sequence was unsuccessful):
+	    // Wait for 4 seconds, after that issue a rebroadcast
+	    // request for packet 0 of what we expect to be the new sequence
+	    // (oldseq + 1)
+	    // This routine will stop as soon as ANY packet arrives, even
+	    // if from a different sequence, at which point the code will
+	    // go to the main packet processing loop further below.
+	    // This code just says that SOMETHING should have been received
+	    // in the first four seconds, and if it isn't, then request
+	    // packet 0 of what we expected to be the next sequence.
+	    //
+	    ////////////////////////////////////////////////////////////
+	    // HOWEVER: DO block as long as necessary if we've not recently
+	    // seen a successful sequence
+	    if(lastSequenceSuccessful == false) {
+	       // request a packet
+	       if (!getPacket()) {
+		  lastSequenceSuccessful = false;
+		  return -3;
+	       }
+	    }
+
+	    // Next code run only if we recently saw a successful sequence
+	    // so that we know that the next packets should be arriving
+	    // in 4 seconds
+	    else {
+	       // Try to read a packet a packet
+	       while (!getPacket (false)) {
+
+		  // Test the time since searchForNewseqStartTime. Issue
+		  // rebroadcast if now - searchForNewseqStartTime > 4.45s
+		  int64_t now = get_timestamp();
+		  int r = teasePacket0(now, 1);
+		  // Bail if putPacket fails 
+		  if(r != 0) return(r);
 	 	
-	 		int r = teasePacket0(now, 1);
-	 		if(r != 0) return(r);
-	 	
-	 		// Wait a little bit before checking again
-	 		timespec wait = {(1000LL * rcvDelayTick) / 1000000000LL, 
-	 				 (1000LL * rcvDelayTick) % 1000000000LL};
-	 		nanosleep (&wait, 0);
-	 	    } // while getPacket
-	 	}
+		  // Wait a little bit before checking again
+		  socketWait(sock, teaseDelay(now), wm_read);
+	       } // while !getPacket
+	    }
 	 } // if(par.teasePacket0)
 	 else {
 	    if (!getPacket()) {
@@ -378,10 +370,10 @@ namespace framexmit {
 # endif // PBTEASEPACKET0
 	 firstPktTime = get_timestamp();
       }
-   #ifdef DEBUG
+#ifdef DEBUG
       cout << "first packet received" << endl;
-   #endif
-      // calculate new sequnence number from old
+#endif
+      // calculate new sequence number from old
       unsigned int newseq;
 
       // if first, set the old sequence number to one of the first packet
@@ -397,11 +389,7 @@ namespace framexmit {
 	 newseq = oldseq + 1;	// actually, we expect the sequence to start with oldseq
       }
 
-#if defined(PBTEASEPACKET0)
-      if(par.teasePacket0) {
-	 lastSequenceSuccessful = false;
-      }
-#endif // PBTEASEPACKET0
+      lastSequenceSuccessful = false;
 
       // main receiver loop
       //bool requestPacket = false;	// request a new packet
@@ -416,41 +404,39 @@ namespace framexmit {
       ctrl_vect pktCntrl;	        // marks received packets
    
       //--------------------------------  Loop over bursts of packets
+      double wait_time = 0;
       while (1) {
 	 seqDone = false;		// reset seqDone to false
 
       	 //-----------------------------  Fill the packet list from the input 
 	 //                               socket (maximum of 10 packets)
-         int max = 10;
-         while (getPacket (false)) {
-	    if(firstPktTime == 0)
-	       	 firstPktTime = get_timestamp();
-
-
-            if (--max <= 0) {
-               break;
-            }
-         }
+	 if (socketWait(sock, wait_time, wm_read) > 0) {
+	    int max = 10;
+	    while (getPacket (false)) {
+	       if(firstPktTime == 0) 
+		  firstPktTime = get_timestamp();
+	       if (--max <= 0) {
+		  break;
+	       }
+	    }
+	 }
+	 wait_time = 100;  // Arbitrarily long starting delay
 
 #if defined(PBTEASEPACKET0)
-	 if(par.teasePacket0) {
-	 	////////////////////////////////////////////////////////////
-	 	// Request rebroadcast of packet 0 if we haven't heard anything
-	 	// for 4 seconds
-	 	////////////////////////////////////////////////////////////
-	 	// Only bother with this code if we haven't even received the first packet
-	 	// of the sequence
-	 	if(firstPacket == true) { // true means we're still waiting for first packet
-	 	   // And if there's still nothing coming in
-	 	   if(pkts.empty()) {
-	 	      // Has it now been too long? Should we issue a rebroadcast request?
-	 	      int64_t now = get_timestamp();
-	 	
-	 	      int r = teasePacket0(now, 0);
-	 	      if(r != 0) return(r);
-	 	
-	 	   } // if pkts.empty()
-	 	} // firstPacket == false
+	 ////////////////////////////////////////////////////////////
+	 // Request rebroadcast of packet 0 if we are waiting for the first
+	 // packet of a sequence and we haven't heard anything for 4 seconds.
+	 ////////////////////////////////////////////////////////////
+	 if(par.teasePacket0 && firstPacket) { // true if we're waiting for first packet
+	    // And if there's still nothing coming in
+	    if(pkts.empty()) {
+	       // Has it now been too long? Should we issue a rebroadcast request?
+	       int64_t now = get_timestamp();
+
+	       int r = teasePacket0(now, 0);
+	       if(r != 0) return(r);
+	       wait_time = teaseDelay(now); // wait as long as necessary
+	    } // if pkts.empty()
 	 } // if par.teasePacket0
 #endif // PBTEASEPACKET0
       
@@ -461,16 +447,15 @@ namespace framexmit {
 
          //  Wait a receiver tick if no packets were received or
          //  only more recent packets are in the queue
-         if (pkts.empty() || (!firstPacket && (diff > 0))) {
-	    micro_delay(rcvDelayTick);
-	 }
+         // if (pkts.empty() || (!firstPacket && (diff > 0))) {
+	 //    micro_delay(par.rcvDelayTick);
+	 // }
       
          //  Process the packets at front of queue if some are there
          if (!pkts.empty()) {
 
             // check if sequence out of sync
-            if (((diff<0 ? -diff : diff) > maxSequenceOutOfSync) ||
-               ((diff >= 1) && (len == 0))) {
+            if (abs(diff) > maxSequenceOutOfSync || (diff >= 1 && len == 0)) {
 	       ////////////////////////////////////////////////////////////
 	       // PB: bug fix for 8 seconds to 4 seconds
 	       ////////////////////////////////////////////////////////////
@@ -490,8 +475,8 @@ namespace framexmit {
 
 #if defined(PBTEASEPACKET0)
                if(par.teasePacket0) {
-		  // If we get here a new sequence has come in with a new sequence number
-		  // which we are going to take.
+		  // If we get here a new sequence has come in with a new 
+		  // sequence number which we are going to take.
 		  // So: don't advance the search start time as we normally do, i.e.
 		  // searchForNewseqStartTime += lastFrameDuration*1000000;
 		  // Since here we have a sequence that just recently arrived, so
@@ -509,9 +494,9 @@ namespace framexmit {
 		      << "New sequence number is " << newseq;
                   addLog(msg.str());
                }
-            #ifdef DEBUG
+#ifdef DEBUG
 	       if (diff) cout << "have to skip (" << int(diff) << ")" << endl;
-            #endif
+#endif
                continue;
             } // end if(out of sequence)
          
@@ -538,11 +523,7 @@ namespace framexmit {
                   // check if data array is long enough
                   int needed = pkts[0]->header.pktTotal * packetSize;
                   if ((maxlen > 0) && (maxlen < needed)) {
-#if defined(PBTEASEPACKET0)
-		     if(par.teasePacket0) {
-			lastSequenceSuccessful = false;
-		     }
-#endif // PBTEASEPACKET0
+		     lastSequenceSuccessful = false;
                      return -needed;
                   }
                   // allocate data array if necessary
@@ -555,16 +536,12 @@ namespace framexmit {
                   if (logison && !data) {
                      char buf[256];
                      sprintf (buf, "Memory allocation for %i failed",
-                             needed);
+			      needed);
                      addLog (buf);
                   }
                   // check if data array is valid
                   if (data == 0) {
-#if defined(PBTEASEPACKET0)
-		     if(par.teasePacket0) {
-			lastSequenceSuccessful = false;
-		     }
-#endif // PBTEASEPACKET0
+		     lastSequenceSuccessful = false;
                      return -5;
                   }
                   // set sequence, time stamp & duration on caller's request
@@ -583,36 +560,36 @@ namespace framexmit {
                // copy packet payload into receiver buffer
                int pktNum = pkts[0]->header.pktNum;
                if ((pktNum >= 0) && (pktNum < (int)pktCntrl.size()) &&
-                  (!pktCntrl[pktNum].received)) {
+		   (!pktCntrl[pktNum].received)) {
                   memcpy (data + pktNum * packetSize, pkts[0]->payload,
-                         pkts[0]->header.pktLen);
+			  pkts[0]->header.pktLen);
                   len += pkts[0]->header.pktLen;
                   pktCntrl[pktNum].received = true;
-               #if DEBUG - 0 > 2 // add a random error rate!
+#if DEBUG - 0 > 2 // add a random error rate!
                   if ((double)rand() < rcvErrorRate * 32768.0) {
                      pktCntrl[pktNum].received = false;
                      len -= pkts[0]->header.pktLen;
                   }
-               #endif
+#endif
                }
             
                // check if last packet
                if (pkts[0]->header.pktTotal == pkts[0]->header.pktNum + 1) {
                   seqDone = true;
-               #ifdef DEBUG
-                  //  cout << "received last package at " <<
-                  //     (TAInow() % 1000000000000) / 1E9 << endl;
-               #endif
+#ifdef DEBUG
+                  cout << "received last package at " 
+		       << (TAInow() % 1000000000000) / 1E9 << endl;
+#endif
 #if defined(PBTEASEPACKET0)
-	       if(par.teasePacket0) {
-		  lastFrameDuration = pkts[0]->header.duration;
-		  // Is the last frame duration a reasonable number?
-		  // used to be: if((lastFrameDuration <= 0) || (lastFrameDuration > par.maxFrameDuration)) {
-		  // but lastFrameDuration is unsigned, so ignoring that
-		  if(lastFrameDuration > par.maxFrameDuration) {
-		     lastFrameDuration = par.lastFrameDurationDefault; // duration of last frame defaults to 4 seconds if no frame seen
+		  if(par.teasePacket0) {
+		     lastFrameDuration = pkts[0]->header.duration;
+		     // Is the last frame duration a reasonable number?
+		     // used to be: if((lastFrameDuration <= 0) || (lastFrameDuration > par.maxFrameDuration)) {
+		     // but lastFrameDuration is unsigned, so ignoring that
+		     if(lastFrameDuration > par.maxFrameDuration) {
+			lastFrameDuration = par.lastFrameDurationDefault; // duration of last frame defaults to 4 seconds if no frame seen
+		     }
 		  }
-	       }
 #endif // PBTEASEPACKET0
 
                }
@@ -635,8 +612,9 @@ namespace framexmit {
 	 int64_t timestampNow = get_timestamp();
 
 	 ////////////////////////////////////////////////////////////
-	 // NOTE: the very first packet of a sequence will be coming in on the order of seconds from
-	 // the last transmission, so we do NOT want to say things have timed out if it's been more
+	 // NOTE: the very first packet of a sequence will be coming in on the
+	 // order of seconds from the last transmission, so we do NOT want to
+	 // say things have timed out if it's been more
 	 // than a couple of milliseconds!
 	 // So we do NOT check the time stamp if:
 	 //  (1) we've only received one packet so far
@@ -644,7 +622,8 @@ namespace framexmit {
 	 // This coincides with zero retries and the first packet
 	 // ALSO:
 	 // Can reduce the number of calls if make use of lastTimeSequenceDone:
-	 // Just take the maximum of (lastPktTime, lastTimeSequenceDone) and compare timestampNow to that
+	 // Just take the maximum of (lastPktTime, lastTimeSequenceDone) and
+	 // compare timestampNow to that
 	 ////////////////////////////////////////////////////////////
 	 if(!(firstPacket && (retry==0))) {
 	    // when trying to figure out if we've lost a packet, take the later of:
@@ -658,9 +637,12 @@ namespace framexmit {
 
 	       seqDone = true;
 	    } // if timestamp
+	    else {
+	       wait_time = (lastInterestingTime+par.lostPacketWait-timestampNow)
+		  /1e6;
+	    }
 	 } // if !firstpacket && (retry==0)
 
-
          //-----------------------------  Continue if sequence not yet finished
          if (!seqDone) {
             continue;
@@ -668,11 +650,11 @@ namespace framexmit {
 
          // calculate # of missing packets
          int missingPackets = count_if (pktCntrl.begin(), pktCntrl.end(), 
-                              isNotValidPacket);
+					isNotValidPacket);
 	 ////////////////////////////////////////////////////////////
 	 // Sequence done: either actually finished, or timed out
 	 ////////////////////////////////////////////////////////////
-	 lastTimeSequenceDone = get_timestamp();	// save the last time a sequence finished
+	 lastTimeSequenceDone = get_timestamp(); // save the last time a sequence finished
       
          //-----------------------------  Check if complete
 	 //         if (missingPackets == 0) {
@@ -680,17 +662,13 @@ namespace framexmit {
 
             oldseq = newseq;
             retransmissionRate = newRetransmissionRate;
-#if defined(PBTEASEPACKET0)
-	    if(par.teasePacket0) {
-	       lastSequenceSuccessful = true;
-	    }
-#endif // PBTEASEPACKET0
+	    lastSequenceSuccessful = true;
             return len; // done!
          }
-         #ifdef DEBUG
-         // cout << "missing packets " << missingPackets << 
-            // " (packets pending " << pkts.size() << ")" << endl;
-         #endif
+#ifdef DEBUG
+	 cout << "missing packets " << missingPackets << 
+	      << " (packets pending " << pkts.size() << ")" << endl;
+#endif
 
 	 ////////////////////////////////////////////////////////////
 	 // Retransmit logic
@@ -701,7 +679,7 @@ namespace framexmit {
 
             // check if retry is expired
 	    retryExpired = !timestampNow || 
-	                   (timestampNow - timestamp) > retryTimeout;
+	       (timestampNow - timestamp) > retryTimeout;
          } // if retry > 0 && !retryExpired
 
 
@@ -709,52 +687,55 @@ namespace framexmit {
          if ((missingPackets > maximumRetransmit) ||
 	     // TURN OFF QoS checking
 	     // (retransmissionRate > qosFrac) || 
-            ((retry >= maxRetry) && (retryExpired)) ||
-            ((int)pkts.size() >= rcvpacketbuffersize)) {
+	     ((retry >= maxRetry) && (retryExpired)) ||
+	     ((int)pkts.size() >= rcvpacketbuffersize)) {
 
-	       // Save the original length of the packet buffer to pkts_size
-	       int pkts_size = pkts.size();
-	       unsigned int sequence_delete = newseq;
-	       int newseq_found = 0;
-	       // Search through packet buffer starting from the head of the buffer                                                 
-	       for(int i = 0; i < pkts_size; ) {
-		  // Is this packet of the same sequence we want to delete?                                                         
-		  if(pkts[i]->header.seq == sequence_delete) {
-		     // Yes: erase this packet from the buffer
+	    // Save the original length of the packet buffer to pkts_size
+	    int pkts_size = pkts.size();
+	    unsigned int sequence_delete = newseq;
+	    int newseq_found = 0;
+	    // Search through packet buffer starting from the head of the buffer                                                 
+	    for(int i = 0; i < pkts_size; ) {
+	       // Is this packet of the same sequence we want to delete?
+	       if(pkts[i]->header.seq == sequence_delete) {
+		  // Yes: erase this packet from the buffer
 
-		     pkts.erase(pkts.begin() + i);
-		     // AND show that this buffer is now one packet shorter                                                         
-		     pkts_size --;
-		     // Note: we don't have to update i here since i will again point                                               
-		     // to the next element (since we deleted this element)                                                         
-		  }
-		  else {
-		     // This is a packet we want to keep. Have we already found a new sequence number?                              
-		     if(!newseq_found) {
-
-			newseq = pkts[i]->header.seq;
-			newseq_found = 1;
-		     }
-		     // Move on to the next packet in the buffer                                                                    
-		     i++;
-		  }
+		  pkts.erase(pkts.begin() + i);
+		  // AND show that this buffer is now one packet shorter
+		  pkts_size --;
+		  // Note: we don't have to update i here since i will again point
+		  // to the next element (since we deleted this element)
 	       }
-	       // Did we not find any new sequence? If so, just increment the sequence number by 1                                  
-	       if(!newseq_found) {
-		  newseq ++;
+	       else {
+		  // This is a packet we want to keep. Have we already
+		  // found a new sequence number?
+		  if(!newseq_found) {
+
+		     newseq = pkts[i]->header.seq;
+		     newseq_found = 1;
+		  }
+		  // Move on to the next packet in the buffer
+		  i++;
 	       }
+	    }
+	    // Did we not find any new sequence? If so, just increment the
+	    // sequence number by 1
+	    if(!newseq_found) {
+	       newseq ++;
+	    }
             firstPacket = true;
             retry = 0;
             len = 0;
             seqDone = false;
 #if defined(PBTEASEPACKET0)
             if(par.teasePacket0) {
-	       // If we got here we (most likely) timed out and are moving on to a new sequence.
-	       // The expected arrival time for the next sequence is the beginning of the last sequence
-	       // plus the expected duration (usually 4 seconds)
-	       // NOT just the time now, i.e. searchForNewseqStartTime = get_timestamp();
-	       // But instead we imagine that the receive() function was called 4 seconds after the
-	       // last start, i.e.
+	       // If we got here we (most likely) timed out and are moving on
+	       // to a new sequence. The expected arrival time for the next
+	       // sequence is the beginning of the last sequence plus the
+	       // expected duration (usually 4 seconds) NOT just the time now,
+	       // i.e. searchForNewseqStartTime = get_timestamp();
+	       // But instead we imagine that the receive() function was
+	       // called 4 seconds after the last start, i.e.
 	       searchForNewseqStartTime += lastFrameDuration*1000000;
 	       retryDrawOut = 0;
 	       currentFrameDrawingOut = newseq;
@@ -763,7 +744,7 @@ namespace framexmit {
             retransmissionRate = newRetransmissionRate;
 
 	    if(!pkts.empty()) {
-	        diff = calcDiff (pkts, newseq);
+	       diff = calcDiff (pkts, newseq);
 	    }
 	    else {
 	       diff = 0;
@@ -772,7 +753,6 @@ namespace framexmit {
          }
          else if ((retry == 0) || (retryExpired)) {
 
-
             // calculate new retransmission rate
             if (retry == 0) {
                newRetransmissionRate = 
@@ -803,29 +783,25 @@ namespace framexmit {
             if (logison) {
                char buf[256];
                sprintf (buf, "Ask for retransmission of %i packets\n"
-                       "New retransmission rate is %g",
-                       n, newRetransmissionRate);
+			"New retransmission rate is %g",
+			n, newRetransmissionRate);
                addLog (buf);
             }
-         #ifdef DEBUG
-            cout << "ask for retransmission " << n << " (rate " <<
-               newRetransmissionRate << "; seq " << newseq << ") at " << 
-               (TAInow() % 1000000000000LL) / 1E9 << endl;
-         #endif
-         #if DEBUG - 0 > 1
+#ifdef DEBUG
+            cout << "Ask for retransmission " << n << " (rate "
+		 << newRetransmissionRate << "; seq " << newseq << ") at "
+		 << (TAInow() % 1000000000000LL) / 1E9 << endl;
+#endif
+#if DEBUG - 0 > 1
             cout << "Packets =";
             for (int jj = 0; jj < n; ++jj) {
                cout << " " << rpkt.pktResend[jj];
             }
             cout << endl;
-         #endif
+#endif
             rpkt.header.pktLen = n * sizeof (int32_t);
             if (!putPacket (rpkt)) {
-#if defined(PBTEASEPACKET0)
-	       if(par.teasePacket0) {
-		  lastSequenceSuccessful = false;
-	       }
-#endif // PBTEASEPACKET0
+	       lastSequenceSuccessful = false;
                return -6;
             }
          
@@ -833,8 +809,12 @@ namespace framexmit {
             retryExpired = false;
 	    timestamp = get_timestamp();
             retry++;
+	    if (wait_time > retryTimeout/1e6) wait_time = retryTimeout/1e6;
          }
-      
+	 else if (retry > 0) {
+	    double tleft = (timestamp + retryTimeout - timestampNow) / 1e6;
+	    if (tleft > 0 && tleft < wait_time) wait_time = tleft;
+	 }
          //requestPacket = true;
       } // end loop over bursts of packets, while (1)
 
@@ -849,9 +829,9 @@ namespace framexmit {
          if (logison) {
             addLog ("Packet buffer is full");
          }
-      #ifdef DEBUG
+#ifdef DEBUG
          cout << "packet buffer full" << endl;
-      #endif
+#endif
          return false;
       }
    
@@ -868,17 +848,11 @@ namespace framexmit {
       do {
          // if not blocking poll socket
          if (!block) {
-            timeval 	wait;		// timeout=0
-            wait.tv_sec = 0;
-            wait.tv_usec = 0;
-            fd_set 		readfds;	
-            FD_ZERO (&readfds);
-            FD_SET (sock, &readfds);
-            int nset = select (FD_SETSIZE, &readfds, 0, 0, &wait);
+	    int nset = socketWait(sock, 0, wm_read);
             if (nset < 0) {
-            #ifdef DEBUG
+#ifdef DEBUG
                cout << "select on receiver socket failed" << endl;
-            #endif
+#endif
                return false;
             }
             else if (nset == 0) {
@@ -888,7 +862,7 @@ namespace framexmit {
       	 // read packet
          socklen_t max = sizeof (name);
          n = recvfrom (sock, (char*) pkt.get(), sizeof (packet), 0, 
-                      (struct sockaddr*) &name, &max);
+		       (struct sockaddr*) &name, &max);
          if (n < 0) {
             return false;
          }
@@ -905,18 +879,18 @@ namespace framexmit {
 
          //-----------------------------  Swap if necessary
          pkt->ntoh();
-      // repeat until valid packet
+	 // repeat until valid packet
       } while (pkt->header.checksum != save_crc || 
 	       n < (int)sizeof(packetHeader) || 
-              ((pkt->header.pktType != PKT_BROADCAST) && 
-              (pkt->header.pktType != PKT_REBROADCAST)) ||
-              (n != (int)sizeof (packetHeader) + pkt->header.pktLen));
+	       ((pkt->header.pktType != PKT_BROADCAST) && 
+		(pkt->header.pktType != PKT_REBROADCAST)) ||
+	       (n != (int)sizeof (packetHeader) + pkt->header.pktLen));
    
-      #if DEBUG - 0 > 1
-         cout << "received packet " << pkt->header.pktNum << 
-             (pkt->header.pktType == PKT_REBROADCAST ? "R " : " ") <<
-             " seq=" << pkt->header.seq << endl;
-      #endif 
+#if DEBUG - 0 > 1
+      cout << "received packet " << pkt->header.pktNum << 
+	 (pkt->header.pktType == PKT_REBROADCAST ? "R " : " ") <<
+	 " seq=" << pkt->header.seq << endl;
+#endif 
       // now add packet to packet buffer
       packetlist::iterator pos = lower_bound (pkts.begin(), pkts.end(), pkt);
       // check if end of list
@@ -949,8 +923,8 @@ namespace framexmit {
       // swap if necessary
       pkt.hton();
       if (sendto (rq_sock, (char*) &pkt, sbytes, 0, 
-                 (struct sockaddr*) &name, 
-                 sizeof (struct sockaddr_in)) != sbytes) {
+		  (struct sockaddr*) &name, 
+		  sizeof (struct sockaddr_in)) != sbytes) {
          ::close (rq_sock);
          rq_sock = -1;
          return false;
@@ -983,7 +957,7 @@ namespace framexmit {
    {
       logbuf = "framexmit log:\n";
       for (deque<string>::iterator i = logs.begin(); 
-          i != logs.end(); i++) {
+	   i != logs.end(); i++) {
          logbuf += *i;
       }
       return logbuf.c_str();
@@ -996,30 +970,47 @@ namespace framexmit {
    }
 
 #if defined(PBTEASEPACKET0)
-   // top indicates if teasePacket0() being run at top/beginning of receive loop or during loop
-   int frameRecv::teasePacket0 ( const int64_t now, int top)
+   double
+   frameRecv::teaseDelay(int64_t now) {
+      int64_t tLate = searchForNewseqStartTime 
+	 + lastFrameDuration * 1000000L
+	 + par.frameDrawRebroadcastInterval * 1000L;
+      if (tLate <= now) return 0.001;
+      return double(tLate - now) * 1e-6;
+   }
+   
+   // top indicates if teasePacket0() being run at top/beginning
+   // of receive loop or during loop
+   int
+   frameRecv::teasePacket0 ( const int64_t now, int top)
    {
+      int64_t drawIntvl = par.frameDrawRebroadcastInterval * 1000L;
+      int64_t lastDt    =  lastFrameDuration * 1000000L;
       // Have at least 4 seconds (plus 450ms) elapsed?
-      if(now > searchForNewseqStartTime + lastFrameDuration*1000000 + par.frameDrawRebroadcastInterval * 1000) {
-	 int top_condition = 1; // default to true
-	 if(!top) {
-	    top_condition = retryDrawOut < par.maxRetryDrawOut;
-	 }
-	 // Yes, we should be in the rebroadcast regime. But rebroadcast only if it's been at
-	 // least 450ms since lastrebroadcast (or if there's never been a rebroadcast since 
-	 // starting this sequence) AND if we haven't already used up our retries
-	 if(top_condition && ((retryDrawOut == 0) || 
-			      ((retryDrawOut > 0) && (now > lastRebroadcastTime + par.frameDrawRebroadcastInterval * 1000)))) {
-	    // Increment the drawOutFrameCount counter by one for this sequence if a new sequence
+      int64_t tLate = searchForNewseqStartTime + lastDt + drawIntvl;
+
+      if (now > tLate) {
+	 bool top_condition = top || retryDrawOut < par.maxRetryDrawOut;
+
+	 // Yes, we should be in the rebroadcast regime. But rebroadcast only
+	 // if it's been at least 450ms since last rebroadcast (or if there's
+	 // never been a rebroadcast since starting this sequence) AND if we
+	 // haven't already used up our retries
+	 if (top_condition && (retryDrawOut == 0 || 
+			       (retryDrawOut > 0 &&
+				now > lastRebroadcastTime + drawIntvl))) {
+	    // Increment the drawOutFrameCount counter by one for this
+	    // sequence if a new sequence
 	    if(currentFrameDrawingOut != lastFrameDrawnOut) {
 	       drawOutFrameCount ++;
 	       lastFrameDrawnOut = currentFrameDrawingOut;
 	    }
 		      
-	    // Have we used up the number of times we have allocated for drawing out a frame?
+	    // Have we used up the number of times we have allocated for
+	    // drawing out a frame?
 	    if(drawOutFrameCount > par.drawOutFrameCountMax) { // defaults to 5
 	       lastSequenceSuccessful = false;
-			 
+
 	       // Don't return. Just be quiet and don't issue any more retransmit requests
 	    }
 	    else {
@@ -1052,7 +1043,7 @@ namespace framexmit {
 		  currentFrameDrawingOut ++;
 		  retryDrawOut = 0;
 		  // Advance start time to where we think it should be
-		  searchForNewseqStartTime += lastFrameDuration*1000000;
+		  searchForNewseqStartTime += lastDt;
 	       }
 		          
 	       // Save this new rebroadcast time
@@ -1065,4 +1056,3 @@ namespace framexmit {
 #endif // PBTEASEPACKET0
 
 }
-
