@@ -198,6 +198,37 @@ namespace framexmit {
       return *this;
    }
 
+   //===================================  Static burst counter instance.
+   static frameSend::burst_counter burster;
+
+   //===================================  Pause if necessary
+   void
+   frameSend::burst_counter::time_check(void) {
+      if (!par.packetBurst || _n < par.packetBurst) return;
+      timestamp_type expire = _time + par.packetBurstInterval;
+      timestamp_type now = get_timestamp();
+      if (now < expire) {
+	 micro_delay(expire - now);
+      }
+      _n = 0;
+   }
+
+   //===================================  Force delay if packet has been written
+   void
+   frameSend::burst_counter::force_delay(void) {
+      if (_n) {
+	 _n = par.packetBurst;
+	 time_check();
+      }
+   }
+
+   //===================================  Bump counter after one packet
+   void 
+   frameSend::burst_counter::packet_time(void) {
+      _n++;
+      _time = get_timestamp();
+   }
+
 
    void xmitDaemonCallback (frameSend& th) 
    {
@@ -481,8 +512,6 @@ extern "C"
 
    bool frameSend::putPackets (packet pkts[], int n)
    {
-      static int pktnum = 0;
-   
       checksum_crc32 crc;
       for (int i = 0; i < n; i++) {
          // swap if necessary
@@ -495,6 +524,9 @@ extern "C"
 	 crc.add(reinterpret_cast<unsigned char*>(pkts + i), sbytes);
 	 pkts[i].header.checksum = crc.result();
 
+	 //-----------------------------  enforce any required delay
+	 burster.time_check();
+
          //-----------------------------  Send the packet
 	 int flags = 0;
 #ifdef P__DARWIN
@@ -515,13 +547,8 @@ extern "C"
 	 stats.total_sent++;
 	 stats.total_bytes += sbytes;
 
-         pktnum++;
-         if (pktnum == packetBurst) {
-            if (packetBurstInterval > 0) {
-	       micro_delay(packetBurstInterval);
-            }
-            pktnum = 0;
-         }
+	 //-----------------------------  Burst structure accounting.
+	 burster.packet_time();
       }
       return true;
    }
@@ -565,7 +592,7 @@ extern "C" {
 
       // allocate packet buffer/install cleanup function
       packet*	pkts   = 0;
-      int       pktdim = max (packetBurst, maximumRetransmit);
+      int       pktdim = max (par.packetBurst, maximumRetransmit);
       pkts = new (nothrow) packet[pktdim];
       if (pkts == 0) {
          ::close (sock);
@@ -584,39 +611,45 @@ extern "C" {
          if ((curbuf != -1) && (curbuf < (int)buffers.size())) {
 	    stats.send_requests++;
 
-            // assemble packets
-            int n;
+            //---------------------------  Get the sequence counters
+            int seqLen   = buffers[curbuf].sequenceLength();
+            int seqNum   = buffers[curbuf].sofar;
+	    int nBurst   = seqLen - seqNum;
+
+	    //---------------------------   Get the raw packet numbers
             int pktTotal = buffers[curbuf].totalPackets();
-            int pktNum   = buffers[curbuf].sofar / packetSize;
-            for (n = 0; (n < packetBurst) && (pktNum + n < pktTotal); n++) {
-	       size_t pktOff = buffers[curbuf].sofar;
+	    int pktNum   = seqNum;
+	    if (pktNum < pktTotal) {
+	       nBurst = pktTotal - pktNum;
+	    } else {
+	       pktNum -= pktTotal;
+	    }
+	    if (nBurst > par.packetBurst) nBurst = par.packetBurst;
+
+            //--------------------------  assemble packets
+            for (int n = 0; n < nBurst; n++) {
+	       size_t pktOff = buffers[curbuf].packetOffset(pktNum + n);
 	       size_t pktLen = buffers[curbuf].packetLength(pktNum + n);
 	       copy_packet(pkts[n], buffers[curbuf], PKT_BROADCAST, 
 			   pktNum+n, pktTotal, pktOff, pktLen);
-               buffers[curbuf].sofar += pktLen;
             }
+	    buffers[curbuf].sofar += nBurst;
+
             // check if buffer is done
-            if (buffers[curbuf].sofar >= buffers[curbuf].len) {
+            if (buffers[curbuf].sofar >= seqLen) {
                curbuf++;
             }
             mux.unlock();
          
-            // now send n packets
-	    stats.send_packets += n;
-            if (!putPackets (pkts, n)) {
+            //--------------------------  Now send nBurst packets
+	    if (!pktNum) burster.force_delay();
+	    stats.send_packets += nBurst;
+            if (!putPackets (pkts, nBurst)) {
             #ifdef DEBUG
                cout << "packet error 1" << endl;
             #endif
                break;
             }
-
-	    // optional duplicate packet
-	    if (!pktNum && pktTotal < par.nRepeatPktMax) {
-	       int nrpt = par.nRepeatPktNum;
-	       if (n < nrpt) nrpt = n;
-	       for (int i=0; i<nrpt; i++) pkts[i].ntoh();
-	       putPackets(pkts, nrpt);
-	    }
          }
 
 	 //----  no current packets... wait a few ticks.
