@@ -1120,7 +1120,6 @@ void
 GainMon::Configure(string& file_name) 
 {
     int NEWargc = 1;
-    char **NEWargv;
     //cout << "Trying to read config file " << file_name << ".\n";
     //---------- Verify that configuration file can be opened.
     ifstream input;
@@ -1142,8 +1141,10 @@ GainMon::Configure(string& file_name)
         //cout << "NEWargc = " << NEWargc << endl;
     }
     input.close();
-    NEWargv = new char*[NEWargc];
-    NEWargv[0] = "Configuration File: ";
+    const char* hedr = "Configuration File: ";
+    char** NEWargv = new char*[NEWargc];
+    NEWargv[0]  = new char[strlen(hedr)+1];
+    strcpy(NEWargv[0], hedr);
     //cout << NEWargv[0] << endl;
     //---------- Open file and copy contents to char arrays.
     ifstream inp2;
@@ -1161,6 +1162,10 @@ GainMon::Configure(string& file_name)
     }
     inp2.close();
     Configure(NEWargc,(const char **) NEWargv);
+    for (int i=0; i<NEWargc; i++) {
+	delete[] NEWargv[i];
+    }
+    delete[] NEWargv;
 }
 
 
