@@ -10,23 +10,37 @@
 // #include "TPaletteAxis.h"
 #include "TStyle.h"
 #include "fSeries/PSD.hh"
+#include "ParseLine.hh"
 #include "TSeries.hh"
 #include "DVector.hh"
 #include "lcl_array.hh"
+#include "WelchPSD.hh"
+#include "Hanning.hh"
 
+#include <cmath>
 #include <iostream>
 #include <stdexcept>
 using namespace std;
 
+static const char* defaultPaletteName="jet";
+
 //======================================  Constructor
 GDSPlot::GDSPlot(TCanvas* tc) 
-  : mCanvas(tc), mTrace(0), mXmin(0), mXmax(0), mYmin(0), mYmax(0),
-    mZmin(0), mZmax(0), mLogX(false), mLogY(false), mPalletID(0),
-    mNextColor(1), mNSmooth(0)
+    : mCanvas(tc), myCanvas(false), mTrace(0), mXmin(0), mXmax(0), mYmin(0),
+      mYmax(0), mZmin(0), mZmax(0), mLogX(false), mLogY(false), mPalletID(0),
+      mNextColor(1), mNSmooth(0)
 {
     gErrorIgnoreLevel=1500;
     set_style(0);
-    if (!mCanvas && gPad) set_canvas(gPad->GetCanvas());
+    if (!mCanvas) {
+	if (gPad) set_canvas(gPad->GetCanvas());
+	else      set_canvas(new TCanvas("GDSPlot"), true);
+    }
+}
+
+//======================================  Constructor
+GDSPlot::~GDSPlot(void) {
+    if (mCanvas && myCanvas) delete mCanvas;
 }
 
 //======================================  Plot a data vector.
@@ -102,9 +116,11 @@ GDSPlot::new_plot(void) {
 //======================================  Plot a PSD (log-log)
 void 
 GDSPlot::plot(const containers::PSD& psd) {
-    xlog();
-    ylog();
-    plot(psd.refDVect(), psd.getLowFreq(), psd.getFStep());
+    double fLow = psd.getLowFreq();
+    double dF   = psd.getFStep();
+    xlog(3 * max(fLow, dF) < psd.getHighFreq());
+    ylog(true);
+    plot(psd.refDVect(), fLow, dF);
 }
 
 //======================================  Plot a PSD (log-log)
@@ -140,6 +156,17 @@ GDSPlot::plot(const DVector& dvec, double xmin, double dx,
   nBin = dvec.getData(bMin, nBin, ytemp);
   bMax = nBin + bMin;
 
+  //------------------------------------  Get the minimum and maximum
+  double ymin  = ytemp[0];
+  double yminp = ymin;
+  double ymax  = ytemp[0];
+  for (int i=1; i<nBin; i++) {
+      double y = ytemp[i];
+      if (yminp <= 0 || (y>0 && y<yminp)) yminp = y;
+      if (y < ymin) ymin = y;
+      if (y > ymax) ymax = y;
+  }
+
   //------------------------------------  Fill a histogram
   TH1D histo("dvect", title.c_str(), nBin, xmin+dx*bMin, xmin+dx*bMax);
   for (int i=0; i<nBin; ++i) {
@@ -155,9 +182,16 @@ GDSPlot::plot(const DVector& dvec, double xmin, double dx,
     if (mLogX) mCanvas->SetLogx(1);
     else       mCanvas->SetLogx(0);
 
-    if (mLogY) mCanvas->SetLogy(1);
-    else       mCanvas->SetLogy(0);
-
+    if (mLogY) {
+	mCanvas->SetLogy(1);
+	if (mYmin <= 0 && yminp > 0) mYmin = 0.8 * yminp;
+	mYmax = 1.25 * ymax;
+	if (mYmin < mYmax * 1e-20) mYmin = mYmax * 1e-20;
+	histo.SetMinimum(mYmin);
+	histo.SetMaximum(mYmax);
+    } else {
+	mCanvas->SetLogy(0);
+    }
     mCanvas->SetTicks(1);
 
     set_axes(histo);
@@ -177,6 +211,34 @@ GDSPlot::print(const std::string& file) const {
     mCanvas->Print(file.c_str());
 }
 
+//======================================  Read a palette file
+void
+GDSPlot::read_palette(const string& palette) {
+    std::vector<double> x, red, green, blue;
+
+    //----------------------------------  Open the file
+    ParseLine pl(palette.c_str());
+    if (!pl.isOpen()) {
+	throw runtime_error("GDSPlot::read_palette can not read palette file");
+    }
+
+    //----------------------------------- Loop until the end
+    while (pl.getLine() >= 0) {
+	int nArg = pl.getCount();
+	if (nArg == 0) continue;
+	if (nArg > 4) {
+	    throw runtime_error("GDSPlot::read_palette invalid file format");
+	}
+	x.push_back(pl.getDouble(0));
+	red.push_back(pl.getDouble(1));
+	green.push_back(pl.getDouble(2));
+	blue.push_back(pl.getDouble(3));
+    }
+
+    //---------------------------------- Set the pallette
+    set_palette(x.size(), &x.at(0), &red.at(0), &green.at(0), &blue.at(0));
+    mPalletID = 99;
+}
 
 //======================================  Set the canvas
 void 
@@ -202,8 +264,9 @@ GDSPlot::set_axes(TH1& slate) {
 
 //======================================  Set the canvas
 void 
-GDSPlot::set_canvas(TCanvas* tc) {
-  mCanvas = tc;
+GDSPlot::set_canvas(TCanvas* tc, bool koe) {
+    mCanvas  = tc;
+    myCanvas = koe;
 }
 
 //======================================  Set color of the next plotted trace
@@ -212,26 +275,122 @@ GDSPlot::set_color(int tc) {
     mNextColor = tc;
 }
 
-//======================================  Set the color palette
+//======================================  Set the color pallette
 void
 GDSPlot::set_palette(void) {
     if (!mPalletID) {
-	//  John's original guess
-	//UInt_t Number = 6;
-	//Double_t Red[6]    = { 0.00, 0.00, 0.70, 1.00, 1.00, 0.50};
-	//Double_t Green[6]  = { 0.00, 0.00, 0.90, 1.00, 0.00, 0.25};
-	//Double_t Blue[6]   = { 0.40, 1.00, 1.00, 0.40, 0.00, 0.00};
-	//Double_t Length[6] = { 0.00, 0.10, 0.32, 0.60, 0.99, 1.00};
-	//  Based on matlab rgb definitions
+	set_palette(defaultPaletteName);
+    }
+}
+
+//======================================  Set the color pallette
+void
+GDSPlot::set_palette(const string& p_spec) {
+    int rev = 0;
+    string palette(p_spec);
+    if (palette.empty()) palette=defaultPaletteName;
+    size_t psufx = p_spec.rfind("_r");
+    if (p_spec.size() > 2 && psufx == p_spec.size() - 2) {
+	rev = 100;
+	palette.erase(psufx, 2);
+    }
+    
+    //---------------------------------- Jet color map
+    if (palette == "jet") {
+	if (mPalletID == 1 + rev) return;
 	UInt_t Number = 6;
 	Double_t Red[6]    = {0.000, 0.000, 0.000, 1.000, 1.000, 0.500};
 	Double_t Green[6]  = {0.000, 0.000, 1.000, 1.000, 0.000, 0.000};
 	Double_t Blue[6]   = {0.500, 1.000, 1.000, 0.000, 0.000, 0.000};
 	Double_t Length[6] = {0.000, 0.125, 0.375, 0.625, 0.875, 1.000};
-	Int_t nb=255;
-	TColor::CreateGradientColorTable(Number,Length,Red,Green,Blue,nb);
-	mPalletID = 1;
+	set_palette(Number, Length, Red, Green, Blue, rev);
+	mPalletID = 1 + rev;
     }
+
+    //---------------------------------- Hot color map
+    else if (palette == "hot") {
+	if (mPalletID == 2 + rev) return;
+	UInt_t Number = 4;
+	Double_t Red[4]    = {0.000, 1.000, 1.000, 1.000};
+	Double_t Green[4]  = {0.000, 0.000, 1.000, 1.000};
+	Double_t Blue[4]   = {0.000, 0.000, 0.000, 1.000};
+	Double_t Length[4] = {0.000, 0.375, 0.750, 1.000};
+	set_palette(Number, Length, Red, Green, Blue, rev);
+	mPalletID = 2 + rev;
+    }
+
+    //---------------------------------- Copper color map
+    else if (palette == "copper") {
+	if (mPalletID == 3 + rev) return;
+	UInt_t Number = 3;
+	Double_t Red[3]    = {0.000, 1.000, 1.000};
+	Double_t Green[3]  = {0.000, 0.640, 0.800};
+	Double_t Blue[3]   = {0.000, 0.400, 0.500};
+	Double_t Length[3] = {0.000, 0.800, 1.000};
+	set_palette(Number, Length, Red, Green, Blue, rev);
+	mPalletID = 3 + rev;
+    }
+
+    //---------------------------------- Bone color map
+    else if (palette == "bone") {
+	if (mPalletID == 4 + rev) return;
+	UInt_t Number = 4;
+	Double_t Red[4]    = {0.000, 0.320, 0.656, 1.000};
+	Double_t Green[4]  = {0.000, 0.320, 0.782, 1.000};
+	Double_t Blue[4]   = {0.000, 0.444, 0.782, 1.000};
+	Double_t Length[4] = {0.000, 0.375, 0.750, 1.000};
+	set_palette(Number, Length, Red, Green, Blue, rev);
+	mPalletID = 4 + rev;
+    }
+
+    //---------------------------------- Viridis color map
+    else if (palette == "viridis") {
+	if (mPalletID == 5 + rev) return;
+	UInt_t Number = 16;
+	Double_t Red[16] =
+	    {0.2670, 0.2832, 0.2730, 0.2431, 0.1856, 0.1287, 0.1206, 0.1433,
+	     0.1966, 0.2965, 0.4494, 0.6889, 0.8146, 0.8761, 0.9359, 0.9932};
+	Double_t Green[16]  =
+	    {0.0049, 0.1157, 0.2045, 0.2921, 0.4186, 0.5633, 0.6258, 0.6695,
+	     0.7118, 0.7616, 0.8138, 0.8654, 0.8834, 0.8911, 0.8986, 0.9062};
+	Double_t Blue[16]   =
+	    {0.3294, 0.4361, 0.5017, 0.5385, 0.5568, 0.5512, 0.5335, 0.5112,
+	     0.4792, 0.4242, 0.3354, 0.1827, 0.1103, 0.0953, 0.1081, 0.1439};
+	Double_t Length[16] = 
+	    {0.0000, 0.0784, 0.1490, 0.2235, 0.3451, 0.4980, 0.5647, 0.6118,
+	     0.6588, 0.7176, 0.7882, 0.8823, 0.9294, 0.9529, 0.9764, 1.0000};
+	set_palette(Number, Length, Red, Green, Blue, rev);
+	mPalletID = 5 + rev;
+    }
+
+    //---------------------------------- Unrecognized palette
+    else {
+	throw runtime_error("GDSPlot:set_palette Unrecognized palette name");
+    }
+}
+
+//======================================  Internal palette setter
+void
+GDSPlot::set_palette(size_t Number, const double* Length, const double* Red,
+		     const double* Green, const double* Blue, bool reverse) {
+    lcl_array<double> x(Number, Length);
+    lcl_array<double> r(Number, Red);
+    lcl_array<double> g(Number, Green);
+    lcl_array<double> b(Number, Blue);
+    if (reverse) {
+	size_t half=Number / 2;
+	for (size_t i=0; i<half; i++) {
+	    double tmp = x[i];
+	    x[i] = 1.0 - x[Number - 1 - i];
+	    x[Number - 1 - i] = 1.0 - tmp;
+	    std::swap<double>(r[i], r[Number - 1 - i]);
+	    std::swap<double>(g[i], g[Number - 1 - i]);
+	    std::swap<double>(b[i], b[Number - 1 - i]);
+	}
+	if ((Number%2) != 0) x[half+1] = 1.0 - x[half+1];
+    }
+    Int_t nb=255;
+    TColor::CreateGradientColorTable(Number, x, r, g, b, nb);
 }
 
 //======================================  Set the Axis ranges
@@ -303,12 +462,18 @@ void
 GDSPlot::surf(int nx, const double* x, int ny, const double* y, 
 	      const double* z) {
     TH2D xyplot("xyplot", mTitle.c_str(), nx, x, ny, y);
+    double zmin = mZmin + 0.001 * (mZmax - mZmin);
     int inx = 0;
     for (int j=0; j<ny; j++) {
 	double y0 = 0.5 * (y[j] + y[j+1]);
 	for (int i=0; i<nx; i++) {
 	    double x0 = 0.5 * (x[i] + x[i+1]);
-	    xyplot.Fill(x0, y0, z[inx++]);
+	    double z0 = z[inx++];
+	    //-------------------------------------
+	    // keep the data value on the color map
+	    if (z0 < zmin) z0 = zmin;
+	    //-------------------------------------
+	    xyplot.Fill(x0, y0, z0);
 	}
     }
 
@@ -339,6 +504,20 @@ GDSPlot::title(const std::string& titl) {
     mTitle = titl;
 }
 
+//======================================  Set canvas title
+void 
+GDSPlot::welch(const TSeries& ts, double fMin, double fMax) {
+    Hanning hw;
+    WelchPSD w(Interval(1.0), 0.50, &hw);
+    w.add(ts);
+    if (fMin == 0.0 && fMax == 0.0) {
+	plot(w.get_psd());
+    } else {
+	if (fMax == 0.0) fMax = 2.0/ts.getTStep();
+	plot(w.get_psd().extract_psd(fMin, fMax-fMin));
+    }
+}
+
 //======================================  Set the X-axis label
 void 
 GDSPlot::xlabel(const std::string& titl) {
