@@ -17,7 +17,7 @@ namespace wpipe {
   double medianbiasfactor(int);
 
   PSD 
-  medianmeanaveragespectrum(const TSeries& data, double fs, int N, Window* w) {
+  medianmeanaveragespectrum(const TSeries& data, double fs, int N, window_api* w) {
     /////////////////////////////////////////////////////////////////////
     //   Checks on input.
     /////////////////////////////////////////////////////////////////////
@@ -34,7 +34,7 @@ namespace wpipe {
     }
 
     // ---- Assign window if (necessary.
-    Window* win = w;
+    window_api* win = w;
     if (!w) win = new Hanning(N);
 
     /////////////////////////////////////////////////////////////////////
