@@ -195,16 +195,19 @@ weventlist::wselect(const weventlist& significants, double dtInflation,
    if (numberOfTiles == 0) return;
 
    //-----------------------------------  Verify that tiles are sorted by E
-   double eLast = significants[0].normalizedEnergy;
-   for (size_t i=1; i<numberOfTiles; i++) {
-      double erg_i = significants[i].normalizedEnergy;
-      if (erg_i > eLast) {
-	 ostringstream msg;
-	 msg << "wselect called with unsorted significants, erg[" << i 
-             << "] = " << erg_i << " prev = " << eLast;
-         throw logic_error(msg.str());
+   if (debugLevel > 2) {
+      cerr << "weventlist::wselect: Checking event list order." << endl;
+      double eLast = significants[0].normalizedEnergy;
+      for (size_t i=1; i<numberOfTiles; i++) {
+	 double erg_i = significants[i].normalizedEnergy;
+	 if (erg_i > eLast) {
+	    ostringstream msg;
+	    msg << "wselect called with unsorted significants, erg[" << i 
+		<< "] = " << erg_i << " prev = " << eLast;
+	    throw logic_error(msg.str());
+	 }
+	 eLast = erg_i;
       }
-      eLast = erg_i;
    }
 
    // initialize event list
@@ -279,8 +282,10 @@ weventlist::wthreshold(const qTransform& qSignal, const wtile& tiling,
       timeRange.resize(2);
       timeRange[0] = -0.5 * (tiling.duration() - 2*tiling.transientDuration());
       timeRange[1] = -timeRange[0];
-   } else {
+   } else if (tRange.size() == 2) {
       timeRange = tRange;
+   } else {
+      error("Time range must be two component vector [tmin tmax].");
    }
 
    dble_vect frequencyRange;
@@ -288,10 +293,12 @@ weventlist::wthreshold(const qTransform& qSignal, const wtile& tiling,
       frequencyRange.resize(2);
       frequencyRange[0] = -dInf;
       frequencyRange[1] = dInf;
-   } else {
+   } else if (fRange.size() == 2) {
       frequencyRange = fRange;
       if (fRange[0] <= 0) frequencyRange[0] = -dInf;
       if (fRange[1] <= 0) frequencyRange[1] =  dInf;
+   } else {
+      error("Frequency range must be two component vector [fmin fmax].");
    }
 
    dble_vect qRange;
@@ -299,8 +306,10 @@ weventlist::wthreshold(const qTransform& qSignal, const wtile& tiling,
       qRange.resize(2);
       qRange[0] = -dInf;
       qRange[1] = dInf;
-   } else {
+   } else if (qRnge.size() <= 2) {
       qRange = qRnge;
+   } else {
+      error("Q range must be scalar or two component vector [Qmin Qmax].");
    }
 
    // if (only a single Q is requested, find nearest Q plane
@@ -310,24 +319,7 @@ weventlist::wthreshold(const qTransform& qSignal, const wtile& tiling,
    }
 
    ////////////////////////////////////////////////////////////////////////////
-   //                      validate command line arguments                   //
-   ////////////////////////////////////////////////////////////////////////////
-
-   // Check for two component range vectors
-   if (timeRange.size() != 2) {
-      error("Time range must be two component vector [tmin tmax].");
-   }
-
-   if (frequencyRange.size() != 2) {
-      error("Frequency range must be two component vector [fmin fmax].");
-   }
-
-   if (qRange.size() > 2) {
-      error("Q range must be scalar or two component vector [Qmin Qmax].");
-   }
-
-   ////////////////////////////////////////////////////////////////////////////
-   //            initialize statistically significant event structure        //
+   //            initialize statistically significant event list             //
    ////////////////////////////////////////////////////////////////////////////
 
    // fill channel names
@@ -380,6 +372,10 @@ weventlist::wthreshold(const qTransform& qSignal, const wtile& tiling,
 	 size_t numberOfSignificants = signal.getNGreater(eventThreshold);
 	 if (numberOfSignificants == 0) continue;
 	 double meanEnergy = signalPlaneRow.meanEnergy;
+	 double amplFact   = meanEnergy / signalPlaneRow.conditioningCoef;
+	 //cout.unsetf(ios_base::floatfield);
+	 //cout << "meanEnergy, ampl-fact = " << meanEnergy << ", " 
+	 //     << amplFact << endl;
 	 if (debugLevel > 2) {
 	    cout << qVal << " " << fVal << " " << numberOfSignificants << endl;
 	 }
@@ -404,8 +400,7 @@ weventlist::wthreshold(const qTransform& qSignal, const wtile& tiling,
 	    if (sig_dvd[ibin] >= eventThreshold) {
 	       evt.t_offset  = double(signal.getBinT(ibin) - _refTime);
 	       evt.normalizedEnergy = sig_dvd[ibin];
-	       evt.amplitude = sqrt((evt.normalizedEnergy - 1.0) * meanEnergy);
-	       evt.incoherentEnergy = 0;
+	       evt.amplitude = sqrt((evt.normalizedEnergy - 1.0) * amplFact);
 	       evec.push_back(evt);
 	    }
 	 }
@@ -781,14 +776,17 @@ weventlist::writeEvent_xml(TrigClient& out) const {
       const wevent& evi(_events[i]);
       Time t0 = _refTime + evi.t_offset;
       Interval dt = evi.duration;
-      // Add a factor of 2 to match omicron
-      //double S2N = sqrt(evi.normalizedEnergy - 1);
-      double S2N = sqrt(2* (evi.normalizedEnergy - 1));
+      // The signal/noise ratio come from the normalized energy as follows:
+      // The nomalized energy is the sum of the signal from two degrees of 
+      // freedom (sin, cosine components), the noise from which add up to 1. 
+      // Thus: snr = sqrt(2*NE - 1)
+      double S2N = sqrt(2*evi.normalizedEnergy - 1);
       trig::SBTrigger wtrig("Omega_C", channelName(), t0 - dt*0.5, dt, 
 			    evi.frequency, evi.bandwidth, S2N);
       wtrig.setPixelCount(evi.cluster_size);
       wtrig.peak_time(t0);
       wtrig.ifo(ifo);
+      wtrig.amplitude(evi.amplitude);
       out.sendTrigger(wtrig);
    }
 }
@@ -843,6 +841,17 @@ weventstack::swap(weventstack& wstak) {
    _lists.swap(wstak._lists);
 }
 
+//======================================  Total events in the stack;
+size_t 
+weventstack::totalEvents(void) const {
+   size_t total = 0;
+   size_t N = _lists.size();
+   for (size_t i=0; i<N; i++) {
+      total += _lists[i].size();
+   }
+   return total;
+}
+
 //======================================  Select event from an event-stack.
 void 
 weventstack::tsort(void) {
