@@ -29,6 +29,7 @@
 #include <iomanip>
 #include <stdexcept>
 #include <algorithm>
+#include <unistd.h>
 
 using namespace std;
 using namespace wpipe;
@@ -85,10 +86,12 @@ wpipe::wplot(const plot_chan& pchan) {
    /////////////////////////////////////////////////////////////////////////////
    //                        process command line arguments                   //
    /////////////////////////////////////////////////////////////////////////////
-   string channelNameString = pchan.channelName;
-   string frameType =         pchan.frameType;
-   int    debugLevel =        pchan.debugLevel;
-
+   str_vect channelName = pchan.channelName;
+   size_t   nChan       = channelName.size();
+   str_vect frameType   = pchan.frameType;
+   dble_vect timeShifts = pchan.timeShifts;
+   int      debugLevel  = pchan.debugLevel;
+   
    // convert string event time and debug level to numbers
    ostringstream outstr;
    outstr << fixed << setprecision(3) << pchan.eventTime.totalS();
@@ -113,8 +116,8 @@ wpipe::wplot(const plot_chan& pchan) {
 
    // load frame file cache
    wframecache frameCache(pchan.frameCacheFile);
-   if (frameCache.empty() && (frameType == "NDS2" || frameType.empty())) {
-      frameType = "NDS2";
+   if (frameCache.empty() && (frameType.empty() || frameType[0] == "NDS2")) {
+      frameType.resize(nChan, "NDS2");
       frameCache.add_group("H-NDS2", 0, 1999999999, 0, pchan.ndsServer);
       frameCache.add_group("G-NDS2", 0, 1999999999, 0, pchan.ndsServer);
       frameCache.add_group("L-NDS2", 0, 1999999999, 0, pchan.ndsServer);
@@ -168,7 +171,7 @@ wpipe::wplot(const plot_chan& pchan) {
    double    searchWindowDuration = pchan.searchWindowDuration;
 
    // standardize channel names
-   channelNameString = strrep(channelNameString, ";", ":");
+   string channelNameString = strrep(channelName[0], ";", ":");
 
    // Plot parameters
    dble_vect plotTimeRanges            = pchan.plotTimeRanges;
@@ -216,11 +219,10 @@ wpipe::wplot(const plot_chan& pchan) {
    //  This has to be fixed for reasonable data I/O
    // read data from frame file
    wlog(debugLevel, 2, "  reading data");
-   double timeShifts = 0;
    tser_vect rawData;
-   wreaddata(frameCache, channelNameString, frameType, 
-	     startTime, stopTime, timeShifts, debugLevel,
-	     rawData);
+   Interval primetime = 1.0;
+   wreaddata(frameCache, channelName, frameType, startTime-primetime,
+	     stopTime, timeShifts, debugLevel, rawData);
 
    // check for read error
    if (rawData.empty()) {
@@ -240,12 +242,14 @@ wpipe::wplot(const plot_chan& pchan) {
       wlog(debugLevel, 1, "  WARNING: channel contains no information");
       return;
    }
+   
+   //--------------------------------  prime the resampler
+   resample.prime_from_data(rawData, primetime, sampleFrequency);
 
    //--------------------------------  Resample data
    wlog(debugLevel, 2, "  resampling data");
-   tser_vect rsamp;
-   rsamp = resample.wresample(rawData, sampleFrequency);
-
+   tser_vect rsamp = resample.wresample(rawData, sampleFrequency);
+   
    // high pass filter and whiten data
    wlog(debugLevel, 2, "  conditioning data");
 
@@ -417,11 +421,12 @@ wpipe::wplot(const plot_chan& pchan) {
       int nPlot = pchan.plotType.size();
       for (int iplot=0; iplot< nPlot; iplot++) {
 	 wfigure figureHandle;
-	 figureHandle.open();
 
 	 if (!pchan.plotFormat.empty()) {
 	    figureHandle.set_format(pchan.plotFormat);
 	 }
+	 
+	 figureHandle.open();
 
 	 if (!pchan.plotSize.empty()) {
 	    int nx = int(pchan.plotSize[0]);
@@ -430,7 +435,15 @@ wpipe::wplot(const plot_chan& pchan) {
 	    figureHandle.set_size(nx, ny);
 	 }
 
+	 if (!pchan.colorMapFile.empty()) {
+	    figureHandle.read_palette(pchan.colorMapFile);
+	 } else if (!pchan.colorMap.empty()) {
+	    figureHandle.set_palette(pchan.colorMap);
+	 }
+	 figureHandle.set_zunits(pchan.zUnits);
+	 
 	 string pltType = pchan.plotType[iplot];
+	 //-----------------------------  Raw time-series
 	 if (pltType == tser_raw) {
 	    figureHandle.wtimeseries(rawData[0], 
 				     centerTime, 
@@ -439,6 +452,7 @@ wpipe::wplot(const plot_chan& pchan) {
 				     channelNameString);
 	 }
 
+	 //-----------------------------  High-passed time-series
 	 else if (pltType == tser_hipas) {
 	    figureHandle.wtimeseries(conditioner.highPassedData(), 
 				     centerTime, 
@@ -446,6 +460,7 @@ wpipe::wplot(const plot_chan& pchan) {
 				     channelNameString);
 	 }
 
+	 //-----------------------------  Whitened time-series
 	 else if (pltType == tser_white) {
 	    figureHandle.wtimeseries(conditioner.whitenedData(), 
 				     centerTime, 
@@ -453,6 +468,7 @@ wpipe::wplot(const plot_chan& pchan) {
 				     channelNameString);
 	 }
 
+	 //-----------------------------  Raw spectrogram
 	 else if (pltType == spec_raw) {
 	    figureHandle.set_smooth(pchan.nSmooth, pchan.smoothOpt);
 	    figureHandle.wspectrogram(rawTransform[0], tiling, 
@@ -462,6 +478,7 @@ wpipe::wplot(const plot_chan& pchan) {
 				      pchan.plotHorizontalResolution);
 	 }
 
+	 //-----------------------------  Whitened spectrogram
 	 else if (pltType == spec_white) {
 	    figureHandle.set_smooth(pchan.nSmooth, pchan.smoothOpt);
 	    figureHandle.wspectrogram(whitenedTransform[0], tiling,
@@ -471,6 +488,7 @@ wpipe::wplot(const plot_chan& pchan) {
 				      pchan.plotHorizontalResolution);
 	 }
 
+	 //-----------------------------  Auto-scaled spectrogram
 	 else if (pltType == spec_auto) {
 	    figureHandle.set_smooth(pchan.nSmooth, pchan.smoothOpt);
 	    figureHandle.wspectrogram(whitenedTransform[0], tiling, 
@@ -480,6 +498,7 @@ wpipe::wplot(const plot_chan& pchan) {
 				      pchan.plotHorizontalResolution);
 	 }
 
+	 //-----------------------------  Raw eventgram
 	 else if (pltType == evnt_raw) {
 	    figureHandle.weventgram(rawEvents[0], tiling, centerTime, 
 				    tRange, plotFrequencyRange, 
@@ -487,6 +506,8 @@ wpipe::wplot(const plot_chan& pchan) {
 				    pchan.plotBandwidthInflation, 
 				    plotNormalizedEnergyRange);
 	 }
+
+	 //-----------------------------  Whitened eventgram
 	 else if (pltType == evnt_white) {
 	    figureHandle.weventgram(whitenedEvents[0], tiling, centerTime,
 				    tRange, plotFrequencyRange, 
@@ -495,6 +516,7 @@ wpipe::wplot(const plot_chan& pchan) {
 				    plotNormalizedEnergyRange);
 	 }
 
+	 //-----------------------------  Auto-scaled eventgram
 	 else if (pltType == evnt_auto) {
 	    figureHandle.weventgram(whitenedEvents[0], tiling, centerTime, 
 				    tRange, plotFrequencyRange, 
@@ -520,6 +542,7 @@ wpipe::wplot(const plot_chan& pchan) {
 	 }
 	 figureHandle.wprintfig(outFile.file_path(Time(0)), 
 				pchan.plotThumbnail);
+	 figureHandle.clear();
       }
    }   // end loop time ranges
 
@@ -541,7 +564,7 @@ plot_chan::plot_chan(void) {
 static const char* plot_types[] = {
       "spectrogram_whitened", "spectrogram_raw", "spectrogram_autoscaled",
       "timeseries_raw", "timeseries_whitened", "timeseries_highpassed", 
-      "eventgram_whitened", "eventgram_raw", "eventgram_autoscaled"};
+      "eventgram_whitened", "eventgram_raw", "eventgram_autoscaled", 0};
 
 
 //======================================  Clear search channel descriptor.
@@ -550,6 +573,7 @@ plot_chan::clear(void) {
    channelName.clear();
    eventTime = Time(0);
    frameType.clear();
+   timeShifts.clear();
    sampleFrequency = 0;
    searchTimeRange = 0;
    searchFrequencyRange.clear();
@@ -561,6 +585,8 @@ plot_chan::clear(void) {
    plotTimeRanges.clear();
    plotFrequencyRange.clear();
    plotNormalizedEnergyRange.clear();
+   colorMap.clear();
+   colorMapFile.clear();
    nSmooth = 0;
    smoothOpt.clear();
    debugLevel = 0;
@@ -571,6 +597,7 @@ void
 plot_chan::def_params(param_list& par) {
    par.add_param("channelName", channelName);
    par.add_param("frameType", frameType);
+   par.add_param("timeShifts", timeShifts);
    par.add_param("frameCacheFile", frameCacheFile);
    par.add_param("ndsServer", ndsServer);
    par.add_param("outDir", outDir);
@@ -601,7 +628,10 @@ plot_chan::def_params(param_list& par) {
    par.add_param("nSmooth", nSmooth);
    par.add_param("smoothOption", smoothOpt);
    par.add_param("plotSize", plotSize);
+   par.add_param("colorMap", colorMap);
+   par.add_param("colorMapFile", colorMapFile);
    par.add_param("plotFormat", plotFormat);
+   par.add_param("zUnits", zUnits);
    par.add_param("debugLevel", debugLevel);
 }
 
@@ -611,7 +641,8 @@ plot_chan::reset(void) {
    clear();
    const double dInf(1.0/0.0);
    channelName.clear();
-   frameType = "NDS2";
+   frameType.resize(1, "NDS2");
+   timeShifts.resize(1, 0.0);
    ndsServer = "nds.ligo.caltech.edu:31200";
    eventTime = Time(0);
    sampleFrequency = 4096;
@@ -648,6 +679,9 @@ plot_chan::reset(void) {
    plotSize.clear();
    plotFormat = "png";
    plotThumbnail = false;
+   colorMap.clear();
+   colorMapFile.clear();
+   zUnits = "NormE";
 
    // limits on number of significant tiles
    maximumSignificants = 10000;
@@ -659,7 +693,36 @@ void
 wpipe::plot_config(int argc, const char* argv[], plot_chan& p) {
    param_list par;
    p.def_params(par);
+   
+   //-----------------------------------  Get init file parameters
+   string parfile;
+   const char* homep = getenv("HOME");
+   if (homep) {
+      parfile = homep;
+      parfile += "/";
+   }
+   parfile += ".wplotrc";
+   if (!access(parfile.c_str(), R_OK)) {
+      ifstream in(parfile.c_str());
+      while (in.good()) {
+	 string line;
+	 getline(in, line);
+	 line = deblank(line);
+	 size_t inx = line.find("#");
+	 if (inx != string::npos) line.erase(inx);
+	 if (!line.empty()) {
+	    inx = line.find_first_of(":=");
+	    // parse configuration line
+	    string parameterName = deblank(line.substr(0, inx));
+	    string parameterValue;
+	    if (inx != string::npos) parameterValue=deblank(line.substr(inx+1));
+	    else                     parameterValue="true";
+	    par.set_param(parameterName, parameterValue);
+	 }
+      }
+   }
 
+   //----------------------------------- Loop over command line arguments
    for (int iarg=1; iarg<argc; iarg++) {
       string cmdstr = argv[iarg];
       if (cmdstr == "--help") {
@@ -680,7 +743,7 @@ wpipe::plot_config(int argc, const char* argv[], plot_chan& p) {
 	      << cmdstr << endl;
 	 error("syntax error processing configuration file");
       }
-    
+
       // parse configuration line
       string parameterName = cmdstr.substr(0, colonIndex);
       string parameterValue = cmdstr.substr(colonIndex + 1);
@@ -688,13 +751,35 @@ wpipe::plot_config(int argc, const char* argv[], plot_chan& p) {
       par.set_param(parameterName, parameterValue);
 
    }      // end loop over command line arguments.
-
+   p.validate();
 }
 
-//======================================  Diaplay channel parameters.
+//======================================  Display channel parameters.
 void
 plot_chan::display(std::ostream& out) const {
    param_list par;
    const_cast<plot_chan*>(this)->def_params(par);
    par.display(out, "  ");
 }
+
+//======================================  Check parameter validity
+void
+plot_chan::validate(void) {
+   //-----------------------------------  See if channel name was specified.
+   size_t nChan = channelName.size();
+   if (nChan == 0) {
+      throw runtime_error("channelName not specified");
+   }
+   //-----------------------------------  Pad out time shifts and frame types
+   if (timeShifts.size() < nChan) timeShifts.resize(nChan, 0.0);
+   if (!frameType.empty() && frameType.size() < nChan) {
+      frameType.resize(nChan, frameType[0]);
+   }
+   //-----------------------------------  Enumerate "all" plot types
+   if (plotType.size() == 1 && plotType[0] == "all") {
+      plotType.clear();
+      for (size_t i=0; plot_types[i] != 0; i++) {
+	 plotType.push_back(plot_types[i]);
+      }
+   }
+}
