@@ -60,9 +60,7 @@ main(int argc, const char* argv[]) {
    }
 
    //------------------------------------  Get the start time.
-   double time = strtod(argv[1], 0);
-   long sec = long(time);
-   Time eventTime = Time(sec, long((time-sec)*1e9));
+   string eventTimeString = argv[1];
 
    std::string configFile = argv[2];
 
@@ -71,7 +69,8 @@ main(int argc, const char* argv[]) {
    if (argc > 4) outDir = argv[4];
 
    if (argc > 5) {
-      generateReport = strtol(argv[5],0,0) > 0;
+      if (string(argv[5]) == "true") generateReport = 1;
+      else                           generateReport = strtol(argv[5],0,0) > 0;
    }
 
    if (argc > 6) {
@@ -79,7 +78,7 @@ main(int argc, const char* argv[]) {
    }
 
    cout << "wscan parameters: " <<endl;
-   cout << "   event time       = " << eventTime << endl;
+   cout << "   event time       = " << eventTimeString << endl;
    cout << "   config file      = " << configFile << endl;
    cout << "   frame cache      = " << frameCacheFile << endl;
    cout << "   output directory = " << outDir << endl;
@@ -87,8 +86,8 @@ main(int argc, const char* argv[]) {
    cout << "   debug level      = " << debugLevel << endl;
 
    try {
-      wscan(eventTime, configFile, frameCacheFile, outDir, generateReport, 
-	    debugLevel);
+      wscan(eventTimeString, configFile, frameCacheFile, outDir,
+	    generateReport, debugLevel);
    }
    catch (exception& e) {
       cerr << "caught exception: " << e.what() << endl;
@@ -143,7 +142,7 @@ insert_scripts(ostream& file) {
 
 //======================================  scan function.
 void
-wpipe::wscan(const Time& eventTime, const std::string& configFile, 
+wpipe::wscan(const std::string& eventTimeString, const std::string& configFile, 
 	     const std::string& frameCacheFile, const std::string& outDir, 
 	     bool generateReport, int debugLevel) {
    const double dInf(1.0/0.0);
@@ -211,9 +210,9 @@ wpipe::wscan(const Time& eventTime, const std::string& configFile,
    }
 
    // convert string event time and debug level to numbers
-   ostringstream outstr;
-   outstr << "0" << fixed << setprecision(9) << eventTime.totalS();
-   string eventTimeString = outstr.str();
+   double time = strtod(eventTimeString.c_str(), 0);
+   long sec = long(time);
+   Time eventTime = Time(sec, long((time-sec)*1e9));
 
    // name of html file
    string htmlFile = "index.html";
@@ -538,19 +537,18 @@ wpipe::wscan(const Time& eventTime, const std::string& configFile,
       /////////////////////////////////////////////////////////////////////////
 
       // extract search configuration for channel
-      string channelNameString = configuration[channelNumber].channelName;
-      string frameType = configuration[channelNumber].frameType;
-      double sampleFrequency = configuration[channelNumber].sampleFrequency;
-      double timeRange = configuration[channelNumber].searchTimeRange;
+      str_vect  channelName = configuration[channelNumber].channelName;
+      string    channelNameString = strrep(channelName[0], ";", ":");
+      str_vect  frameType = configuration[channelNumber].frameType;
+      dble_vect timeShifts = configuration[channelNumber].timeShifts;
+      double    sampleFrequency = configuration[channelNumber].sampleFrequency;
+      double    timeRange = configuration[channelNumber].searchTimeRange;
       dble_vect frequencyRange=configuration[channelNumber].searchFrequencyRange;
       dble_vect qRange = configuration[channelNumber].searchQRange;
       double maximumEnergyLoss = configuration[channelNumber].searchMaximumEnergyLoss;
       double searchWindowDuration = configuration[channelNumber].searchWindowDuration;
       bool alwaysPlotFlag = configuration[channelNumber].alwaysPlotFlag;
 
-      // standardize channel names
-      channelNameString = strrep(channelNameString, ";", ":");
-
       // Plot parameters
       dble_vect plotTimeRanges;
       dble_vect plotFrequencyRange;
@@ -566,28 +564,6 @@ wpipe::wscan(const Time& eventTime, const std::string& configFile,
       if (debugLevel >= 1) {
 	 cout << "processing channel " << channelNameString << endl;
       }
-
-      // censor channels used for "blind" injections
-      if (channelNameString == "H1:LSC-ETMY_EXC_DAQ" ||
-	  channelNameString == "H2:LSC-ETMY_EXC_DAQ" || 
-	  channelNameString == "L1:LSC-ETMY_EXC_DAQ") {
-	 if (debugLevel >= 1) {
-	    cout << "  WARNING: " << channelNameString 
-		 << " censored for 'blind' injections." << endl;
-	 }
-	 continue;
-      }
-
-      // override censor for channels used in "blind" injections
-      if (channelNameString == "H1:LSC-ETMY_EXC_DAQ!" || 
-	  channelNameString == "H2:LSC-ETMY_EXC_DAQ!" ||
-	  channelNameString == "L1:LSC-ETMY_EXC_DAQ!") {
-	 channelNameString = strrep(channelNameString, "!", "");
-	 if (debugLevel >= 1) {
-	    cout << "  WARNING: overriding censor for " << channelNameString 
-		 << "." << endl;
-	 }
-      }
   
       // find closest sample time to event time
       Time centerTime = Time(eventTime.getS());
@@ -630,9 +606,8 @@ wpipe::wscan(const Time& eventTime, const std::string& configFile,
       //  This has to be fixed for reasonable data I/O
       // read data from frame file
       wlog(debugLevel, 2, "  reading data");
-      double timeShifts = 0;
       tser_vect rawData;
-      wreaddata(frameCache, channelNameString, frameType, 
+      wreaddata(frameCache, channelName, frameType, 
 		startTime, stopTime, timeShifts, debugLevel,
 		rawData);
 
@@ -847,14 +822,27 @@ wpipe::wscan(const Time& eventTime, const std::string& configFile,
 
 	 expfmt(mostSignificantNormalizedEnergy, oss);
 	 string mostSignificantNormalizedEnergyHtml =  oss.str();
-	 oss.str("");
+	 oss.str("");   Time eventTime = Time(sec, long((time-sec)*1e9));
+
+   // name of html file
+   string htmlFile = "index.html";
+
+   // name of text summary file
+   string textSummaryFile = "summary.txt";
+
+   // name of xml summary file
 
 	 expfmt(mostSignificantAmplitude, oss);
 	 string mostSignificantAmplitudeHtml =  oss.str();
 	 oss.str("");
 
-	 oss << fixed << setprecision(1) 
-	     << sqrt(2 * mostSignificantNormalizedEnergy);
+#ifdef OMEGA_MATLAB_BUGS
+	 double snrsq  = 2 * mostSignificantNormalizedEnergy;
+#else
+	 double snrsq  = 2 * mostSignificantNormalizedEnergy - 1;
+	 if (snrsq < 0) snrsq = 0;
+#endif
+	 oss << fixed << setprecision(1) << sqrt(snrsq);
 	 string mostSignificantSNRHtml =  oss.str();
 	 oss.str("");
 
@@ -936,6 +924,12 @@ wpipe::wscan(const Time& eventTime, const std::string& configFile,
 	 //                     begin loop over time ranges                   //
 	 ///////////////////////////////////////////////////////////////////////
 
+	 //  Set figure style parameters
+	 if (!configuration[channelNumber].colorMap.empty()) {
+	    figureHandle.set_palette(configuration[channelNumber].colorMap);
+	 }
+	 
+	 
 	 // begin loop over time ranges
 	 for (size_t iPlTiR=0; iPlTiR < plotTimeRanges.size(); iPlTiR++) {
 	    double plotTimeRange =  plotTimeRanges[iPlTiR];
@@ -1248,6 +1242,8 @@ search_chan::clear(void) {
    plotFrequencyRange.clear();
    plotNormalizedEnergyRange.clear();
    alwaysPlotFlag = false;
+   colorMap.clear();
+   zUnits.clear();
 }
 
 //======================================  Scan the configuration.
@@ -1354,15 +1350,19 @@ wpipe::scan_config(const std::string& file, const search_chan& def,
       if (configurationLine == "}") {
 
 	 // validate channel configuration
-	 if (temp_chan.channelName.empty()) {
+	 size_t nChan = temp_chan.channelName.size();
+	 if (!nChan) {
 	    cerr << "channel name not specified for channel number " 
 		 << channelNumber << endl;
 	    error("channel name not specified");
 	 }
-	 if (temp_chan.frameType.empty()) {
-	    cerr << "frame type not specified for channel number " 
+	 if (temp_chan.frameType.size() < nChan) {
+	    cerr << "frame type(s) not specified for channel number " 
 		 << channelNumber << endl;
-	    error("frame type not specified");
+	    error("frame type(s) not specified");
+	 }
+	 if (temp_chan.timeShifts.size() < nChan) {
+	    temp_chan.timeShifts.resize(nChan, 0.0);
 	 }
 	 if (temp_chan.sampleFrequency == 0) {
 	    cerr << "WSCAN:incompleteConfiguration" 
@@ -1469,9 +1469,11 @@ wpipe::scan_config(const std::string& file, const search_chan& def,
 
       // assign parameters based on name
       if (parameterName == "channelName") {
-	 temp_chan.channelName = unquote(parameterValue);
+	 temp_chan.channelName = eval_str(parameterValue);
       } else if (parameterName == "frameType") {
-	 temp_chan.frameType = unquote(parameterValue);
+	 temp_chan.frameType = eval_str(parameterValue);
+      } else if (parameterName == "timeShifts") {
+	 temp_chan.timeShifts = eval(parameterValue);
       } else if (parameterName ==  "sampleFrequency") {
 	 temp_chan.sampleFrequency = strtod(parValue, 0);
       } else if (parameterName ==  "searchTimeRange") {
@@ -1502,7 +1504,14 @@ wpipe::scan_config(const std::string& file, const search_chan& def,
 	 } else {
 	    temp_chan.alwaysPlotFlag = strtol(parValue,0,0)!=0;
 	 }
-      } else {
+      }
+      else if (parameterName ==  "colorMap") {
+	 temp_chan.colorMap=parameterValue;
+      }
+      else if (parameterName ==  "zUnits") {
+	 temp_chan.zUnits=parameterValue;
+      }
+      else {
 	 cerr << "unknown configuration parameter " << parameterName << endl;
 	 error("unknown configuration parameter");
       }
@@ -1514,35 +1523,31 @@ wpipe::scan_config(const std::string& file, const search_chan& def,
    configurationFID.close();
 }
 
-//======================================  Diaplay a numeric vector parameter.
-static void
-display_dble_vect(const dble_vect& dv, std::ostream& out) {
-   size_t N = dv.size();
-   out << "[";
-   for (size_t i=0; i<N; i++) {
-      if (i) out << " ";
-      out << dv[i];
-   }
-   out << "]" << endl;
-}
-
 //======================================  Diaplay channel parameters.
 void
 search_chan::display(std::ostream& out) const {
-   out << "channelName:               " << channelName << endl;
-   out << "frameType:                 " << frameType << endl;
+   out << "channelName:               ";
+   wpipe::display(channelName, out) << endl;
+   out << "frameType:                 ";
+   wpipe::display(frameType, out) << endl;
+   out << "timeShifts:                ";
+   wpipe::display(timeShifts, out) << endl;
    out << "sampleFrequency:           " << sampleFrequency << endl;
    out << "searchTimeRange:           " << searchTimeRange << endl;
    out << "searchFrequencyRange:      "; 
-   display_dble_vect(searchFrequencyRange, out);
-   out << "searchQRange:              "; display_dble_vect(searchQRange, out);
+   wpipe::display(searchFrequencyRange, out) << endl;
+   out << "searchQRange:              ";
+   wpipe::display(searchQRange, out) << endl;
    out << "searchMaximumEnergyLoss:   " << searchMaximumEnergyLoss << endl;
    out << "whiteNoiseFalseRate:       " << whiteNoiseFalseRate << endl;
    out << "searchWindowDuration:      " << searchWindowDuration << endl;
-   out << "plotTimeRange:             "; display_dble_vect(plotTimeRanges, out);
-   out << "plotFrequencyRange:        "; 
-   display_dble_vect(plotFrequencyRange, out);
+   out << "plotTimeRange:             ";
+   wpipe::display(plotTimeRanges, out) << endl;
+   out << "plotFrequencyRange:        ";
+   wpipe::display(plotFrequencyRange, out) << endl;
    out << "plotNormalizedEnergyRange: ";
-   display_dble_vect(plotNormalizedEnergyRange, out);
+   wpipe::display(plotNormalizedEnergyRange, out) << endl;
    out << "alwaysPlotFlag:            " << boolstr_tf(alwaysPlotFlag) << endl;
+   out << "colorMap:                  " << colorMap << endl;
+   out << "zUnits:                    " << zUnits << endl;
 }
