@@ -102,6 +102,7 @@ wstream_chan::wstream_chan(const wparameters& par, int dbugLvl)
 //======================================  wstream_chan destructor.
 wstream_chan::~wstream_chan(void) {
    if (oscList) delete oscList;
+   oscList = 0;
 }
 
 //======================================  book a channel in th appropriate mdacc
@@ -154,6 +155,7 @@ wstream_chan::addChannels(MultiDacc& md) {
 	 id = 0;
       }
       oscList = new OperStateCondList(*(md.getDacc(id)));
+      // oscList->setDebug(debugLevel);
       oscList->readConfig(pars.oscFile.c_str());
    }
    
@@ -171,6 +173,9 @@ wstream_chan::addChannels(MultiDacc& md) {
 	 bookInput(md, name, pars.injectionTypes[i]);
       }
    }
+
+   //-----------------------------------  Print channel summary
+   if (debugLevel > 1) md.list(cout);
 }
 
 //======================================  Add output files.
@@ -575,13 +580,18 @@ wstream_chan::read_data(MultiDacc& md, const Time& startTime, double duration) {
    else           wlog(debugLevel, 1, "  no OSC conditions");
    for (size_t i=0; i<nOscConds; i++) {
       int inx = osc_index[i];
-      if (inx < 0 || !oscList) {
+      if (inx < 0) {
 	 if (debugLevel > 1) {
 	    cout << "   channel[" << i << "] no osc requested." << endl;
          }
 	 validChannels[i] = true;
       }
 
+      //--------------------------------  Check OSC list is defined.
+      else if (!oscList) {
+	 error("OSC list not created!");
+      }
+
       //--------------------------------  Was condition already evaluated?
       else if (inx < int(i)) {
 	 if (debugLevel > 1) {
@@ -593,11 +603,21 @@ wstream_chan::read_data(MultiDacc& md, const Time& startTime, double duration) {
       //--------------------------------  Fill data with the evaluated condition
       else {
 	 Time tFill = md.getFillTime();
-	 Interval stride = md.getStride();
+	 Interval stride = md.getFillStride();
 	 size_t nsample = size_t(stride / pars.oscStride);
 	 DVectI idata;
-	 idata.replace_with_zeros(0, 0, nsample);
+	 if (!oscList->defined(pars.oscConditions[inx])) {
+	    cerr << "*** Error *** OSC condition: " << pars.oscConditions[inx]
+		 << " is not defined!" << endl;
+	 }
+	 idata.replace_with_zeros(0, idata.size(), nsample);
 	 if (oscList->satisfied(pars.oscConditions[inx])) idata += 1;
+	 if (debugLevel > 1) {
+	    cout << "OSC condition: " << pars.oscConditions[inx]
+		 << " satisfied = " << idata[0] << " for gps " << tFill.getS()
+		 << "[" << double(stride) << "]" << endl;
+	 }
+
 	 TSeries ts(tFill, Interval(pars.oscStride), idata);
 
 	 //--------------------------------  Remove data before this stride.
@@ -607,14 +627,17 @@ wstream_chan::read_data(MultiDacc& md, const Time& startTime, double duration) {
 	    osc_data[i].eraseStart(tDelete);
 	 }
 
+	 //--------------------------------  Set the osc data series if no data
 	 if (osc_data[i].empty()) {
 	    osc_data[i] = ts;
-	 } else {
-	    osc_data[i].Append(ts);
+	 }
+	 //--------------------------------  Otherwise append
+	 else if (osc_data[i].Append(ts)) {
+	    error("Unaligned OSC time-series");
 	 }
 	 if (osc_data[i].getMinimum() == 1.0 &&
 	     osc_data[i].getMaximum() == 1.0) {
-	    validChannels[i] =true;
+	    validChannels[i] = true;
 	 } else {
 	    validChannels[i] = false;
 	 }
@@ -763,8 +786,9 @@ wstream_chan::start_chunk(void) {
    size_t nOscCond = pars.oscConditions.size();
    if (nOscCond > nChan) {
       error("Invalid number of state channels");
-   }   
-   if (osc_data.size() != nOscCond) osc_data.resize(nOscCond);
+   }
+   osc_data.clear();
+   if (nOscCond) osc_data.resize(nOscCond);
 }
 
 //======================================  Thread entry point
