@@ -326,12 +326,12 @@ size_t
 wtile::nearest_plane(double qTest) const {
    size_t pBest = 0;
    if (_numberOfPlanes != 0 && qTest > 0) {
-      double dq = abs(log(planes(0).q/qTest));
+      double dq = fabs(log(_planes[0].q/qTest));
       for (int i=1; i < _numberOfPlanes; i++) {
-	 double dqi = abs(log(_planes[i].q/qTest));
+	 double dqi = fabs(log(_planes[i].q/qTest));
 	 if (dqi < dq) {
 	    pBest = i;
-	    dqi = dq;
+	    dq = dqi;
 	 }
       }
    }
