@@ -66,7 +66,7 @@
 
 #ifndef __CINT__
 
-#define PIDCVSHDR "$Header: https://redoubt.ligo-wa.caltech.edu/svn/gds/trunk/Monitors/PlaneMon/PlaneMon.cc 7203 2014-10-23 15:32:52Z john.zweizig@LIGO.ORG $"
+#define PIDCVSHDR "$Header: https://redoubt.ligo-wa.caltech.edu/svn/gds/trunk/Monitors/PlaneMon/PlaneMon.cc 7624 2016-05-13 08:11:59Z john.zweizig@LIGO.ORG $"
 #define PIDTITLE  "PlaneMon: Airplane Detection Monitor"
 #include "ProcIdent.hh"
 
@@ -2062,46 +2062,49 @@ void PlaneMon::fitcurve(const int& i, const Time& t) {
 //======================================  Generate Segment
 void PlaneMon::genSegment(TrigClient& tc) {
     if(mLoc.compare("L")==0 && mCandidate) {
-      trig::Segment s("DMT-AIRCRAFT_VERY_LIKELY", 1, mArchiveTime, mArchiveTime+(Interval)120);
+      trig::Segment s("DMT-AIRCRAFT_VERY_LIKELY", 1, mArchiveTime, 
+		      mArchiveTime + Interval(120));
       s.setIfos("L1");
       lmsg::error_type rc = tc.sendSegment(s);
       if(rc) {
-        cout << "Error sending segment with code " << rc << endl;
+        lmsg::put_lmsg_error("[PlaneMon] Error sending segment", rc);
       }
     } else if(mLoc.compare("L")==0 && !mCandidate) {
-      trig::Segment s("DMT-AIRCRAFT_LIKELY", 1, mArchiveTime, mArchiveTime+(Interval)120);
+      trig::Segment s("DMT-AIRCRAFT_LIKELY", 1, mArchiveTime, 
+		      mArchiveTime + Interval(120));
       s.setIfos("L1");
       lmsg::error_type rc = tc.sendSegment(s);
       if(rc) {
-        cout << "Error sending segment with code " << rc << endl;
+        lmsg::put_lmsg_error("[PlaneMon] Error sending segment", rc);
       }
     } else if(mLoc.compare("H")==0 && mCandidate) {
-      trig::Segment s("DMT-AIRCRAFT_VERY_LIKELY", 1, mArchiveTime, mArchiveTime+(Interval)120);
+      trig::Segment s("DMT-AIRCRAFT_VERY_LIKELY", 1, mArchiveTime, 
+		      mArchiveTime + Interval(120));
       s.setIfos("H1");
       lmsg::error_type rc = tc.sendSegment(s);
       if(rc) {
-        cout << "Error sending segment with code " << rc << endl;
+        lmsg::put_lmsg_error("[PlaneMon] Error sending segment", rc);
       }
       /* Commenting out H2 segments - no longer needed
       s.setIfos("H2");
       rc = tc.sendSegment(s);
       if(rc) {
-        cout << "Error sending segment with code " << rc << endl;
+        lmsg::put_lmsg_error("[PlaneMon] Error sending segment", rc);
       }
       */
     } else if(mLoc.compare("H")==0 && !mCandidate) {
       trig::Segment s("DMT-AIRCRAFT_LIKELY", 1, mArchiveTime, 
-		      mArchiveTime+(Interval)120);
+		      mArchiveTime + Interval(120));
       s.setIfos("H1");
       lmsg::error_type rc = tc.sendSegment(s);
       if(rc) {
-        cout << "Error sending segment with code " << rc << endl;
+        lmsg::put_lmsg_error("[PlaneMon] Error sending segment", rc);
       }
       /* Commenting out H2 segments - no longer needed
       s.setIfos("H2");
       rc = tc.sendSegment(s);
       if(rc) {
-        cout << "Error sending segment with code " << rc << endl;
+        lmsg::put_lmsg_error("[PlaneMon] Error sending segment", rc);
       }
       */
     } else {
