@@ -6,7 +6,7 @@
 //
 //   Added DQ flags in March 2009 - Keith Riles (kriles@umich.edu)
 //
-//   $Id: SenseMonitor.cc 7395 2015-06-19 01:47:14Z john.zweizig@LIGO.ORG $
+//   $Id: SenseMonitor.cc 7637 2016-06-10 14:57:45Z john.zweizig@LIGO.ORG $
 //
 ///////////////////////////////////////////////////////////////////////////
 
@@ -15,7 +15,7 @@
 //---------- Time (sec) in lock between each dump of a calibrated AS_Q PSD.
 #define CalPSD_Dump_Period 1000  
 //---------- Time (sec) over which to display range histories on DMTViewer.
-#define DMTViewer_Period 43200  
+#define DMTViewer_Period 86400  
 
 
 //----- Include this first
@@ -25,6 +25,7 @@
 #include <stdlib.h>
 #include <iomanip>
 #include <sstream>
+#include <vector>
 
 //----- Include LIGO-specific headers
 #include "Dacc.hh"
@@ -43,6 +44,8 @@
 #include "html/table.hh"
 #include "html/text.hh"
 #include "TrigRslt.hh"
+#include "ReadTrend.hh"
+
 //----- Windows
 #include "Hamming.hh"
 #include "Hanning.hh"
@@ -56,7 +59,7 @@
 
 //-->  The next three lines are needed if you are going to generate triggers.
 //     The descriptive title in PIDTITLE should the monitor function.
-#define PIDCVSHDR "$Header: https://redoubt.ligo-wa.caltech.edu/svn/gds/trunk/Monitors/SenseMonitor/SenseMonitor.cc 7395 2015-06-19 01:47:14Z john.zweizig@LIGO.ORG $"
+#define PIDCVSHDR "$Header: https://redoubt.ligo-wa.caltech.edu/svn/gds/trunk/Monitors/SenseMonitor/SenseMonitor.cc 7637 2016-06-10 14:57:45Z john.zweizig@LIGO.ORG $"
 #define PIDTITLE  "Inspiral range monitor"
 #include "ProcIdent.hh"
 
@@ -341,55 +344,13 @@ SenseMonitor::SenseMonitor(int argc, const char *argv[])
         amplvalues = new double[tserieslength];
         alphavalues = new double[tserieslength];
         alphabetavalues = new double[tserieslength];
-        //---------- Initialize TSeries using data from previous run, if possible.
-        ifstream dmtviewer_data;
-        dmtviewer_data.open(mOutput.dmtviewer_file_name.c_str());
-        if (dmtviewer_data.fail()) {
 
-	  if (Debug >= 2) { 
-	    cout << "In dmtviewer_data.fail true block" << endl;
-	  }
-
-            for (int i=0; i<tserieslength; i++) {
-                calvalues[i] = -1.0;
-                amplvalues[i] = -1.0;
-                alphavalues[i] = -1.0;
-                alphabetavalues[i] = -1.0;
-            }
-        } else {
-
-	  if (Debug >= 2) { 
-	    cout << "In dmtviewer_data.fail false block" << endl;
-	  }
-            //:KLUDGE:  This could go wrong if there is a long time gap between runs of program.
-            int i=0;
-            while (i<tserieslength && dmtviewer_data >> calvalues[i]) { 
-                dmtviewer_data >> amplvalues[i];
-                dmtviewer_data >> alphavalues[i];
-                dmtviewer_data >> alphabetavalues[i];
-                ++i;
-            }
-            //----- Reset latest values to -2 to indicate that histories have been read from a file.
-            calvalues[i-1] = -2.0;
-            amplvalues[i-1] = -2.0;
-            alphavalues[i-1] = -2.0;
-            alphabetavalues[i-1] = -2.0;
-            //----- Check that our new histories are of the desired length.
-            if (i != tserieslength) {
-                cerr << "SenseMonitor WARNING:  Stored DMTViewer history from previous run not of ";
-                cerr << "required length.  Re-initializing DMTViewer histories.\n";
-                for (int i=0; i<tserieslength; i++) {
-                    calvalues[i] = -1.0;
-                    amplvalues[i] = -1.0;
-                    alphavalues[i] = -1.0;
-                    alphabetavalues[i] = -1.0;
-                }
-            }
-        }
-        dmtviewer_data.close();
-
-	if (Debug >= 2) { 
-	  cout << "Done with dmtviewer_data_close()" << endl;
+	// get history from trends. preset with -1.0
+	for (int i=0; i<tserieslength; i++) {
+	    calvalues[i] = -1.0;
+	    amplvalues[i] = -1.0;
+	    alphavalues[i] = -1.0;
+	    alphabetavalues[i] = -1.0;
 	}
 
         //-------- Set window type and intialize PSD object.
@@ -519,6 +480,7 @@ SenseMonitor::~SenseMonitor()
     if (!fatal_error) {
         //---------- Export DMTViewer data to a file to be read if monitor is restarted.
         //           Name of file set elsewhere; uses mOutput.html_dir if defined.
+#if 0
         ofstream dmtviewer_data;
         dmtviewer_data.open(mOutput.dmtviewer_file_name.c_str());
         for (int i=0; i<tserieslength; i++) {
@@ -529,7 +491,7 @@ SenseMonitor::~SenseMonitor()
             dmtviewer_data << endl; 
         } 
         dmtviewer_data.close();
-
+#endif
         //----- Pointers assigned in SenseMonitor::SenseMonitor().
         //---------- These pointers are always assigned.
         delete[] calvalues;
@@ -632,6 +594,14 @@ SenseMonitor::ProcessData(void) {
 	}
 	ts_asq = mDecimate(raw_asq);
 
+        //--------- Read waveform file that contains the integrand
+        FSpectrum waveform;
+        if (not Run_Par.waveform_file_name.empty())
+        {
+            waveform = ReadWaveform(Run_Par.waveform_file_name, PSDed_data.getLowFreq(),
+                                    PSDed_data.getFStep());
+        }
+
 	//---------- Things to do once only.
     	if (NStride == 0) {
 
@@ -659,6 +629,22 @@ SenseMonitor::ProcessData(void) {
 	        mRangeSumm.init(summary_name, time);
 	    }
 
+	    ReadTrend::string_vect rt_name;
+	    rt_name.push_back(Trend_Channel("EFFECTIVE_RANGE_MPC"));
+	    rt_name.push_back(Trend_Channel("CAL_LINE_AMPL_ASQ"));
+	    rt_name.push_back(Trend_Channel("CAL_CAV_FAC"));
+	    rt_name.push_back(Trend_Channel("CAL_OLOOP_FAC"));
+	    ReadTrend::type_vect rt_type(4, ReadTrend::kMean);
+	    ReadTrend::tseries_vect rt_data(4);
+	    string rt_prefix = Run_Par.IFO.substr(0,1) + "-"
+		             + Run_Par.MonitorName;
+	    ReadTrend rt(getenv("DMTRENDOUT"), rt_prefix.c_str());
+	    Interval dT = Run_Par.T * tserieslength;
+	    rt.getSeries(rt_name, rt_type, history_start, dT, rt_data);
+	    rt_data[0].getData(tserieslength, calvalues);
+	    rt_data[1].getData(tserieslength, amplvalues);
+	    rt_data[2].getData(tserieslength, alphavalues);
+	    rt_data[3].getData(tserieslength, alphabetavalues);
             //---------- Write histories to DMTViewer.
             //----- Initialize and publish history of
 	    //      range with calibration line.
@@ -691,8 +677,16 @@ SenseMonitor::ProcessData(void) {
 	    DMTViewer_name = DMTView_Channel("CALIBRATED NOISE SPECTRUM");
 	    serveData(DMTViewer_name.c_str(), &strain_noise_ampl);
 
-	    //---------- Generate fake f_7_3 for DMTViewer initialization. 
-	    integrand(f_7_3, PSDed_data);
+            //---------- Generate fake f_7_3 or integrand for DMTViewer
+            //           initialization
+            if ( Run_Par.waveform_file_name.empty() )
+            {
+                integrand(f_7_3, PSDed_data);
+            }
+            else
+            {
+                integrand_waveform(f_7_3, PSDed_data, waveform);
+            }
 	    DMTViewer_name = DMTView_Channel("RANGE INTEGRAND (ARB UNITS)");
 	    serveData(DMTViewer_name.c_str(), &f_7_3);
         }
@@ -720,12 +714,26 @@ SenseMonitor::ProcessData(void) {
 	    R_Dat.beta = mCalibrate->GetBeta();
 	    R_Dat.ampl = mCalibrate->GetLineAmplitudeASQ();
 
-	    //---------- Compute integrand of range integral ([PSD*f^7/3]^-1).
-	    integrand(f_7_3, PSDed_data);
+            //---------- Compute integrand of range integral ([PSD*f^7/3]^-1).
+            if ( Run_Par.waveform_file_name.empty() )
+            {
+                integrand(f_7_3, PSDed_data);
+            }
+            else
+            {
+                integrand_waveform(f_7_3, PSDed_data, waveform);
+            }
 
-	    //---------- Calculate distance at which detector is sensitive
+            //---------- Calculate distance at which detector is sensitive
             R_Dat.range = integrate(f_7_3, Run_Par.l_freq, Run_Par.h_freq);
-	    R_Dat.range = range(R_Dat.range, (Run_Par.mChannel).c_str());
+            if (Run_Par.waveform_file_name.empty())
+            {
+                R_Dat.range = range(R_Dat.range, (Run_Par.mChannel).c_str());
+            }
+            else
+            {
+                R_Dat.range = range_waveform(R_Dat.range, (Run_Par.mChannel).c_str());
+            }
 
 	    //---------- Compute strain_noise_ampl from PSDed_data, for DMTViewer.
             //           This code effectively duplicates the Calibrated PSD dump code.
@@ -1616,6 +1624,8 @@ SenseMonitor::Configure(int argc, const char *argv[])
                 Run_Par.mChannel = argv[++i];
 	    } else if(argi == "-readalphabeta") {
                 Run_Par.genCalParam = false; 
+            } else if (argi == "-waveformfile") {
+                Run_Par.waveform_file_name = argv[++i];
 	    } else if(argi == "-screen") {
                 mOutput.screen_output = true;
      	        mOutput.write_log = false;
@@ -1787,6 +1797,34 @@ SenseMonitor::VerifyParameters(Parameter& Run_Par, bool& fatal_error)
     }
 }
 
+//====================================== Read waveform file.
+//std::string
+FSpectrum
+SenseMonitor::ReadWaveform(const std::string& filename, double low_freq, double df) const {
+
+    //---------- Initializations.
+    FSpectrum waveform;
+    vector <float> freq;
+    vector <float> val;
+
+    //---------- Read columns.
+    ifstream in(filename);
+    while (!in.eof())
+    {
+        float tmp_freq, tmp_val;
+        in >> tmp_freq >> tmp_val;
+        freq.push_back(tmp_freq);
+        val.push_back(tmp_val);
+    }
+
+    //---------- Set low-frequency cutoff and frequency step.
+    waveform.clear(low_freq, df);
+
+    //---------- Set the waveform values.
+    waveform.setData(val.size(), val.data());
+
+    return waveform;
+};
 
 //====================================== Write current range, etc to specified
 //                                       output stream.
