@@ -22,11 +22,7 @@ inline int isfinite(double x) {return finite(x);}
 #endif
 #include <fstream>
 #include "PConfig.h"
-#ifdef __GNU_STDC_OLD
-#include <gnusstream.h>
-#else
 #include <sstream>
-#endif
 #include <sys/time.h>
 #include <cstdlib>
 
@@ -34,7 +30,7 @@ using namespace std;
 using namespace trig;
 using namespace lmsg;
 
-static const char* TrigMgrVsn = "$Header: https://redoubt.ligo-wa.caltech.edu/svn/gds/trunk/Services/TrigMgr/TrigMgr.cc 7362 2015-05-01 00:34:20Z john.zweizig@LIGO.ORG $";
+static const char* TrigMgrVsn = "$Header: https://redoubt.ligo-wa.caltech.edu/svn/gds/trunk/Services/TrigMgr/TrigMgr.cc 7582 2016-03-01 22:25:48Z john.zweizig@LIGO.ORG $";
 
 static const unsigned int idLength = 26;
 
