@@ -12,14 +12,8 @@
 #include "framecpp/FrRawData.hh"
 #include "framecpp/FrVect.hh"
 #include "FrVectRef.hh"
-#include "ldas/ldasconfig.hh"
-#if LDAS_VERSION_NUMBER < 200000
-#include "general/gpstime.hh"
-using General::GPSTime;
-#else
 #include "framecpp/GPSTime.hh"
 using FrameCPP::GPSTime;
-#endif
 
 #include "TSeries.hh"
 #include "FSeries.hh"
@@ -281,7 +275,7 @@ Channel::FillSeries(fradcdata_pointer adc, const Time& t0,
     //----------------------------------  Get data  pointers.
     if (adc->RefData().empty()) return -3;
     FrVectRef vect(adc->RefData().front());
-    if (vect.getUnits() != adc->GetUnits()) {
+    if (!vect.getUnits().empty() && vect.getUnits() != adc->GetUnits()) {
 	cerr << "Mismatched units in FrAdcData: " << adc->GetName() << endl;
     }
 
