@@ -15,15 +15,9 @@
 #include "framecpp/FrameCPP.hh"
 #include "framecpp/FrameH.hh"
 #include "framecpp/FrHistory.hh"
-#if LDAS_VERSION_NUMBER < 200000
-#include "general/gpstime.hh"
-using General::GPSTime;
-#else
 #include "framecpp/GPSTime.hh"
 using FrameCPP::GPSTime;
-#endif
 #include "framecpp/Common/Verify.hh"
-#if LDAS_VERSION_NUMBER >= 109360
 #include "framecpp/Common/IOStream.hh"
 using FrameCPP::Common::FrameBuffer;
 template<class T>
@@ -31,7 +25,6 @@ T*
 stream_ptr(FrameCPP::Common::FrameBufferInterface* p) {
     return dynamic_cast< FrameBuffer<T>* >(p);
 }
-#endif
 #include "framecpp/IFrameStream.hh"
 #include "FrVectRef.hh"
 #include "FrStatDataRef.hh"
@@ -44,13 +37,8 @@ stream_ptr(FrameCPP::Common::FrameBufferInterface* p) {
 #include <fstream>
 typedef std::filebuf filebuffer_type;
 #else
-  #if LDAS_VERSION_NUMBER < 200000
-    #include "general/fstream.hh"
-    typedef General::filebuf filebuffer_type;
-  #else
-    #include "ldastoolsal/fstream.hh"
-    typedef LDASTools::AL::filebuf filebuffer_type;
-  #endif
+#include "ldastoolsal/fstream.hh"
+typedef LDASTools::AL::filebuf filebuffer_type;
 #endif
 
 using FrameCPP::FrAdcData;
@@ -79,32 +67,20 @@ getGPS(const Time t) {
 //======================================  Default (Null) constructor.
 DaccIn::DaccIn(void) 
   : mDebug(0), mT0(0), mFramesInFile(0), mFrameNo(0), mTOCMode(s_TOC),
- 
-#if LDAS_VERSION_NUMBER < 109360
-    mSbuf(0), mStream(0), 
-#else
-    mFrameBuf(0),
-#endif
+     mFrameBuf(0),
 #ifdef FCPP_SHARED_PTRS
     mReader(0), mTotalFrames(0)
 #else
     mReader(0), mFrame(0), mTotalFrames(0)
 #endif
 {
-#if LDAS_VERSION_NUMBER > 119020
     FrameCPP::Initialize();
-#endif
     mSource = s_none;
 }
 
 //======================================  Construct and connect to source.
 DaccIn::DaccIn(IFrameStream* reader) 
-  : mDebug(0), mT0(0), mFramesInFile(0), mFrameNo(0), 
-#if LDAS_VERSION_NUMBER < 109360
-     mSbuf(0), mStream(0),
-#else
-    mFrameBuf(0),
-#endif
+  : mDebug(0), mT0(0), mFramesInFile(0), mFrameNo(0), mFrameBuf(0),
 #ifdef FCPP_SHARED_PTRS
     mReader(reader), mTotalFrames(0)
 #else
@@ -113,9 +89,7 @@ DaccIn::DaccIn(IFrameStream* reader)
 {
     mSource = s_none;
     setTOCMode(false);
-#if LDAS_VERSION_NUMBER > 119020
     FrameCPP::Initialize();
-#endif
 }
 
 //======================================  Destructor.
@@ -128,12 +102,8 @@ void
 DaccIn::setBuffer(int N) {
 #ifndef DMTOFFLINE
     if (!isOnline()) return;
-#if LDAS_VERSION_NUMBER < 109360
-    dynamic_cast<iSMbuf*>(mSbuf)->setBCount(N);
-#else
     stream_ptr<iSMbuf>(mFrameBuf)->setBCount(N);
 #endif
-#endif
 }
 
 //======================================  Set the TOC mode
@@ -183,11 +153,7 @@ DaccIn::getStaticData(const std::string& name,  const std::string& det,
 	if (mDebug > 1) {
 	    cerr << "Name  gps-start gps-stop version" << endl;
 	    for (int i=0; i<N; i++) {
-#if LDAS_VERSION_NUMBER < 109360
-	        const FrStatData* p = mStatData[i];
-		cerr << p->GetName()    << " " << p->GetTimeStart() << " "
-		     << p->GetTimeEnd() << " " << p->GetVersion()   << endl;
-#elif !defined(FCPP_SHARED_PTRS)
+#if !defined(FCPP_SHARED_PTRS)
 	        const FrStatData* p = dynamic_cast<FrStatData*>(mStatData[i]);
 		cerr << p->GetName()    << " " << p->GetTimeStart() << " "
 		     << p->GetTimeEnd() << " " << p->GetVersion()   << endl;
@@ -250,11 +216,7 @@ DaccIn::getHistory(const std::string& name, Time& t, std::string& com) const {
 int
 DaccIn::open(void) {
     if (isOpen()) return 0;
-#if LDAS_VERSION_NUMBER < 109360
-    if (!mStream)
-#else
     if (!mFrameBuf)
-#endif
     {
 	if (mFile.empty()) {
 	    cout << "No more requested files" << endl;
@@ -301,11 +263,7 @@ DaccIn::endFrame(void) {
     //                           first buffer.
     if (isOnline()) {
         if (mTOCMode != s_TOC || mFramesInFile <= mFrameNo) {
-#if LDAS_VERSION_NUMBER < 109360
-	    dynamic_cast<iSMbuf*>(mSbuf)->relse();
-#else
 	    stream_ptr<iSMbuf>(mFrameBuf)->relse();
-#endif
 	    closeReader();
 	}
     }
@@ -326,11 +284,7 @@ DaccIn::nextFrame(void) {
 	    close();
 	} else {
 #ifndef DMTOFFLINE
-#if LDAS_VERSION_NUMBER < 109360
-	    dynamic_cast<iSMbuf*>(mSbuf)->relse();
-#else
 	    stream_ptr<iSMbuf>(mFrameBuf)->relse();
-#endif
 #endif
 	    closeReader();
 	}
@@ -350,11 +304,7 @@ bool
 DaccIn::waitData(bool nowait) const {
 #ifndef DMTOFFLINE
     if (!isOnline()) return true;
-#if LDAS_VERSION_NUMBER < 109360
-    return dynamic_cast<iSMbuf*>(mSbuf)->waitBuf(nowait);
-#else
     return stream_ptr<iSMbuf>(mFrameBuf)->waitBuf(nowait);
-#endif
 #else
     return true;
 #endif
@@ -559,23 +509,12 @@ DaccIn::findStat(const std::string& name, const Time& gps, int vsn) {
 int 
 DaccIn::closeFile(void) {
     if (mDebug > 3) cout << "Closing frame file ...";
-#if LDAS_VERSION_NUMBER < 109360
-    if (mStream) {
-        delete mStream;
-	mStream = 0;
-    }
-    if (mSbuf) {
-        delete mSbuf;
-        mSbuf = 0;
-	mFile.pop_front();
-    }
-#else
+
     if (mFrameBuf) {
         delete mFrameBuf;
 	mFrameBuf = 0;
 	mFile.pop_front();
     }
-#endif
 
     mSource = s_none;
     if (mDebug > 3) cout << " Done" << endl;
@@ -593,18 +532,6 @@ DaccIn::openFile(const std::string& Source) {
 #ifndef DMTOFFLINE
         if (mDebug > 3) cout << "Opening partition ..." << endl;
 
-#if LDAS_VERSION_NUMBER < 109360
-        iSMbuf* buf = new iSMbuf;
-        mSbuf = buf->open(part, ios::in);
-	if (!mSbuf) {
-	    cerr << "Unable to open partition " << part << endl;
-	    mFile.pop_front();
-	    return -1;
-	}
-
-	mStream = new std::istream(mSbuf);
-	if (!mStream) return -1;
-#else
 	mFrameBuf = new FrameBuffer< iSMbuf >(ios::in);
 	if (!mFrameBuf) {
 	    cerr << "Unable to construct Framebuffer<iSMbuf> for partition " 
@@ -619,7 +546,7 @@ DaccIn::openFile(const std::string& Source) {
 	    mFrameBuf = 0;
 	    return -1;
 	}
-#endif
+
 	if (mDebug) {
 	    cout << "Opened partition " << part << " for frame input." 
 		 << endl;
@@ -634,23 +561,10 @@ DaccIn::openFile(const std::string& Source) {
     //----------------------------------  Forget about offline for now
     } else {
         if (mDebug > 3) cout << "Opening file ..." << endl;
-#if LDAS_VERSION_NUMBER < 109360
-	filebuffer_type* fbuf = new filebuffer_type;
-	mSbuf = fbuf->open(Source.c_str(), ios::in);
-	if (!mSbuf) {
-	    cerr << "Unable to open file " << Source << endl;
-	    mFile.pop_front();
-	    return -1;
-	}
-
-	mStream = new std::istream(mSbuf);
-	if (!mStream) return -1;
-#else
 	FrameBuffer< filebuffer_type >* 
 	    fbuf = new FrameBuffer<filebuffer_type>(ios::in);
 	fbuf->open(Source.c_str(), ios::in);
 	mFrameBuf = fbuf;
-#endif
 	if (mDebug) {
 	    cout << "Opened file " << Source << " for frame input." << endl;
 	}
@@ -677,9 +591,6 @@ DaccIn::closeReader(void) {
         cerr << "Unidentified exception caught." << endl;
 	rc = -1;
     }
-#if LDAS_VERSION_NUMBER < 109360
-    mReader   = 0;
-#else
     mReader   = 0;
     //----------------------------------  delete source file name when the 
     //                                    frambuffer is closed
@@ -687,7 +598,6 @@ DaccIn::closeReader(void) {
 	mFile.pop_front();
 	mFrameBuf = 0;
     }
-#endif
     return rc;
 }
 
@@ -696,12 +606,8 @@ int
 DaccIn::openReader(void) {
     if (mDebug > 3) cout << "Opening stream reader ...";
     try {
-#if LDAS_VERSION_NUMBER < 109360
-        mReader = new IFrameStream(*mStream);
-#else
 	mReader = new IFrameStream(mFrameBuf);
 	if (isOnline()) mReader->SetAutoDeleteBuffer(false);
-#endif
     } catch (exception& e) {
         cerr << "Exception constructing IFrameStream: " << e.what()<< endl;
 	return -1;
@@ -751,15 +657,18 @@ DaccIn::deleteFrame(void) {
 
 int 
 DaccIn::readFrame(void) {
-    if (mDebug > 3) cout << "Reading next frame...";
-    if (haveFrame() || !isOpen()) return -1;
+    if (haveFrame() || !isOpen()) {
+	cerr << "readFrame: reader not open or frame already read." << endl;
+	return -1;
+    }
     try {
         frrawdata_pointer raw;
         switch (mTOCMode) {
 	case s_noTOC:
 	    //--------------------------  Read a frame. Versions of framecpp
 	    //                            sometimes throws an exception if
-	    //                            no more frames in reall mode.
+	    //                            no more frames in read-all mode.
+	    if (mDebug > 3) cout << "Reading next (full) frame...";
 	    try {
 		mFrame = mReader->ReadNextFrame();
 	    } 
@@ -786,44 +695,27 @@ DaccIn::readFrame(void) {
 	    break;
 	case s_TOC:
 	    if (mFrameNo && mFramesInFile <= mFrameNo) return -1;
+	    if (mDebug > 3) cout << "Reading next frame header + TOC...";
 	    unsigned int rflgs = FrameH::DETECT_SIM | FrameH::DETECT_PROC 
 	                       | FrameH::HISTORY;
 	    mFrame = mReader->ReadFrameH(mFrameNo, rflgs);
 	    if (!haveFrame()) {
 	        cerr << "Unable to read frame header!" << endl;
 #ifdef ISMBUF_TRACE
-#if LDAS_VERSION_NUMBER < 109360
-		if (mSbuf && isOnline()) {
-		    dynamic_cast<iSMbuf*>(mSbuf)->pTrace();
-		}
-#else
 		if (mFrameBuf && isOnline()) {
 		    stream_ptr<iSMbuf>(mFrameBuf)->pTrace();
 		}
-#endif
 #endif
 		return -1;
 	    }
 	    //raw = mReader->ReadFrRawData(mFrameNo);
-#if LDAS_VERSION_NUMBER < 109360
-	    raw = new FrRawData;
-#else
 	    raw = frrawdata_pointer(new FrRawData);
-#endif
 	    if (raw) {
-#if LDAS_VERSION_NUMBER < 109360
-	        mFrame->SetRawData(raw, false, true);
-#else
 	        mFrame->SetRawData(raw);
-#endif
 		mAdcIter = raw->RefFirstAdc().begin();
 	    }
 	    if (!mFrameNo) {
-#if LDAS_VERSION_NUMBER < 109360
-		mFramesInFile = mReader->GetNumberOfFrames();
-#else
 		mFramesInFile = mReader->GetTOC()->nFrame();
-#endif
 	    }
 	}
 	if (mDebug > 2) cout << "Frame read successfully. ID: " 
@@ -836,15 +728,9 @@ DaccIn::readFrame(void) {
 #endif
 	if (mDebug) {
 	    cerr << "Exception reading Frame: " << e.what() << endl;
-#if LDAS_VERSION_NUMBER < 109360
-	    cerr << "Error occurred at offset " 
-		 << mSbuf->pubseekoff(0, ios_base::cur, ios_base::in)
-		 << endl;
-#else
 	    cerr << "Error occurred at offset " 
 		 << stream_ptr<filebuffer_type>(mFrameBuf)->pubseekoff(0, ios_base::cur, ios_base::in)
 		 << endl;
-#endif
 	}
 	return -1;
     } catch (...) {
@@ -983,7 +869,7 @@ DaccIn::readSim(const string& name) const {
     return p;
 }
 
-#if !defined(DMTOFFLINE) && LDAS_VERSION_NUMBER >= 109360
+#if !defined(DMTOFFLINE)
 #include "framecpp/Common/IOStream.icc"
  template class FrameCPP::Common::FrameBuffer< iSMbuf >;
 #endif
