@@ -17,7 +17,7 @@ MultiDacc::MultiDacc(void)
 }
 
 //======================================  Construct and connect to source.
-MultiDacc::MultiDacc(const char* Source, const Time& STime)
+MultiDacc::MultiDacc(const std::string& Source, const Time& STime)
 {
     addMulti(Source);
 }
@@ -53,6 +53,21 @@ MultiDacc::addSingle(const string& name) {
     else                newDacc->addPathList(name);
     mIn.push_back(newDacc);
 }
+//======================================  Close all frame readers
+void 
+MultiDacc::channel_id(const string& chanid, int& daccid, string& chan) {
+    size_t inx = chanid.find_first_of('/');
+    if (inx == string::npos) {
+	chan = chanid;
+	daccid = getDaccIndex(chan);
+    } else if (chanid[0] >= '0' && chanid[0] <= '9') {
+	daccid = strtoul(chanid.c_str(), 0, 0);
+	chan = chanid.substr(inx+1);
+    } else {
+	daccid = frame_type(chanid.substr(0, inx));
+	chan = chanid.substr(inx+1);
+    }
+}
 
 //======================================  Close all frame readers
 void
@@ -140,6 +155,13 @@ MultiDacc::getCurrentTime(void) const {
     return mIn[0]->getCurrentTime();
 }
 
+//======================================  Get fill time (or Time(0))
+Interval
+MultiDacc::getFillStride(void) const {
+    if (mIn.empty()) return 0.0;
+    return mIn[0]->getFillStride();
+}
+
 //======================================  Get fill time (or Time(0))
 Time
 MultiDacc::getFillTime(void) const {
@@ -156,190 +178,183 @@ MultiDacc::getStride(void) const {
 
 //======================================  Add a channel to the request list.
 void 
-MultiDacc::addChannel(const char* Name, int id, int Decimate, TSeries **TSptr) {
-    chan_iter i=findChannel(Name);
+MultiDacc::addChannel(const string& Name, int id, int Decimate,
+		      TSeries **TSptr) {
+    int daccid;
+    string channame;
+    channel_id(Name, daccid, channame);
+
+    chan_iter i=findChannel(channame);
     if (i != mChanList.end()) {
 	if (getDebug()) cout << "Existing channel " << Name 
 			     << " entry replaced." << endl;
 	mChanList.erase(i);
     }
-    ChannelIndex newchan(Name, id);
-    mChanList.push_back(newchan);
-    int index=getDaccIndex(Name);
-    if(index!=-1) {
-	mIn[index]->addChannel(Name, Decimate, TSptr);
-    }
-    else {
-	if (getDebug()) cout << "Channel " << Name 
-			     << " does not exist in data streams" << endl;
-	chan_iter j=findChannel(Name);
-	if (j != mChanList.end()) {
-	    mChanList.erase(j);
-	}
+    
+    if (id < 0) id = daccid;
+    mChanList.push_back(ChannelIndex(channame, id));
+    if (id < 0) {
+	cerr << "MultiDacc: failed to add channel " << Name << endl;
+    } else {
+	mIn[id]->addChannel(channame, Decimate, TSptr);
     }
 }
 
 //======================================  Add a channel to the request list.
 void 
-MultiDacc::addFSeries(const char* Name, int id, FSeries **FSptr) {
-    chan_iter i=findChannel(Name);
+MultiDacc::addFSeries(const string& Name, int id, FSeries **FSptr) {
+    int daccid;
+    string channame;
+    channel_id(Name, daccid, channame);
+
+    chan_iter i=findChannel(channame);
     if (i != mChanList.end()) {
 	if (getDebug()) cout << "Existing channel " << Name 
 			     << " entry replaced." << endl;
 	mChanList.erase(i);
     }
-    ChannelIndex newchan(Name, id);
-    mChanList.push_back(newchan);
-    int index=getDaccIndex(Name);
-    if(index!=-1) {
-	mIn[index]->addFSeries(Name, FSptr);
-    }
-    else {
-	if (getDebug()) cout << "Channel " << Name 
-			     << " does not exist in data streams" << endl;
-	chan_iter j=findChannel(Name);
-	if (j != mChanList.end()) {
-	    mChanList.erase(j);
-	}
+    
+    if (id < 0) id = daccid;
+    mChanList.push_back(ChannelIndex(channame, id));
+    if (id != -1) {
+	mIn[id]->addFSeries(channame, FSptr);
     }
 }
 
 //======================================  Add a channel to the request list.
 void 
-MultiDacc::addRaw(const char* Name, int id, int Decimate, TSeries **TSptr) {
-    chan_iter i=findChannel(Name);
+MultiDacc::addRaw(const string& Name, int id, int Decimate, TSeries **TSptr) {
+    int daccid;
+    string channame;
+    channel_id(Name, daccid, channame);
+
+    chan_iter i=findChannel(channame);
     if (i != mChanList.end()) {
 	if (getDebug()) cout << "Existing channel " << Name 
 			     << " entry replaced." << endl;
 	mChanList.erase(i);
     }
-    ChannelIndex newchan(Name, id);
-    mChanList.push_back(newchan);
-    int index=getDaccIndex(Name);
-    if(index!=-1) {
-	mIn[index]->addRaw(Name, Decimate, TSptr);
-    }
-    else {
-	if (getDebug()) cout << "Channel " << Name 
-			     << " does not exist in data streams" << endl;
-	chan_iter j=findChannel(Name);
-	if (j != mChanList.end()) {
-	    mChanList.erase(j);
-	}
+    
+    if (id < 0) id = daccid;
+    mChanList.push_back(ChannelIndex(channame, id));
+    if (id != -1) {
+	mIn[id]->addRaw(channame, Decimate, TSptr);
     }
 }
 
 //======================================  Add a channel to the request list.
 void 
-MultiDacc::addProcessed(const char* Name, int id, int Decimate, TSeries **TSptr) {
-    chan_iter i=findChannel(Name);
+MultiDacc::addProcessed(const string& Name, int id, int Decimate,
+			TSeries **TSptr) {
+    int daccid;
+    string channame;
+    channel_id(Name, daccid, channame);
+
+    chan_iter i=findChannel(channame);
     if (i != mChanList.end()) {
 	if (getDebug()) cout << "Existing channel " << Name 
 			     << " entry replaced." << endl;
 	mChanList.erase(i);
     }
-    ChannelIndex newchan(Name, id);
-    mChanList.push_back(newchan);
-    int index=getDaccIndex(Name);
-    if(index!=-1) {
-	mIn[index]->addProcessed(Name, Decimate, TSptr);
-    }
-    else {
-	if (getDebug()) cout << "Channel " << Name 
-			     << " does not exist in data streams" << endl;
-	chan_iter j=findChannel(Name);
-	if (j != mChanList.end()) {
-	    mChanList.erase(j);
-	}
+    
+    if (id < 0) id = daccid;
+    mChanList.push_back(ChannelIndex(channame, id));
+    if (id != -1) {
+	mIn[id]->addProcessed(channame, Decimate, TSptr);
     }
 }
 
 //======================================  Add a channel to the request list.
 void 
-MultiDacc::addSimulated(const char* Name, int id, int Decimate, TSeries **TSptr) {
-    chan_iter i=findChannel(Name);
+MultiDacc::addSimulated(const string& Name, int id, int Decimate,
+			TSeries **TSptr) {
+    int daccid;
+    string channame;
+    channel_id(Name, daccid, channame);
+
+    chan_iter i=findChannel(channame);
     if (i != mChanList.end()) {
 	if (getDebug()) cout << "Existing channel " << Name 
 			     << " entry replaced." << endl;
 	mChanList.erase(i);
     }
-    ChannelIndex newchan(Name, id);
-    mChanList.push_back(newchan);
-    int index=getDaccIndex(Name);
-    if(index!=-1) {
-	mIn[index]->addSimulated(Name, Decimate, TSptr);
-    }
-    else {
-	if (getDebug()) cout << "Channel " << Name 
-			     << " does not exist in data streams" << endl;
-	chan_iter j=findChannel(Name);
-	if (j != mChanList.end()) {
-	    mChanList.erase(j);
-	}
+    
+    if (id < 0) id = daccid;
+    mChanList.push_back(ChannelIndex(channame, id));
+    if (id != -1) {
+	mIn[id]->addSimulated(channame, Decimate, TSptr);
     }
 }
 
 //======================================  See if channel was requested.
 bool 
-MultiDacc::isChannelRead(const char* Name) const {
+MultiDacc::isChannelRead(const string& Name) const {
     return findChannel(Name) != mChanList.end();
 }
 
 //======================================  List requested channels
 ostream&
 MultiDacc::list(ostream& out) const {
-    char buf[2048];
-    sprintf (buf, "Channel\n");
-    out << buf;
-    for (const_chan_iter i=mChanList.begin() ; i != mChanList.end() ; i++) {
-	sprintf (buf, "%-25s \n", i->getName());
-	out << buf;
+    for (size_t id=0; id<mIn.size(); id++) {
+	out << "Data stream " << id << ": frame_type: " << frame_name(id)
+	    << endl;
+	mIn[id]->list(out);
     }
     return out;
 }
 
+//======================================  Frame type of the specified stream
+std::string
+MultiDacc::frame_name(int id) const {
+    if (id < 0 || id >= int(mIn.size())) return "";
+    string framepath = mIn[id]->refList().first();
+    string::size_type inx = framepath.rfind("/");
+    if (inx != string::npos) framepath.erase(0, inx+1);
+    inx = framepath.find("-");
+    if (inx != string::npos) {
+	string::size_type inx2 = framepath.find("-", inx+1);
+	if (inx2 != string::npos) inx = inx2;
+    }
+    if (inx != string::npos) framepath.erase(inx);
+    return framepath;
+}
+
 //======================================  Stream with the specified frame type
 int
 MultiDacc::frame_type(const std::string& frameType) const {
 
    //--------------------------------  Find an input stream ID.
-   int id = -1;
    size_t N = mIn.size();
    for (size_t i=0; i < N; ++i) {
-      string framepath = mIn[i]->refList().first();
-      string::size_type inx = framepath.rfind("/");
-      if (inx != string::npos) framepath.erase(0, inx+1);
-      inx = framepath.find("-");
-      if (inx != string::npos) {
-	 string::size_type inx2 = framepath.find("-", inx);
-	 if (inx2 != string::npos) inx = inx2;
-      }
-      if (inx != string::npos) framepath.erase(inx);
-      if (framepath == frameType) id = i;
+       if (frame_name(i) == frameType) return i;
    }
-   return id;
+   return -1;
 }
 
 //======================================  Find channel in request list.
 MultiDacc::const_chan_iter 
-MultiDacc::findChannel(const char* Name) const {
+MultiDacc::findChannel(const std::string& name) const {
+    size_t inx = name.find_first_of('/');
+    string raw = (inx == string::npos) ? name : name.substr(inx+1);
     for (const_chan_iter i=mChanList.begin() ; i != mChanList.end() ; i++) {
-	if (i->EqName(Name)) return i;
+	if (i->EqName(raw)) return i;
     }
     return mChanList.end();
 }
 
 MultiDacc::chan_iter 
-MultiDacc::findChannel(const char* Name) {
+MultiDacc::findChannel(const string& name) {
+    size_t inx = name.find_first_of('/');
+    string raw = (inx == string::npos) ? name : name.substr(inx+1);
     for (chan_iter i=mChanList.begin() ; i != mChanList.end() ; i++) {
-	if (i->EqName(Name)) return i;
+	if (i->EqName(raw)) return i;
     }
     return mChanList.end();
 }
 
 //======================================  Remove channel from request list.
 void 
-MultiDacc::rmChannel(const char* Name) {
+MultiDacc::rmChannel(const string& Name) {
     int index=getDaccIndex(Name);
     if (index != -1) {
 	mIn[index]->rmChannel(Name);
@@ -399,10 +414,12 @@ MultiDacc::synch(void) {
 //     -8  Interrupter by signal.
 int 
 MultiDacc::fillData(Interval Stride, bool start) {
-    int rc = 0;
+    int rc = synch();
+    if (rc) return rc;
     size_type N = mIn.size();
     for (size_type i=0; i<N; i++) {
-	rc = mIn[i]->fillData(Stride,start);
+	int rci = mIn[i]->fillData(Stride, start);
+	if (rci != 0) rc = rci;
     }
     return rc;
 }
@@ -428,46 +445,13 @@ MultiDacc::fillChans(Interval Offset, Interval dT) {
 }
 
 int
-MultiDacc::getDaccIndex(const char* name) {
+MultiDacc::getDaccIndex(const string& name) {
     chan_iter i=findChannel(name); 
     if (i != mChanList.end()) {
 	int index=i->getIndex();
-	if(index==-1) {
-	    // find a way to detect where is the channel automatically
-	    // not so easy, may do it later
-	    /*
-	      int j;
-	      for(j=0;j<(int)mIn.size();j++) {
-	      cout<<"entering loop"<<endl;
-	      const FrameCPP::FrAdcData* abc;
-	      cout<<"before find"<<endl;
-	      abc=mIn[j]->nextAdc(name);
-	      cout<<"after find"<<endl;
-	      if(abc) {
-	      cout<<"before set"<<endl;
-	      i->setIndex(j);
-	      cout<<"after set"<<endl;
-	      return j;
-	      }
-	      const FrameCPP::FrProcData* proc;
-	      proc = mIn[j]->nextProc(name);
-	      if(proc) {
-	      i->setIndex(j);
-	      return j;
-	      }
-	      else {
-	      const FrameCPP::FrSimData* sim;
-	      sim = mIn[j]->nextSim(name);
-	      if(sim) {
-	      i->setIndex(j);
-	      return j;
-	      }
-	      }
-	      }
-	      if(j==(int)mIn.size())
-	      return -1;
-	    */
-	}
+	if(index==-1)
+	    //--------------------------  Take the easy way out for now
+	    return 0;
 	else
 	    return index;
     }
@@ -496,26 +480,23 @@ MultiDacc::seek(Time STime) {
     return rc;
 }
 
-
 //======================================  Get a reference to the named Channel
 TSeries* 
-MultiDacc::refData(const char *name) {
-    chan_iter i=findChannel(name);
-    if (i != mChanList.end()) {
-	int index=getDaccIndex(name);
-	if(index!=-1)
-	    return mIn[index]->refData(name);
+MultiDacc::refData(const string& name) {
+    chan_iter chaninx = findChannel(name);
+    if (chaninx != mChanList.end()) {
+	int id = chaninx->getIndex();
+	if (id != -1) return mIn[id]->refData(chaninx->getName());
     }
     return (TSeries*) 0;
 }
 
 FSeries* 
-MultiDacc::refFData(const char *name) {
-    chan_iter i=findChannel(name);
-    if (i != mChanList.end()) {
-	int index=getDaccIndex(name);
-	if(index!=-1)
-	    return mIn[index]->refFData(name);
+MultiDacc::refFData(const string& name) {
+    chan_iter chaninx = findChannel(name);
+    if (chaninx != mChanList.end()) {
+	int id = chaninx->getIndex();
+	if (id != -1) return mIn[id]->refFData(chaninx->getName());
     }
     return (FSeries*) 0;
 }
