@@ -12,23 +12,12 @@
 #include "framecpp/FrHistory.hh"
 #include "framecpp/FrStatData.hh"
 
-#if LDAS_VERSION_NUMBER < 109360
-#include "framecpp/util.hh"
-#else
 #include "framecpp/FrameCPP.hh"
-#if LDAS_VERSION_NUMBER > 119013
 #include "framecpp/Common/FrameBuffer.hh"
-#endif
-#endif
 #include "framecpp/OFrameStream.hh"
 
-#if LDAS_VERSION_NUMBER < 200000
-#include "general/gpstime.hh"
-using General::GPSTime;
-#else
 #include "framecpp/GPSTime.hh"
 using FrameCPP::GPSTime;
-#endif
 
 #define DACC_ONLDEV "/online/"
 #ifndef DMTOFFLINE
@@ -64,13 +53,11 @@ CPPTime(const Interval& t)
 }
 
 //======================================  Stream buffer cast
-#if LDAS_VERSION_NUMBER >= 109360
 template <class SB>
 inline SB&
 stream_ref(FrameBufferInterface& fbi) {
     return dynamic_cast<FrameBuffer<SB>&>(fbi);
 }
-#endif
 
 //======================================  Constructors and destructors.
 FrWriter::FrWriter(void) 
@@ -78,10 +65,7 @@ FrWriter::FrWriter(void)
     mCompress(FrVectRef::kUncompressed), mOnline(false), mChkSumStruct(1),
     mChkSumFile(1)
 {
-#if LDAS_VERSION_NUMBER < 109360
-    mFrame  = 0;
-    mStream = 0;
-#elif !defined(FCPP_SHARED_PTRS)
+#if !defined(FCPP_SHARED_PTRS)
     mFrame  = 0;
     mFrameBuf = 0;
 #else
@@ -95,10 +79,7 @@ FrWriter::FrWriter(const char* Name, int Run)
     mCompress(FrVectRef::kUncompressed), mOnline(false), mChkSumStruct(1),
     mChkSumFile(1)
 {
-#if LDAS_VERSION_NUMBER < 109360
-    mFrame  = 0;
-    mStream = 0;
-#elif !defined(FCPP_SHARED_PTRS)
+#if !defined(FCPP_SHARED_PTRS)
     mFrame  = 0;
     mFrameBuf = 0;
 #else
@@ -111,15 +92,11 @@ FrWriter::FrWriter(const char* Name, int Run)
 FrWriter::~FrWriter() {
     mChanList.erase(mChanList.begin(), mChanList.end());
     if (mWriter) close();
-#if LDAS_VERSION_NUMBER < 109360
-    if (mStream) delete mStream;
-#else
     if (mFrameBuf) {
 	if (mFrameBuf) stream_ref<std::filebuf>(*mFrameBuf).close();
 	delete mFrameBuf;
 	mFrameBuf = 0;
     }
-#endif
     erase();
 }
 
@@ -164,12 +141,7 @@ FrWriter::open(const char* File, bool toc) {
     mFile = File;
     if (mDebug) cout << "Opening file: " << File << endl;
     FrameCPP::OFrameStream* w;
-#if LDAS_VERSION_NUMBER < 109360
-    mStream = new ofstream(File, ios::out);
-    if (!mStream->good()) return -1;
-    w = new FrameCPP::OFrameStream(*mStream);
-#else
-    auto_ptr <FrameBufferInterface > fb;
+    unique_ptr <FrameBufferInterface > fb;
     string::size_type l = strlen(DACC_ONLDEV);
     if (mFile.substr(0, l) == DACC_ONLDEV) {
 #ifdef DMTOFFLINE
@@ -189,7 +161,6 @@ FrWriter::open(const char* File, bool toc) {
     mFrameBuf = fb.release();
     w = new FrameCPP::OFrameStream(mFrameBuf);
     w->SetCheckSumFile(cvt_cksum_type(mChkSumFile));
-#endif
     return open(w);
 }
  
@@ -202,17 +173,6 @@ FrWriter::open(FrameCPP::OFrameStream* writer) {
 //======================================  Close the frame writer.
 void
 FrWriter::close(void) {
-#if LDAS_VERSION_NUMBER < 109360
-    if (mWriter) {
-        delete mWriter;
-	mWriter = 0;
-    }
-    if (mStream) {
-	((ofstream*)mStream)->close();
-        delete mStream;
-	mStream = 0;
-    }
-#else
     if (mWriter) {
 	mWriter->Close();
 	if (mFrameBuf) {
@@ -223,7 +183,6 @@ FrWriter::close(void) {
 	mWriter = 0;
 	mFrameBuf = 0;
     }
-#endif
 }
 
 //======================================  Erase the frame.
@@ -357,7 +316,7 @@ FrWriter::addProcData(const std::string& Name, const std::string& comment,
 
 //======================================  Add a raw data series
 void
-FrWriter::addRawSeries(const char* Name, const TSeries& TS) {
+FrWriter::addRawSeries(const std::string& Name, const TSeries& TS) {
     if (TS.isEmpty()) return;
 
     //----------------------------------  Make sure start time is valid.
@@ -397,7 +356,7 @@ FrWriter::addRawSeries(const char* Name, const TSeries& TS) {
 
 //======================================  Add a processed data channel
 void
-FrWriter::addProcSeries(const char* Name, const TSeries& TS) {
+FrWriter::addProcSeries(const std::string& Name, const TSeries& TS) {
 
     //----------------------------------  Build a vector
     Interval Offset = TS.getStartTime() - mTime;
@@ -424,13 +383,8 @@ FrWriter::addSimEvent(const char* Name, const char* Comment,
 		      Interval before, Interval after, float A, 
 		      const std::vector<std::pair<std::string, float> >& v)
 {
-#if LDAS_VERSION_NUMBER < 109360
-    FrameCPP::FrSimEvent sim(Name, Inputs, CPPTime(GPSMax), before, after, A);
-    sim.AppendComment(Comment);
-#else
     FrameCPP::FrSimEvent sim(Name, Comment, Inputs, CPPTime(GPSMax), before, 
 			     after, A, FrameCPP::FrSimEvent::ParamList_type());
-#endif
     int N = v.size();
     sim.GetParam().resize(N);
     for (int i=0 ; i<N ; i++)
@@ -590,7 +544,7 @@ FrWriter::writeFrame(void) {
     mWriter->WriteFrame(mFrame, cvt_cksum_type(mChkSumStruct));
 #endif
 
-#if !defined(DMTOFFLINE) && LDAS_VERSION_NUMBER >= 109360
+#if !defined(DMTOFFLINE)
     if (mOnline) stream_ref<oSMbuf>(*mFrameBuf).set_id(mTime.getS());
 #endif
     erase();
@@ -688,7 +642,7 @@ FrWriter::setName(const std::string& name) {
     mName = name;
 }
 
-#if !defined(DMTOFFLINE) && LDAS_VERSION_NUMBER >= 109360
+#if !defined(DMTOFFLINE)
 #include "framecpp/Common/IOStream.icc"
  template class FrameCPP::Common::FrameBuffer< oSMbuf >;
 #endif
