@@ -1,10 +1,6 @@
 /* -*- mode: c++; c-basic-offset: 3; -*- */
 #include "PConfig.h"
-#ifdef __GNU_STDC_OLD
-#include <gnusstream.h>
-#else
 #include <sstream>
-#endif
 #include <cstring>
 #include <cstdio>
 #include <iostream>
