@@ -10,11 +10,7 @@
 #include <unistd.h>
 #include <dirent.h>
 #include <iostream>
-#ifndef __GNU_STDC_OLD
 #include <sstream>
-#else
-#include "gnusstream.h"
-#endif
 
 namespace web {
    using namespace std;
