@@ -469,10 +469,10 @@ lineData LineFilter::getHeteroLine(WaveData &TD)
 {
    long i,k,m;
    double a,b,u,v;
-   register double C,S, c,s, ch,sh, x;
+   double C,S, c,s, ch,sh, x;
    double omega;
    long N = TD.size();
-   register double* p = TD.data; 
+   double* p = TD.data; 
    size_t mode;
    lineData Q;
 
