@@ -163,7 +163,7 @@ Table::readtxt(const string& in) {
     } else {
 	inf = new ifstream(in.c_str());
     }
-    if (!inf || !*inf) {
+    if (!inf || inf->bad()) {
 	throw runtime_error(string("Unable to open file: ")+in);
     }
 
